@@ -5,13 +5,13 @@
  *
  *      Description:  Type definitions for benchmarking framework
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -41,6 +41,7 @@ typedef enum {
     INT} DataType;
 
 typedef enum {
+    STREAM_0 = 0,
     STREAM_1 = 1,
     STREAM_2,
     STREAM_3,
@@ -96,6 +97,8 @@ typedef struct {
     int instr_const;
     int instr_loop;
     int uops;
+    int loadstores;
+    void* dlhandle;
 } TestCase;
 
 typedef struct {
@@ -105,6 +108,7 @@ typedef struct {
     const TestCase* test;
     uint64_t   cycles;
     uint32_t numberOfThreads;
+    int    init_per_thread;
     int* processors;
     void** streams;
 } ThreadUserData;
