@@ -5,13 +5,13 @@
  *
  *      Description:  A flexible and extensible benchmarking toolbox
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -442,7 +442,7 @@ int main(int argc, char** argv)
         for (i=0; i < affinity->numberOfAffinityDomains; i++ )
         {
             ownprintf("Domain %d:\n",i);
-            ownprintf("\tTag %s:",bdata(affinity->domains[i].tag));
+            ownprintf("\tTag %s:",affinity->domains[i].tag);
 
             for ( uint32_t j=0; j < affinity->domains[i].numberOfProcessors; j++ )
             {
@@ -491,10 +491,10 @@ int main(int argc, char** argv)
                         if ((int)(floor(orig_size/currentWorkgroup->numberOfThreads)) % test->stride)
                         {
                             int typesize = allocator_dataTypeLength(test->type);
-                            newsize = (((int)(floor(orig_size/nrThreads))/stride)*(stride))*nrThreads;
+                            newsize = (((size_t)(floor(orig_size/nrThreads))/stride)*(stride))*nrThreads;
                             if (newsize > 0 && warn_once)
                             {
-                                fprintf (stdout, "Warning: Sanitizing vector length to a multiple of the loop stride %d and thread count %d from %d elements (%d bytes) to %d elements (%d bytes)\n",stride, nrThreads, orig_size, orig_size*typesize, newsize, newsize*typesize);
+                                fprintf (stdout, "Warning: Sanitizing vector length to a multiple of the loop stride %d and thread count %d from %ld elements (%ld bytes) to %ld elements (%ld bytes)\n",stride, nrThreads, orig_size, orig_size*typesize, newsize, newsize*typesize);
                                 warn_once = 0;
                             }
                             else if (newsize == 0)
