@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of allocator module.
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -112,7 +112,7 @@ allocator_allocateVector(
 
     for (i=0;i<domains->numberOfAffinityDomains;i++)
     {
-        if (biseq(domainString, domains->domains[i].tag))
+        if (strncmp(domains->domains[i].tag, bdata(domainString), strlen(domains->domains[i].tag)) == 0)
         {
             domain = domains->domains + i;
         }
@@ -157,7 +157,7 @@ allocator_allocateVector(
     affinity_pinProcess(domain->processorList[0]);
     printf("Allocate: Process running on hwthread %d (Domain %s) - Vector length %llu/%llu Offset %d Alignment %llu\n",
             affinity_processGetProcessorId(),
-            bdata(domain->tag),
+            domain->tag,
             LLU_CAST size,
             LLU_CAST bytesize,
             offset,
