@@ -5,13 +5,13 @@
  *
  *      Description:  Utility string routines building upon bstrlib
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com.
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -142,7 +142,7 @@ parse_workgroup(Workgroup* group, const_bstring str, DataType type)
 
         for (int i = 0; i < doms->numberOfAffinityDomains; i++)
         {
-            if (bstrcmp(doms->domains[i].tag, tokens->entry[0]) == BSTR_OK)
+            if (strncmp(doms->domains[i].tag, bdata(tokens->entry[0]), strlen(doms->domains[i].tag)) == 0)
             {
                 domidx = i;
                 break;
@@ -154,7 +154,7 @@ parse_workgroup(Workgroup* group, const_bstring str, DataType type)
         }
         else
         {
-            fprintf(stderr, "Unknown affinity domain %s\n", bdata(tokens->entry[2]));
+            fprintf(stderr, "Unknown affinity domain %s\n", bdata(tokens->entry[0]));
             bstrListDestroy(tokens);
             return NULL;
         }
