@@ -5,13 +5,13 @@
  *
  *      Description:  Example how to use the LIKWID API in C/C++ applications
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:  Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:  Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -43,6 +43,8 @@ int main(int argc, char* argv[])
     int gid;
     double result = 0.0;
     char estr[] = "L2_LINES_IN_ALL:PMC0,L2_TRANS_L2_WB:PMC1";
+    char* enames[2] = {"L2_LINES_IN_ALL:PMC0","L2_TRANS_L2_WB:PMC1"};
+    int n = sizeof(enames) / sizeof(enames[0]);
     //perfmon_setVerbosity(3);
     // Load the topology module and print some values.
     err = topology_init();
@@ -111,28 +113,61 @@ int main(int argc, char* argv[])
         topology_finalize();
         return 1;
     }
-    // Perform something
-    sleep(10);
-    // Stop all counters in the previously started event set.
-    err = perfmon_stopCounters();
+
+
+    // Perform some work.
+    sleep(2);
+
+    // Read and record current event counts.
+    err = perfmon_readCounters();
     if (err < 0)
     {
-        printf("Failed to stop counters for group %d for thread %d\n",gid, (-1*err)-1);
+        printf("Failed to read counters for group %d for thread %d\n",gid, (-1*err)-1);
         perfmon_finalize();
         topology_finalize();
         return 1;
     }
-    err = perfmon_startCounters();
+
+    // Print the result of every thread/CPU for all events in estr, counting from last read/startCounters().
+    printf("Work task 1/2 measurements:\n");
+    for (j=0; j<n; j++)
+    {
+        for (i = 0;i < topo->numHWThreads; i++)
+        {
+            result = perfmon_getLastResult(gid, j, i);
+            printf("- event set %s at CPU %d: %f\n", enames[j], cpus[i], result);
+        }
+    }
+
+
+    // Perform another piece of work
+    sleep(2);
+
+    // Read and record current event counts.
+    err = perfmon_readCounters();
     if (err < 0)
     {
-        printf("Failed to start counters for group %d for thread %d\n",gid, (-1*err)-1);
+        printf("Failed to read counters for group %d for thread %d\n",gid, (-1*err)-1);
         perfmon_finalize();
         topology_finalize();
         return 1;
     }
-    // Perform something
-    sleep(10);
-    // Stop all counters in the previously started event set.
+
+    // Print the result of every thread/CPU for all events in estr, counting between the 
+    // previous two calls of perfmon_readCounters().
+    printf("Work task 2/2 measurements:\n");
+    for (j=0; j<n; j++)
+    {
+        for (i = 0;i < topo->numHWThreads; i++)
+        {
+            result = perfmon_getLastResult(gid, j, i);
+            printf("- event set %s at CPU %d: %f\n", enames[j], cpus[i], result);
+        }
+    }
+
+
+
+    // Stop all counters in the currently-active event set.
     err = perfmon_stopCounters();
     if (err < 0)
     {
@@ -142,19 +177,15 @@ int main(int argc, char* argv[])
         return 1;
     }
 
-
-    // Print the result of every thread/CPU for all events in estr.
-    char* ptr = strtok(estr,",");
-    j = 0;
-    while (ptr != NULL)
+    // Print the result of every thread/CPU for all events in estr, counting since counters first started.
+    printf("Total sum measurements:\n");
+    for (j=0; j<n; j++)
     {
         for (i = 0;i < topo->numHWThreads; i++)
         {
             result = perfmon_getResult(gid, j, i);
-            printf("Measurement result for event set %s at CPU %d: %f\n", ptr, cpus[i], result);
+            printf("- event set %s at CPU %d: %f\n", enames[j], cpus[i], result);
         }
-        ptr = strtok(NULL,",");
-        j++;
     }
 
 
