@@ -5,13 +5,14 @@
  *
  *      Description:  Example how to use the C/C++ Marker API
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:  Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Authors:  Thomas Gruber (tr), thomas.roehl@googlemail.com
+ *                Riley Weber, rileyw13@protonmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,17 +34,24 @@
 #include <unistd.h>
 
 #include <omp.h>
-#include <likwid.h>
+#include <likwid-marker.h>
+
+#ifdef LIKWID_PERFMON
+#define MAX_NUM_EVENTS 10
+#else
+#define MAX_NUM_EVENTS 0
+#endif
 
 #define SLEEPTIME 2
 
 int main(int argc, char* argv[])
 {
     int i, g;
-    int nevents = 10;
-    double events[10];
+    int nevents = MAX_NUM_EVENTS;
+    double events[MAX_NUM_EVENTS];
     double time;
     int count;
+
     // Init Marker API in serial region once in the beginning
     LIKWID_MARKER_INIT;
     #pragma omp parallel
@@ -51,39 +59,58 @@ int main(int argc, char* argv[])
         // Each thread must add itself to the Marker API, therefore must be
         // in parallel region
         LIKWID_MARKER_THREADINIT;
-        // Optional. Register region name
+
+        // Register region name. Optional, but highly recommended. Reduces the
+        // overhead of LIKWID_MARKER_START. Furthermore, if regions are not
+        // registered but the access daemon is used, short regions will report
+        // lower values for the first region.
         LIKWID_MARKER_REGISTER("example");
     }
 
-    // perfmon_getNumberOfGroups is not part of the MarkerAPI,
-    // it belongs to the normal LIKWID API. But the MarkerAPI
-    // has no function to get the number of configured groups.
-    for (g=0;g < perfmon_getNumberOfGroups(); g++)
+    // if the number of iterations is not greater than the number of groups you
+    // are measuring, groups after the final iteration will not be measured. In
+    // other words, if the loop specifies n iterations, only the first n groups
+    // will be measured. Furthermore, if the number of iterations is greater
+    // than the number of groups, some groups will be meausured multiple times
+    // in a round-robin fashion.
+    for (g=0; g<10; g++)
     {
         #pragma omp parallel
         {
             printf("Thread %d sleeps now for %d seconds\n", omp_get_thread_num(), SLEEPTIME);
             // Start measurements inside a parallel region
             LIKWID_MARKER_START("example");
+
             // Insert your code here.
             // Often contains an OpenMP for pragma. Regions can be nested.
             sleep(SLEEPTIME);
+
             // Stop measurements inside a parallel region
             LIKWID_MARKER_STOP("example");
             printf("Thread %d wakes up again\n", omp_get_thread_num());
+
             // If you need the performance data inside your application, use
+            // LIKWID_MARKER_GET. events is an array of doubles with
+            // nevents entries, time is a double* and count an int*.
             LIKWID_MARKER_GET("example", &nevents, events, &time, &count);
-            // where events is an array of doubles with nevents entries,
-            // time is a double* and count an int*.
-            printf("Region example measures %d events, total measurement time is %f\n", nevents, time);
-            printf("The region was called %d times\n", count);
+
+            // this check ensures that nothing will be printed if
+            // -DLIKWID_PERFMON is not included
+            if(nevents > 0){
+                printf("Region example measures %d events, total measurement time is %f\n", nevents, time);
+                printf("The region was called %d times\n", count);
+            }
             for (i = 0; i < nevents; i++)
             {
                 printf("Event %d: %f\n", i, events[i]);
             }
-            // If multiple groups given, you can switch to the next group
-            LIKWID_MARKER_SWITCH;
         }
+
+        // If multiple groups are given, you can switch to the next group. This
+        // function has no effect if one group is specified. Notice that this
+        // is called outside the parallel region, as it should only be run by a
+        // single thread
+        LIKWID_MARKER_SWITCH;
     }
 
     // Close Marker API and write results to file for further evaluation done
