@@ -1,3 +1,33 @@
+/*
+ * =======================================================================================
+ *
+ *      Filename:  monitoring.c
+ *
+ *      Description:  Example how to use the LIKWID API in C/C++ applications for monitoring
+ *
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
+ *
+ *      Author:  Thomas Gruber (tr), thomas.roehl@googlemail.com
+ *      Project:  likwid
+ *
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
+ *
+ *      This program is free software: you can redistribute it and/or modify it under
+ *      the terms of the GNU General Public License as published by the Free Software
+ *      Foundation, either version 3 of the License, or (at your option) any later
+ *      version.
+ *
+ *      This program is distributed in the hope that it will be useful, but WITHOUT ANY
+ *      WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+ *      PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+ *
+ *      You should have received a copy of the GNU General Public License along with
+ *      this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * =======================================================================================
+ */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -25,7 +55,6 @@ int main (int argc, char* argv[])
     numa_init();
     affinity_init();
     timer_init();
-    HPMmode(0);
     CpuInfo_t cpuinfo = get_cpuInfo();
     CpuTopology_t cputopo = get_cpuTopology();
     int numCPUs = cputopo->activeHWThreads;
@@ -64,13 +93,13 @@ int main (int argc, char* argv[])
         err = 1;
         goto monitor_exit;
     }
-    /*int gid3 = perfmon_addEventSet("ENERGY");
+    int gid3 = perfmon_addEventSet("ENERGY");
     if (gid3 < 0)
     {
         printf("Failed to add performance group ENERGY\n");
         err = 1;
         goto monitor_exit;
-    }*/
+    }
     signal(SIGINT, INThandler);
 
     while (run)
@@ -97,7 +126,7 @@ int main (int argc, char* argv[])
                 printf("%s,cpu=%d %f\n", perfmon_getMetricName(gid2, i), cpus[c], perfmon_getLastMetric(gid2, i, c));
             }
         }
-        /*perfmon_setupCounters(gid3);
+        perfmon_setupCounters(gid3);
         perfmon_startCounters();
         sleep(sleeptime);
         perfmon_stopCounters();
@@ -107,7 +136,7 @@ int main (int argc, char* argv[])
             {
                 printf("%s,cpu=%d %f\n", perfmon_getMetricName(gid3, i), cpus[c], perfmon_getLastMetric(gid3, i, c));
             }
-        }*/
+        }
     }
 monitor_exit:
     free(cpus);
