@@ -12,13 +12,13 @@ my $lc = ' *';
 
 #my $VERSION   = '<VERSION>';
 #my $DATE   = '<DATE>';
-my $VERSION   = '5.2.1';
-my $DATE   = '03.12.2021';
-my $YEAR  = '2021';
-my $AUTHOR = 'NHR@FAU, University Erlangen-Nuremberg';
+my $VERSION   = '5.4.0';
+my $DATE   = '15.11.2024';
+my $YEAR  = '2024';
+my $AUTHOR = 'RRZE, University Erlangen-Nuremberg';
 my $LICENSE = 'gpl';
 
-my @SKIPLIST = ('ghash.c','ghash.h','loadData.S','loadDataARM.S','bstrlib.c','bstrlib.h', 'calculator_stack.h', 'calculator_stack.c');
+my @SKIPLIST = ('ghash.c','ghash.h','loadData.S','loadDataARM.S','bstrlib.c','bstrlib.h', 'calculator_stack.h', 'calculator_stack.c', 'voltage.c', 'voltage.h', 'likwid-benchmark.sh', 'calculator.c', 'calculator.h');
 
 sub print_copyright
 {
