@@ -5,13 +5,13 @@
  *
  *      Description:  Interface to the access daemon for the access module.
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -72,6 +72,7 @@ static pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t *cpuLocks = NULL;
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
+void __attribute__((destructor (104))) close_access_client(void);
 
 static char*
 access_client_strerror(AccessErrorType det)
@@ -106,23 +107,31 @@ access_client_errno(AccessErrorType det)
     }
 }
 
+static void access_client_signal_catcher(int sig) {
+    close_access_client();
+}
+
 static int
-access_client_startDaemon(int cpu_id)
+access_client_catch_signal()
+{
+    struct sigaction sia;
+    sia.sa_handler = access_client_signal_catcher;
+    sigemptyset(&sia.sa_mask);
+    sia.sa_flags = SA_ONSTACK;
+    sigaction(SIGCHLD, &sia, NULL);
+    return 0;
+}
+
+static int
+access_client_startDaemon_direct(int cpu_id, struct sockaddr_un *address)
 {
     /* Check the function of the daemon here */
-    int res = 0;
-    char* filepath;
     char *newargv[] = { NULL };
     char *newenv[] = { NULL };
     char *safeexeprog = TOSTRING(ACCESSDAEMON);
     char exeprog[1024];
-    struct sockaddr_un address;
-    size_t address_length;
     int  ret;
     pid_t pid;
-    int timeout = 1000;
-    int socket_fd = -1;
-    int print_once = 0;
 
     if (config.daemonPath != NULL)
     {
@@ -136,7 +145,7 @@ access_client_startDaemon(int cpu_id)
     if (access(exeprog, X_OK))
     {
         ERROR_PRINT(Failed to find the daemon '%s'\n, exeprog);
-        exit(EXIT_FAILURE);
+        return -1;
     }
     DEBUG_PRINT(DEBUGLEV_INFO, Starting daemon %s, exeprog);
     pid = fork();
@@ -158,7 +167,7 @@ access_client_startDaemon(int cpu_id)
         {
             //ERRNO_PRINT;
             ERROR_PRINT(Failed to execute the daemon '%s'\n, exeprog);
-            exit(EXIT_FAILURE);
+            return ret;
         }
     }
     else if (pid < 0)
@@ -167,18 +176,141 @@ access_client_startDaemon(int cpu_id)
         return pid;
     }
 
-    EXIT_IF_ERROR(socket_fd = socket(AF_LOCAL, SOCK_STREAM, 0), socket() failed);
+    address->sun_family = AF_LOCAL;
+    snprintf(address->sun_path, sizeof(address->sun_path), TOSTRING(LIKWIDSOCKETBASE) "-%d", pid);
 
-    address.sun_family = AF_LOCAL;
-    address_length = sizeof(address);
-    snprintf(address.sun_path, sizeof(address.sun_path), TOSTRING(LIKWIDSOCKETBASE) "-%d", pid);
-    filepath = strdup(address.sun_path);
+    daemon_pids[cpu_id] = pid;
+    nr_daemons++;
+    return 0;
+}
 
-    res = connect(socket_fd, (struct sockaddr *) &address, address_length);
+static int
+access_client_startDaemon_bridge(int cpu_id, const char *bridge_path, struct sockaddr_un *daemon_address) {
+    struct sockaddr_un bridge_address;
+    int socket_fd = -1;
+    int address_length;
+    char *filepath;
+    int timeout = 1000;
+    int res;
+    int io_buf;
+    long io_count;
+    int daemon_pid;
+
+    bridge_address.sun_family = AF_LOCAL;
+    snprintf(bridge_address.sun_path, sizeof(bridge_address.sun_path), "%s", bridge_path);
+
+    socket_fd = socket(AF_LOCAL, SOCK_STREAM, 0);
+    if (socket_fd < 0)
+    {
+        ERROR_PRINT(socket() failed);
+        return -1;
+    }
+
+    address_length = sizeof(struct sockaddr_un);
+    filepath = strdup(bridge_address.sun_path);
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Waiting for bridge socket file %s, bridge_address.sun_path);
+    while (access(bridge_address.sun_path, F_OK) && timeout > 0)
+    {
+        usleep(2500);
+        timeout--;
+    }
+    if (!access(bridge_address.sun_path, F_OK))
+    {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Bridge socket file %s exists, bridge_address.sun_path);
+    }
+    timeout = 1000;
+
+    res = connect(socket_fd, (struct sockaddr *) &bridge_address, address_length);
     while (res && timeout > 0)
     {
         usleep(2500);
-        res = connect(socket_fd, (struct sockaddr *) &address, address_length);
+        res = connect(socket_fd, (struct sockaddr *) &bridge_address, address_length);
+
+        if (res == 0)
+        {
+            break;
+        }
+
+        timeout--;
+        DEBUG_PRINT(DEBUGLEV_INFO, Still waiting for bridge socket %s for CPU %d..., filepath, cpu_id);
+    }
+
+    if (timeout <= 0)
+    {
+        ERRNO_PRINT;  /* should hopefully still work, as we make no syscalls in between. */
+        fprintf(stderr, "Exiting due to timeout: The bridge socket file at '%s' could not be\n", filepath);
+        fprintf(stderr, "opened within 10 seconds. Consult the error message above\n");
+        fprintf(stderr, "this to find out why. If the error is 'no such file or directoy',\n");
+        fprintf(stderr, "it usually means that the bridge socket filesystem isn't shared between\n");
+        fprintf(stderr, "the bridge and the client processes.\n");
+        free(filepath);
+        close(socket_fd);
+        return -1;
+    }
+    DEBUG_PRINT(DEBUGLEV_INFO, Successfully opened bridge socket %s to daemon for CPU %d, filepath, cpu_id);
+    free(filepath);
+
+    // request socket creation via the connected bridge
+    io_buf = 1;
+    io_count = send(socket_fd, (char*) &io_buf, sizeof(io_buf), 0);
+
+    if (io_count != sizeof(io_buf)) {
+        ERROR_PRINT(Failed to send msg to the bridge socket)
+        close(socket_fd);
+        return -1;
+    }
+
+    io_count = recv(socket_fd, (char*) &io_buf, sizeof(io_buf), 0);
+
+    if (io_count != sizeof(io_buf)) {
+        ERROR_PRINT(Failed to recv msg from the bridge socket)
+        close(socket_fd);
+        return -1;
+    }
+
+    daemon_pid = io_buf;
+    daemon_address->sun_family = AF_LOCAL;
+    snprintf(daemon_address->sun_path, sizeof(daemon_address->sun_path), TOSTRING(LIKWIDSOCKETBASE) "-%d", daemon_pid);
+
+    daemon_pids[cpu_id] = daemon_pid;
+    nr_daemons++;
+    return 0;
+}
+
+static int
+access_client_daemon_connect(int cpu_id, struct sockaddr_un *address) {
+    int res = 0;
+    char* filepath;
+    size_t address_length;
+    int timeout = 1000;
+    int socket_fd = -1;
+
+    socket_fd = socket(AF_LOCAL, SOCK_STREAM, 0);
+    if (socket_fd < 0)
+    {
+        ERROR_PRINT(socket() failed);
+        return -1;
+    }
+
+    address_length = sizeof(struct sockaddr_un);
+    filepath = strdup(address->sun_path);
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Waiting for socket file %s, address->sun_path);
+    while (access(address->sun_path, F_OK) && timeout > 0)
+    {
+        usleep(2500);
+        timeout--;
+    }
+    if (!access(address->sun_path, F_OK))
+    {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Socket file %s exists, address->sun_path);
+    }
+    timeout = 1000;
+
+    res = connect(socket_fd, (struct sockaddr *) address, address_length);
+    while (res && timeout > 0)
+    {
+        usleep(2500);
+        res = connect(socket_fd, (struct sockaddr *) address, address_length);
 
         if (res == 0)
         {
@@ -196,12 +328,35 @@ access_client_startDaemon(int cpu_id)
         fprintf(stderr, "opened within 10 seconds. Consult the error message above\n");
         fprintf(stderr, "this to find out why. If the error is 'no such file or directoy',\n");
         fprintf(stderr, "it usually means that likwid-accessD just failed to start.\n");
-        exit(EXIT_FAILURE);
+        free(filepath);
+        close(socket_fd);
+        return -1;
     }
     DEBUG_PRINT(DEBUGLEV_INFO, Successfully opened socket %s to daemon for CPU %d, filepath, cpu_id);
     free(filepath);
-    daemon_pids[cpu_id] = pid;
-    nr_daemons++;
+    return socket_fd;
+}
+
+static int
+access_client_startDaemon(int cpu_id) {
+    const char *bridge_path;
+    struct sockaddr_un address;
+    int daemon_ret_code;
+    int socket_fd;
+
+    bridge_path = getenv("LIKWID_BRIDGE_PATH");
+
+    if (!bridge_path) {
+        daemon_ret_code = access_client_startDaemon_direct(cpu_id, &address);
+    } else {
+        daemon_ret_code = access_client_startDaemon_bridge(cpu_id, bridge_path, &address);
+    }
+
+    if (daemon_ret_code < 0) {
+        return daemon_ret_code;
+    }
+
+    socket_fd = access_client_daemon_connect(cpu_id, &address);
     return socket_fd;
 }
 
@@ -342,12 +497,12 @@ access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t
             if (dev == MSR_DEV)
             {
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, Got error '%s' from access daemon reading reg 0x%X at CPU %d,
-                            access_client_strerror(record.errorcode), reg, cpu_id);
+                            access_client_strerror(record.errorcode), reg, record.cpu);
             }
             else
             {
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, Got error '%s' from access daemon reading reg 0x%X on socket %d,
-                            access_client_strerror(record.errorcode), reg, cpu_id);
+                            access_client_strerror(record.errorcode), reg, record.cpu);
             }
             *data = 0;
             return access_client_errno(record.errorcode);
@@ -432,12 +587,12 @@ access_client_write(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t
             if (dev == MSR_DEV)
             {
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, Got error '%s' from access daemon writing reg 0x%X at CPU %d,
-                            access_client_strerror(record.errorcode), reg, cpu_id);
+                            access_client_strerror(record.errorcode), reg, record.cpu);
             }
             else
             {
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, Got error '%s' from access daemon writing reg 0x%X on socket %d,
-                            access_client_strerror(record.errorcode), reg, cpu_id);
+                            access_client_strerror(record.errorcode), reg, record.cpu);
             }
             return access_client_errno(record.errorcode);
         }
@@ -518,7 +673,16 @@ access_client_check(PciDeviceIndex dev, int cpu_id)
         }
         else
         {
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, Device check for dev %d on CPU %d with accessDaemon failed: %s\n, dev, cpu_id, access_client_strerror(record.errorcode));
+            if (dev == MSR_DEV)
+            {
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, Device check for dev %d on CPU %d with accessDaemon failed,
+                            dev, record.cpu, access_client_strerror(record.errorcode));
+            }
+            else
+            {
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, Device check for dev %d on socket %d with accessDaemon failed,
+                            dev, record.cpu, access_client_strerror(record.errorcode));
+            }
         }
     }
     return 0;
