@@ -9,14 +9,14 @@
  *                   sys interface of the Linux 2.6 kernel. This module
  *                   is based on the msr-util tools.
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com.
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -61,8 +61,6 @@
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define MAX_LENGTH_MSR_DEV_NAME  24
-#define STRINGIFY(x) #x
-#define TOSTRING(x) STRINGIFY(x)
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -182,18 +180,26 @@ void
 access_x86_msr_finalize(const int cpu_id)
 {
     int i = 0;
-
+    access_x86_rdpmc_finalize(cpu_id);
     if (FD && FD[cpu_id] > 0)
     {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Closing FD for CPU %d, cpu_id);
         close(FD[cpu_id]);
-        FD[cpu_id] = 0;
+        FD[cpu_id] = -1;
     }
-    if (FD)
+    int c = 0;
+    for (i = 0; i < cpuid_topology.numHWThreads; i++)
     {
+        if (FD[i] >= 0) c++;
+    }
+    if (c == 0 && FD)
+    {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Free FD space);
+        memset(FD, -1, cpuid_topology.numHWThreads * sizeof(int));
         free(FD);
         FD = NULL;
     }
-    access_x86_rdpmc_finalize(cpu_id);
+    
 }
 
 int
