@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of rdpmc module to bypass costly msr or accessdaemon
  *
- *      Version:   5.2.2
- *      Released:  26.07.2022
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -89,7 +89,7 @@ __rdpmc(int cpu_id, int counter, uint64_t* value)
 void
 segfault_sigaction_rdpmc(int signal, siginfo_t *si, void *arg)
 {
-    exit(1);
+    quick_exit(1);
 }
 
 static int
@@ -106,6 +106,11 @@ test_rdpmc(int cpu_id, uint64_t value, int flag)
     }
     if (!pid)
     {
+        // Note: when exiting the child process we use quick_exit() instead of exit()
+        // to avoid calling things like exit handlers registered by the parent
+        // process (e.g. libuv functions from Julia) or static destructors from
+        // C++, which can cause crashes.
+
         uint64_t tmp;
         struct sigaction sa;
         memset(&sa, 0, sizeof(struct sigaction));
@@ -118,7 +123,7 @@ test_rdpmc(int cpu_id, uint64_t value, int flag)
             __rdpmc(cpu_id, value, &tmp);
             usleep(100);
         }
-        exit(0);
+        quick_exit(0);
     }
     else
     {
