@@ -8,13 +8,13 @@
  *                    that is used by likwid to avoid reading the systems architecture at
  *                    each start.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -86,6 +86,8 @@ if file == nil then
 end
 
 
+likwid.setenv("LIKWID_NO_ACCESS", "1")
+local config = likwid.getConfiguration()
 local cpuinfo = likwid.getCpuInfo()
 local cputopo = likwid.getCpuTopology()
 local numainfo = likwid.getNumaInfo()
@@ -97,7 +99,7 @@ end
 print_stdout(string.format("Writing new topology file %s", filename))
 cpuinfo["clock"] = likwid.getCpuClock()
 
-local threadPool_order = {"threadId", "coreId", "packageId", "apicId"}
+local threadPool_order = {"threadId", "coreId", "packageId", "apicId", "dieId"}
 local cacheLevels_order = {"type", "associativity", "sets", "lineSize", "size", "threads", "inclusive"}
 
 for field, value in pairs(cpuinfo) do
