@@ -6,13 +6,13 @@
  *
  *      Description:  An application to pin a program including threads
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -140,12 +140,17 @@ for opt,arg in likwid.getopt(arg, {"c:", "C:", "d:", "h", "i", "m", "p", "q", "s
     elseif (opt == "p") then
         print_domains = true
     elseif opt == "s" or opt == "skip" then
-        local s,e = arg:find("0x")
-        if s == nil then
-            print_stderr("Skip mask must be given in hex, hence start with 0x")
-            close_and_exit(1)
+        if arg:match("0x[0-9A-Fa-f]") then
+            skip_mask = arg
+        else
+            if arg:match("[0-9A-Fa-f]") then
+                print_stderr("Given skip mask looks like hex, sanitizing arg to 0x"..arg)
+                skip_mask = "0x"..arg
+            else
+                print_stderr("Skip mask must be given in hex")
+                close_and_exit(1)
+            end
         end
-        skip_mask = arg
     elseif opt == "q" or opt == "quiet" then
         likwid.setenv("LIKWID_SILENT","true")
         quiet = 1
@@ -159,7 +164,11 @@ for opt,arg in likwid.getopt(arg, {"c:", "C:", "d:", "h", "i", "m", "p", "q", "s
 end
 local execList = {}
 for i=1, likwid.tablelength(arg)-2 do
-    table.insert(execList, arg[i])
+    if string.find(arg[i], " ") then
+        table.insert(execList, "\""..arg[i].."\"")
+    else
+        table.insert(execList, arg[i])
+    end
 end
 
 likwid.setenv("LIKWID_NO_ACCESS", "1")
