@@ -7,13 +7,13 @@
  *      Description:  An application to get information about power
  *      consumption on architectures implementing the RAPL interface.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -61,7 +61,7 @@ local function usage()
     print_stdout("-i, --info\t Print information from MSR_PKG_POWER_INFO register and Turbo mode")
     print_stdout("-s <duration>\t Set measure duration in us, ms or s. (default 2s)")
     print_stdout("-p\t\t Print dynamic clocking and CPI values, uses likwid-perfctr")
-    print_stdout("-t\t\t Print current temperatures of all CPU cores")
+    print_stdout("-t\t\t Print current temperatures of all hardware threads")
     print_stdout("-f\t\t Print current temperatures in Fahrenheit")
     print_stdout("")
     examples()
@@ -84,8 +84,7 @@ time_interval = 2.E06
 time_orig = "2s"
 read_interval = 30.E06
 sockets = {}
-
-
+raw_selection = nil
 cpuinfo = likwid.getCpuInfo()
 if cpuinfo["isIntel"] == 1 then
     domainList = {"PKG", "PP0", "PP1", "DRAM", "PLATFORM"}
@@ -116,6 +115,7 @@ for opt,arg in likwid.getopt(arg, {"V:", "c:", "h", "i", "M:", "p", "s:", "v", "
         if num_sockets == 0 then
             os.exit(1)
         end
+        raw_selection = arg
     elseif (opt == "M") then
         access_mode = tonumber(arg)
         if (access_mode == nil) then
@@ -157,56 +157,55 @@ end
 cpulist = {}
 before = {}
 after = {}
-if #sockets > 0 then
-    for i,socketId in pairs(sockets) do
-        local affinityID = "S"..tostring(socketId)
-        for j, domain in pairs(affinity["domains"]) do
-            if domain["tag"] == affinityID then
-                table.insert(cpulist,domain["processorList"][1])
-                before[domain["processorList"][1]] = {}
-                after[domain["processorList"][1]] = {}
-                for _, id in pairs(domainList) do
-                    before[domain["processorList"][1]][id] = 0
-                    after[domain["processorList"][1]][id] = 0
-                end
-            end
-        end
-    end
-else
-    for j, domain in pairs(affinity["domains"]) do
-        if domain["tag"]:match("S%d+") then
-            table.insert(cpulist,domain["processorList"][1])
-            table.insert(sockets, domain["tag"]:match("S(%d+)"))
-            before[domain["processorList"][1]] = {}
-            after[domain["processorList"][1]] = {}
-            for _, id in pairs(domainList) do
-                before[domain["processorList"][1]][id] = 0
-                after[domain["processorList"][1]][id] = 0
-            end
+sock_cpulist = {}
+if #sockets == 0 then
+    for i, domain in pairs(affinity["domains"]) do
+        if domain["tag"]:match("S%d+") and domain["numberOfProcessors"] > 0 then
+            sid = domain["tag"]:match("S(%d+)")
+            table.insert(sockets, sid)
         end
     end
 end
-
+for i, sid in pairs(sockets) do
+    pin = string.format("S%s", tostring(sid))
+    _, sock_cpulist[sid] = likwid.cpustr_to_cpulist(pin)
+    before[sock_cpulist[sid][1]] = {}
+    after[sock_cpulist[sid][1]] = {}
+    for _, id in pairs(domainList) do
+        before[sock_cpulist[sid][1]][id] = 0
+        after[sock_cpulist[sid][1]][id] = 0
+    end
+end
 
 if likwid.setAccessClientMode(access_mode) ~= 0 then
     os.exit(1)
 end
 
+if #sockets == 0 or likwid.tablelength(sock_cpulist) == 0 then
+    if raw_selection then
+        print_stderr(string.format("No CPU accessible for selection -c %s", raw_selection))
+    else
+        print_stderr(string.format("No CPU accessible in all sockets"))
+    end
+    os.exit(1)
+end
+
+
 power = likwid.getPowerInfo()
 if not power then
-    print_stderr(string.format("The %s does not support reading power data or access is locked",cpuinfo["name"]))
+    print_stderr(string.format("The %s does not support reading power data or access is locked", cpuinfo["name"]))
     os.exit(1)
 end
 
 
 if not use_perfctr then
     print_stdout(likwid.hline);
-    print_stdout(string.format("CPU name:\t%s",cpuinfo["osname"]))
-    print_stdout(string.format("CPU type:\t%s",cpuinfo["name"]))
+    print_stdout(string.format("CPU name:\t%s", cpuinfo["osname"]))
+    print_stdout(string.format("CPU type:\t%s", cpuinfo["name"]))
     if cpuinfo["clock"] > 0 then
-        print_stdout(string.format("CPU clock:\t%3.2f GHz",cpuinfo["clock"] *  1.E-09))
+        print_stdout(string.format("CPU clock:\t%3.2f GHz", cpuinfo["clock"] * 1.E-09))
     else
-        print_stdout(string.format("CPU clock:\t%3.2f GHz",likwid.getCpuClock() *  1.E-09))
+        print_stdout(string.format("CPU clock:\t%3.2f GHz", likwid.getCpuClock() * 1.E-09))
     end
     print_stdout(likwid.hline)
 end
@@ -217,7 +216,7 @@ if print_info or verbose > 0 then
         print_stdout(string.format("Minimal clock:\t%.2f MHz", power["minFrequency"]))
         print_stdout("Turbo Boost Steps:")
         for i,step in pairs(power["turbo"]["steps"]) do
-            print_stdout(string.format("C%d %.2f MHz",i-1,power["turbo"]["steps"][i]))
+            print_stdout(string.format("C%d %.2f MHz", i-1, power["turbo"]["steps"][i]))
         end
     end
     print_stdout(likwid.hline)
@@ -233,10 +232,10 @@ if (print_info) then
         local domain = power["domains"][dname]
         if domain and domain["supportInfo"] then
             print_stdout(string.format("Info for RAPL domain %s:", dname));
-            print_stdout(string.format("Thermal Spec Power: %g Watt",domain["tdp"]*1E-6))
-            print_stdout(string.format("Minimum Power: %g Watt",domain["minPower"]*1E-6))
-            print_stdout(string.format("Maximum Power: %g Watt",domain["maxPower"]*1E-6))
-            print_stdout(string.format("Maximum Time Window: %g micro sec",domain["maxTimeWindow"]))
+            print_stdout(string.format("Thermal Spec Power: %g Watt", domain["tdp"]*1E-6))
+            print_stdout(string.format("Minimum Power: %g Watt", domain["minPower"]*1E-6))
+            print_stdout(string.format("Maximum Power: %g Watt", domain["maxPower"]*1E-6))
+            print_stdout(string.format("Maximum Time Window: %g micro sec", domain["maxTimeWindow"]))
             print_stdout()
         end
     end
@@ -259,16 +258,15 @@ if (stethoscope) and (time_interval < power["timeUnit"]) then
 end
 
 local execString = ""
+affinity = likwid.getAffinityInfo()
+local pinList = {}
+for i,socket in pairs(sockets) do
+    table.insert(pinList, string.format("S%u:0-%u",socket,(cputopo["numCoresPerSocket"]*cputopo["numThreadsPerCore"])-1))
+end
 if use_perfctr then
-    affinity = likwid.getAffinityInfo()
-    argString = ""
-    for i,socket in pairs(sockets) do
-        argString = argString .. string.format("S%u:0-%u",socket,(cputopo["numCoresPerSocket"]*cputopo["numThreadsPerCore"])-1)
-        if (i < #sockets) then
-            argString = argString .. "@"
-        end
-    end
-    execString = string.format("<INSTALLED_PREFIX>/bin/likwid-perfctr -C %s -f -g CLOCK ",argString)
+    execString = string.format("<INSTALLED_PREFIX>/bin/likwid-perfctr -C %s -f -g CLOCK ",table.concat(pinList, "@"))
+elseif not stethoscope then
+    execString = string.format("<INSTALLED_PREFIX>/bin/likwid-pin -c %s -q ",table.concat(pinList, "@"))
 end
 
 local execList = {}
@@ -281,23 +279,33 @@ if #arg == 0 then
     end
 else
     for i=1, likwid.tablelength(arg)-2 do
-        table.insert(execList, arg[i])
-    end
-    if use_perfctr then
-        execString = execString .. table.concat(execList," ")
-    else
-        execString = table.concat(execList," ")
+        if string.find(arg[i], " ") then
+            table.insert(execList, "\""..arg[i].."\"")
+        else
+            table.insert(execList, arg[i])
+        end
     end
+    execString = execString .. table.concat(execList," ")
 end
 
 local exitvalue = 0
 if not print_info and not print_temp then
     if stethoscope or (#arg > 0 and not use_perfctr) then
         for i,socket in pairs(sockets) do
-            cpu = cpulist[i]
             for idx, dom in pairs(domainList) do
-                if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
-                    before[cpu][dom] = likwid.startPower(cpu, idx)
+                if dom ~= "CORE" then
+                    if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
+                        cpu = sock_cpulist[socket][1]
+                        before[cpu][dom] = likwid.startPower(cpu, idx)
+                    end
+                else
+                    if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
+                        local sum = 0
+                        for j, c in pairs(sock_cpulist[socket]) do
+                            sum = sum + likwid.startPower(c, idx)
+                        end
+                        before[sock_cpulist[socket][1]][dom] = sum
+                    end
                 end
             end
         end
@@ -308,10 +316,20 @@ if not print_info and not print_temp then
                 while ((read_interval <= time_interval) and (time_interval > 0)) do
                     likwid.sleep(read_interval)
                     for i,socket in pairs(sockets) do
-                        cpu = cpulist[i]
                         for idx, dom in pairs(domainList) do
-                            if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
-                                after[cpu][dom] = likwid.stopPower(cpu, idx)
+                            if dom ~= "CORE" then
+                                cpu = sock_cpulist[socket][1]
+                                if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
+                                    after[cpu][dom] = likwid.stopPower(cpu, idx)
+                                end
+                            else
+                                if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
+                                    local sum = 0
+                                    for j, c in pairs(sock_cpulist[socket]) do
+                                        sum = sum + likwid.stopPower(c, idx)
+                                    end
+                                    after[sock_cpulist[socket][1]][dom] = sum
+                                end
                             end
                         end
                     end
@@ -324,9 +342,9 @@ if not print_info and not print_temp then
                 likwid.sleep(time_interval)
             end
         else
-            local pid = likwid.startProgram(table.concat(execList," "), 0, {})
+            local pid = likwid.startProgram(execString, 0, {})
             if not pid then
-                print_stderr(string.format("Failed to execute %s!",table.concat(execList," ")))
+                print_stderr(string.format("Failed to execute %s!",execString))
                 likwid.finalize()
                 os.exit(1)
             end
@@ -337,15 +355,15 @@ if not print_info and not print_temp then
                 end
                 local remain = likwid.sleep(read_interval)
                 for i,socket in pairs(sockets) do
-                    cpu = cpulist[i]
+                    cpu = sock_cpulist[socket][1]
                     for idx, dom in pairs(domainList) do
                         if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
                             after[cpu][dom] = likwid.stopPower(cpu, idx)
                         end
                     end
                 end
-                exitvalue = likwid.checkProgram(pid)
-                if remain > 0 or exitvalue >= 0 then
+                exitvalue, exited = likwid.checkProgram(pid)
+                if exited then
                     io.stdout:flush()
                     break
                 end
@@ -354,10 +372,21 @@ if not print_info and not print_temp then
         time_after = likwid.stopClock()
 
         for i,socket in pairs(sockets) do
-            cpu = cpulist[i]
+            cpu = sock_cpulist[socket][1]
             for idx, dom in pairs(domainList) do
-                if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
-                    after[cpu][dom] = likwid.stopPower(cpu, idx)
+                if dom ~= "CORE" then
+                    cpu = sock_cpulist[socket][1]
+                    if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
+                        after[cpu][dom] = likwid.stopPower(cpu, idx)
+                    end
+                else
+                    if (power["domains"][dom] and power["domains"][dom]["supportStatus"]) then
+                        local sum = 0
+                        for j, c in pairs(sock_cpulist[socket]) do
+                            sum = sum + likwid.stopPower(c, idx)
+                        end
+                        after[sock_cpulist[socket][1]][dom] = sum
+                    end
                 end
             end
         end
@@ -367,15 +396,14 @@ if not print_info and not print_temp then
         print_stdout(string.format("Runtime: %g s",runtime))
 
         for i,socket in pairs(sockets) do
-            cpu = cpulist[i]
-            print_stdout(string.format("Measure for socket %d on CPU %d", socket,cpu ))
-            table.concat(domainList, ",")
+            cpu = sock_cpulist[socket][1]
+            print_stdout(string.format("Measure for socket %d on CPU %d", socket, cpu))
             for j, dom in pairs(domainList) do
                 if power["domains"][dom] and power["domains"][dom]["supportStatus"] then
                     local energy = likwid.calcPower(before[cpu][dom], after[cpu][dom], j-1)
                     print_stdout(string.format("Domain %s:", dom))
-                    print_stdout(string.format("Energy consumed: %g Joules",energy))
-                    print_stdout(string.format("Power consumed: %g Watt",energy/runtime))
+                    print_stdout(string.format("Energy consumed: %g Joules", energy))
+                    print_stdout(string.format("Power consumed: %g Watt", energy/runtime))
                 end
             end
             if i < #sockets then print_stdout("") end
@@ -394,7 +422,7 @@ end
 
 if print_temp and (string.find(cpuinfo["features"],"TM2") ~= nil) then
     print_stdout(likwid.hline)
-    print_stdout("Current core temperatures:");
+    print_stdout("Current HW thread temperatures:");
     for i=1,cputopo["numSockets"] do
         local tag = "S" .. tostring(i-1)
         for _, domain in pairs(affinity["domains"]) do
@@ -404,9 +432,9 @@ if print_temp and (string.find(cpuinfo["features"],"TM2") ~= nil) then
                     likwid.initTemp(cpuid);
                     if (fahrenheit) then
                         local f = 1.8*tonumber(likwid.readTemp(cpuid))+32
-                        print_stdout(string.format("Socket %d Core %d: %.0f F",i-1,cpuid, f));
+                        print_stdout(string.format("Socket %d HWThread %d: %.0f F",i-1,cpuid, f));
                     else
-                        print_stdout(string.format("Socket %d Core %d: %.0f C",i-1,cpuid, tonumber(likwid.readTemp(cpuid))));
+                        print_stdout(string.format("Socket %d HWThread %d: %.0f C",i-1,cpuid, tonumber(likwid.readTemp(cpuid))));
                     end
                 end
             end
