@@ -5,15 +5,16 @@
  *      Filename:  likwid-topology.lua
  *
  *      Description:  A application to determine the thread and cache topology
- *                    on x86 processors.
+ *                    on x86, ARM and POWER processors. Also the Nvidia GPU topology
+ *                    is printed.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -42,25 +43,27 @@ end
 
 function usage()
     version()
-    io.stdout:write("A tool to print the thread and cache topology on x86 CPUs.\n\n")
+    io.stdout:write("A tool to print the thread and cache topology on CPUs and GPUs.\n\n")
     io.stdout:write("Options:\n")
     io.stdout:write("-h, --help\t\t Help message\n")
     io.stdout:write("-v, --version\t\t Version information\n")
     io.stdout:write("-V, --verbose <level>\t Set verbosity\n")
     io.stdout:write("-c, --caches\t\t List cache information\n")
     io.stdout:write("-C, --clock\t\t Measure processor clock\n")
+    io.stdout:write("-G, --gpus\t\t List GPU information\n")
     io.stdout:write("-O\t\t\t CSV output\n")
     io.stdout:write("-o, --output <file>\t Store output to file. (Optional: Apply text filter)\n")
     io.stdout:write("-g\t\t\t Graphical output\n")
 end
 
 print_caches = false
+print_gpus = false
 print_graphical = false
 measure_clock = false
 outfile = nil
 output_csv = {}
 
-for opt,arg in likwid.getopt(arg, {"h","v","c","C","g","o:","V:","O","help","version","verbose:","clock","caches","output:"}) do
+for opt,arg in likwid.getopt(arg, {"h","v","c","C","g","o:","V:", "G","O","help","version","verbose:","clock","caches","output:", "gpus"}) do
     if (type(arg) == "string") then
         local s,e = arg:find("-");
         if s == 1 then
@@ -87,6 +90,8 @@ for opt,arg in likwid.getopt(arg, {"h","v","c","C","g","o:","V:","O","help","ver
         measure_clock = true
     elseif opt == "g" then
         print_graphical = true
+    elseif opt == "G" or opt == "gpus" then
+        print_gpus = true
     elseif opt == "O" then
         print_csv = true
     elseif opt == "o" or opt == "output" then
@@ -139,23 +144,30 @@ table.insert(output_csv, "STRUCT,Hardware Thread Topology,3")
 table.insert(output_csv, "Hardware Thread Topology")
 table.insert(output_csv, likwid.sline)
 table.insert(output_csv, string.format("Sockets:\t\t%u",cputopo["numSockets"]))
+table.insert(output_csv, string.format("CPU dies:\t\t%u",cputopo["numDies"]))
 table.insert(output_csv, string.format("Cores per socket:\t%u",cputopo["numCoresPerSocket"]))
 table.insert(output_csv, string.format("Threads per core:\t%u",cputopo["numThreadsPerCore"]))
 table.insert(output_csv, likwid.hline)
 table.insert(output_csv, "TABLE,Topology,"..tostring(cputopo["numHWThreads"]))
-table.insert(output_csv, "HWThread\tThread\t\tCore\t\tSocket\t\tAvailable")
-
+header = {"HWThread", "Thread", "Core", "Die", "Socket", "Available"}
 for cntr=0,cputopo["numHWThreads"]-1 do
+    local line = {tostring(cntr),
+                  tostring(cputopo["threadPool"][cntr]["threadId"]),
+                  tostring(cputopo["threadPool"][cntr]["coreId"]),
+                  tostring(cputopo["threadPool"][cntr]["dieId"]),
+                  tostring(cputopo["threadPool"][cntr]["packageId"])}
     if cputopo["threadPool"][cntr]["inCpuSet"] == 1 then
-        table.insert(output_csv, string.format("%d\t\t%u\t\t%u\t\t%u\t\t*",cntr,
-                            cputopo["threadPool"][cntr]["threadId"],
-                            cputopo["threadPool"][cntr]["coreId"],
-                            cputopo["threadPool"][cntr]["packageId"]))
+        table.insert(line, "*")
     else
-        table.insert(output_csv, string.format("%d\t\t%u\t\t%u\t\t%u",cntr,
-                            cputopo["threadPool"][cntr]["threadId"],
-                            cputopo["threadPool"][cntr]["coreId"],
-                            cputopo["threadPool"][cntr]["packageId"]))
+        table.insert(line, "")
+    end
+    if print_csv then
+        if cntr == 0 then
+            table.insert(output_csv, table.concat(header, ","))
+        end
+        table.insert(output_csv, table.concat(line, ","))
+    else
+        table.insert(output_csv, likwid.printTextTable(header, line, cntr == 0))
     end
 end
 table.insert(output_csv, likwid.hline)
@@ -187,7 +199,12 @@ for level=1,cputopo["numCacheLevels"] do
         if (cputopo["cacheLevels"][level]["size"] < 1048576) then
             table.insert(output_csv, string.format("Size:\t\t\t%.0f kB",cputopo["cacheLevels"][level]["size"]/1024))
         else
-            table.insert(output_csv, string.format("Size:\t\t\t%.0f MB",cputopo["cacheLevels"][level]["size"]/1048576))
+            local x = cputopo["cacheLevels"][level]["size"]/1048576
+            if tonumber(string.format("%.2f", x)) == tonumber(string.format("%.0f", x)) then
+                table.insert(output_csv, string.format("Size:\t\t\t%.0f MB", x))
+            else
+                table.insert(output_csv, string.format("Size:\t\t\t%.2f MB", x))
+            end
         end
 
         if (print_caches) then
@@ -259,7 +276,126 @@ else
     end
 end
 
+if likwid.nvSupported() then
+    local cudatopo = likwid.getCudaTopology()
+    if cudatopo then
+        table.insert(output_csv, likwid.sline)
+        table.insert(output_csv, "GPU Topology")
+        table.insert(output_csv, likwid.sline)
+        table.insert(output_csv, string.format("GPU count:\t\t%d", cudatopo["numDevices"]))
+        table.insert(output_csv, likwid.hline)
 
+        for i=1, cudatopo["numDevices"] do
+            gpu = cudatopo["devices"][i]
+            table.insert(output_csv, string.format("STRUCT,GPU Topology %d,9", gpu["id"]))
+            table.insert(output_csv, string.format("ID:\t\t\t%d", gpu["id"]))
+            table.insert(output_csv, string.format("Name:\t\t\t%s", gpu["name"]))
+            table.insert(output_csv, string.format("Compute capability:\t%d.%d", gpu["ccapMajor"], gpu["ccapMinor"]))
+            table.insert(output_csv, string.format("L2 size:\t\t%.2f MB", gpu["l2Size"]/(1024*1024)))
+            table.insert(output_csv, string.format("Memory:\t\t\t%.2f GB", gpu["memory"]/(1024*1024*1024)))
+            table.insert(output_csv, string.format("SIMD width:\t\t%d", gpu["simdWidth"]))
+            table.insert(output_csv, string.format("Clock rate:\t\t%d kHz", gpu["clockRatekHz"]))
+            table.insert(output_csv, string.format("Memory clock rate:\t%d kHz", gpu["memClockRatekHz"]))
+            table.insert(output_csv, string.format("Attached to NUMA node:\t%d", gpu["numaNode"]))
+            if print_gpus then
+                table.insert(output_csv, string.format("Number of SPs:\t\t%d", gpu["numMultiProcs"]))
+                table.insert(output_csv, string.format("Max. threads per SP:\t%d", gpu["maxThreadPerMultiProc"]))
+                table.insert(output_csv, string.format("Max. threads per block:\t%d", gpu["maxThreadsPerBlock"]))
+                local s = {}
+                for i, data in pairs(gpu["maxThreadsDim"]) do
+                    table.insert(s, string.format("%d", data))
+                end
+                table.insert(output_csv, string.format("Max. thread dimensions:\t%s", table.concat(s, "/")))
+                table.insert(output_csv, string.format("Max. regs per block:\t%d", gpu["regsPerBlock"]))
+                table.insert(output_csv, string.format("Shared mem per block:\t%d", gpu["sharedMemPerBlock"]))
+
+                table.insert(output_csv, string.format("Memory bus width:\t%d", gpu["memBusWidth"]))
+                table.insert(output_csv, string.format("Texture alignment:\t%d", gpu["textureAlign"]))
+                table.insert(output_csv, string.format("Surface alignment:\t%d", gpu["surfaceAlign"]))
+                if gpu["ecc"] == 0 then
+                    table.insert(output_csv, "ECC:\t\t\toff")
+                else
+                    table.insert(output_csv, "ECC:\t\t\ton")
+                end
+                if gpu["integrated"] == 0 then
+                    table.insert(output_csv, "GPU integrated:\t\tno")
+                else
+                    table.insert(output_csv, "GPU integrated:\t\tyes")
+                end
+                s = {}
+                for i, data in pairs(gpu["maxGridSize"]) do
+                    table.insert(s, string.format("%d", data))
+                end
+                table.insert(output_csv, string.format("Max. grid sizes:\t%s", table.concat(s, "/")))
+                table.insert(output_csv, string.format("PCI bus:\t\t0x%x", gpu["pciBus"]))
+                table.insert(output_csv, string.format("PCI domain:\t\t0x%x", gpu["pciDom"]))
+                table.insert(output_csv, string.format("PCI device:\t\t0x%x", gpu["pciDev"]))
+                table.insert(output_csv, string.format("PCI function:\t\t0x%x", gpu["pciFunc"]))
+            end
+            table.insert(output_csv, likwid.hline)
+        end
+    end
+    likwid.putCudaTopology()
+end
+
+if likwid.rocmSupported() then
+    local rocmtopo = likwid.getRocmTopology()
+    if rocmtopo then
+        table.insert(output_csv, likwid.sline)
+        table.insert(output_csv, "ROCm GPU Topology")
+        table.insert(output_csv, likwid.sline)
+        table.insert(output_csv, string.format("GPU count:\t\t%d", rocmtopo["numDevices"]))
+        table.insert(output_csv, likwid.hline)
+
+        for i=1, rocmtopo["numDevices"] do
+            gpu = rocmtopo["devices"][i]
+            table.insert(output_csv, string.format("STRUCT,GPU Topology %d,9", gpu["id"]))
+            table.insert(output_csv, string.format("ID:\t\t\t%d", gpu["id"]))
+            table.insert(output_csv, string.format("Name:\t\t\t%s", gpu["name"]))
+            table.insert(output_csv, string.format("Compute capability:\t%d.%d", gpu["ccapMajor"], gpu["ccapMinor"]))
+            table.insert(output_csv, string.format("L2 size:\t\t%.2f MB", gpu["l2Size"]/(1024*1024)))
+            table.insert(output_csv, string.format("Memory:\t\t\t%.2f GB", gpu["memory"]/(1024*1024*1024)))
+            table.insert(output_csv, string.format("Clock rate:\t\t%d kHz", gpu["clockRatekHz"]))
+            table.insert(output_csv, string.format("Memory clock rate:\t%d kHz", gpu["memClockRatekHz"]))
+            table.insert(output_csv, string.format("Attached to NUMA node:\t%d", gpu["numaNode"]))
+            if print_gpus then
+                table.insert(output_csv, string.format("Number of SPs:\t\t%d", gpu["numMultiProcs"]))
+                table.insert(output_csv, string.format("Max. threads per SP:\t%d", gpu["maxThreadPerMultiProc"]))
+                table.insert(output_csv, string.format("Max. threads per block:\t%d", gpu["maxThreadsPerBlock"]))
+                local s = {}
+                for i, data in pairs(gpu["maxThreadsDim"]) do
+                    table.insert(s, string.format("%d", data))
+                end
+                table.insert(output_csv, string.format("Max. thread dimensions:\t%s", table.concat(s, "/")))
+                table.insert(output_csv, string.format("Max. regs per block:\t%d", gpu["regsPerBlock"]))
+                table.insert(output_csv, string.format("Shared mem per block:\t%d", gpu["sharedMemPerBlock"]))
+
+                table.insert(output_csv, string.format("Memory bus width:\t%d", gpu["memBusWidth"]))
+                table.insert(output_csv, string.format("Texture alignment:\t%d", gpu["textureAlign"]))
+                if gpu["ecc"] == 0 then
+                    table.insert(output_csv, "ECC:\t\t\toff")
+                else
+                    table.insert(output_csv, "ECC:\t\t\ton")
+                end
+                if gpu["integrated"] == 0 then
+                    table.insert(output_csv, "GPU integrated:\t\tno")
+                else
+                    table.insert(output_csv, "GPU integrated:\t\tyes")
+                end
+                s = {}
+                for i, data in pairs(gpu["maxGridSize"]) do
+                    table.insert(s, string.format("%d", data))
+                end
+                table.insert(output_csv, string.format("Max. grid sizes:\t%s", table.concat(s, "/")))
+                table.insert(output_csv, string.format("PCI bus:\t\t0x%x", gpu["pciBus"]))
+                table.insert(output_csv, string.format("PCI domain:\t\t0x%x", gpu["pciDom"]))
+                table.insert(output_csv, string.format("PCI device:\t\t0x%x", gpu["pciDev"]))
+            end
+            table.insert(output_csv, likwid.hline)
+        end
+    end
+    likwid.putRocmTopology()
+end
 
 if print_csv then
     longest_line = 0
