@@ -5,13 +5,13 @@
  *
  *      Description:  Lua LIKWID interface library
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -86,6 +86,7 @@ likwid.putTopology = likwid_putTopology
 likwid.getNumaInfo = likwid_getNumaInfo
 likwid.putNumaInfo = likwid_putNumaInfo
 likwid.setMemInterleaved = likwid_setMemInterleaved
+likwid.setMembind = likwid_setMembind
 likwid.getAffinityInfo = likwid_getAffinityInfo
 likwid.putAffinityInfo = likwid_putAffinityInfo
 likwid.getPowerInfo = likwid_getPowerInfo
@@ -130,11 +131,13 @@ likwid.sockstr_to_socklist = likwid_sockstr_to_socklist
 likwid.markerInit = likwid_markerInit
 likwid.markerThreadInit = likwid_markerThreadInit
 likwid.markerClose = likwid_markerClose
+likwid.markerWriteFile = likwid_markerWriteFile
 likwid.markerNextGroup = likwid_markerNextGroup
 likwid.registerRegion = likwid_registerRegion
 likwid.startRegion = likwid_startRegion
 likwid.stopRegion = likwid_stopRegion
 likwid.getRegion = likwid_getRegion
+likwid.resetRegion = likwid_resetRegion
 likwid.initCpuFeatures = likwid_cpuFeaturesInit
 likwid.getCpuFeatures = likwid_cpuFeaturesGet
 likwid.enableCpuFeatures = likwid_cpuFeaturesEnable
@@ -151,13 +154,18 @@ likwid.markerRegionTime = likwid_markerRegionTime
 likwid.markerRegionCount = likwid_markerRegionCount
 likwid.markerRegionResult = likwid_markerRegionResult
 likwid.markerRegionMetric = likwid_markerRegionMetric
+likwid.initFreq = likwid_initFreq
+likwid.getCpuClockBase = likwid_getCpuClockBase
 likwid.getCpuClockCurrent = likwid_getCpuClockCurrent
 likwid.getCpuClockMin = likwid_getCpuClockMin
+likwid.getConfCpuClockMin = likwid_getConfCpuClockMin
 likwid.setCpuClockMin = likwid_setCpuClockMin
 likwid.getCpuClockMax = likwid_getCpuClockMax
+likwid.getConfCpuClockMax = likwid_getConfCpuClockMax
 likwid.setCpuClockMax = likwid_setCpuClockMax
 likwid.getGovernor = likwid_getGovernor
 likwid.setGovernor = likwid_setGovernor
+likwid.finalizeFreq = likwid_finalizeFreq
 likwid.setTurbo = likwid_setTurbo
 likwid.getTurbo = likwid_getTurbo
 likwid.setUncoreFreqMin = likwid_setUncoreFreqMin
@@ -172,6 +180,72 @@ likwid.seteuid = likwid_seteuid
 likwid.setresuid = likwid_setresuid
 likwid.setresuser = likwid_setresuser
 
+likwid.initSysFeatures = likwid_initSysFeatures
+likwid.sysFeatures_list = likwid_sysFeatures_list
+likwid.sysFeatures_get = likwid_sysFeatures_get
+likwid.sysFeatures_set = likwid_sysFeatures_set
+likwid.finalizeSysFeatures = likwid_finalizeSysFeatures
+
+likwid.createDevice = likwid_createDevice
+likwid.createDevicesFromString = likwid_createDevicesFromString
+likwid.getAvailableDevices = likwid_getAvailableDevices
+
+likwid.getCudaTopology = likwid_getCudaTopology
+likwid.putCudaTopology = likwid_putCudaTopology
+likwid.getCudaEventsAndCounters = likwid_getCudaEventsAndCounters
+likwid.getCudaGroups = likwid_getCudaGroups
+likwid.gpustr_to_gpulist_cuda = likwid_gpustr_to_gpulist_cuda
+likwid.nvGetNameOfGroup = likwid_nvGetNameOfGroup
+likwid.nvGetNameOfMetric = likwid_nvGetNameOfMetric
+likwid.nvGetNameOfEvent = likwid_nvGetNameOfEvent
+likwid.nvGetNameOfCounter = likwid_nvGetNameOfCounter
+
+likwid.nvSupported = likwid_nvSupported
+likwid.readNvMarkerFile = likwid_readNvMarkerFile
+likwid.destroyNvMarkerFile = likwid_destroyNvMarkerFile
+likwid.nvMarkerNumRegions = likwid_nvMarkerNumRegions
+likwid.nvMarkerRegionGroup = likwid_nvMarkerRegionGroup
+likwid.nvMarkerRegionTag = likwid_nvMarkerRegionTag
+likwid.nvMarkerRegionEvents = likwid_nvMarkerRegionEvents
+likwid.nvMarkerRegionMetrics = likwid_nvMarkerRegionMetrics
+likwid.nvMarkerRegionGpulist = likwid_nvMarkerRegionGpulist
+likwid.nvMarkerRegionGpus = likwid_nvMarkerRegionGpus
+likwid.nvMarkerRegionTime = likwid_nvMarkerRegionTime
+likwid.nvMarkerRegionCount = likwid_nvMarkerRegionCount
+likwid.nvMarkerRegionResult = likwid_nvMarkerRegionResult
+likwid.nvMarkerRegionMetric = likwid_nvMarkerRegionMetric
+likwid.nvInit = likwid_nvInit
+likwid.nvAddEventSet = likwid_nvAddEventSet
+likwid.nvFinalize = likwid_nvFinalize
+
+likwid.rocmSupported = likwid_rocmSupported
+likwid.getRocmTopology = likwid_getRocmTopology
+likwid.putRocmTopology = likwid_putRocmTopology
+likwid.getRocmEventsAndCounters = likwid_getRocmEventsAndCounters
+likwid.getRocmGroups = likwid_getRocmGroupss
+likwid.gpustr_to_gpulist_rocm = likwid_gpustr_to_gpulist_rocm
+likwid.init_rocm = likwid_init_rocm
+likwid.addEventSet_rocm = likwid_addEventSet_rocm
+likwid.finalize_rocm = likwid_finalize_rocm
+likwid.getNameOfEventRocm = likwid_getNameOfEvent_rocm
+likwid.getNameOfCounterRocm = likwid_getNameOfCounter_rocm
+likwid.getNameOfGroupRocm = likwid_getNameOfGroup_rocm
+likwid.getNameOfMetricRocm = likwid_getNameOfMetric_rocm
+likwid.readMarkerFileRocm = likwid_readMarkerFile_rocm
+likwid.destroyMarkerFileRocm = likwid_markerFile_destroy_rocm
+likwid.markerNumRegionsRocm = likwid_markerNumRegions_rocm
+likwid.markerRegionGroupRocm = likwid_markerRegionGroup_rocm
+likwid.markerRegionTagRocm = likwid_markerRegionTag_rocm
+likwid.markerRegionEventsRocm = likwid_markerRegionEvents_rocm
+likwid.markerRegionMetricsRocm = likwid_markerRegionMetrics_rocm
+likwid.markerRegionGpulistRocm = likwid_markerRegionGpulist_rocm
+likwid.markerRegionGpusRocm = likwid_markerRegionGpus_rocm
+likwid.markerRegionTimeRocm = likwid_markerRegionTime_rocm
+likwid.markerRegionCountRocm = likwid_markerRegionCount_rocm
+likwid.markerRegionResultRocm = likwid_markerRegionResult_rocm
+likwid.markerRegionMetricRocm = likwid_markerRegionMetric_rocm
+
+
 likwid.cpuFeatures = { [0]="HW_PREFETCHER", [1]="CL_PREFETCHER", [2]="DCU_PREFETCHER", [3]="IP_PREFETCHER",
                         [4]="FAST_STRINGS", [5]="THERMAL_CONTROL", [6]="PERF_MON", [7]="FERR_MULTIPLEX",
                         [8]="BRANCH_TRACE_STORAGE", [9]="XTPR_MESSAGE", [10]="PEBS", [11]="SPEEDSTEP",
@@ -187,6 +261,17 @@ likwid.signals = { [1] = "SIGHUP", [2] = "SIGINT", [3] = "SIGQUIT", [4] = "SIGIL
 
 infinity = math.huge
 
+likwid.invalid = 0
+likwid.hwthread = likwid.invalid + 1
+likwid.core = likwid.hwthread + 1
+likwid.llc = likwid.core + 1
+likwid.numa = likwid.llc + 1
+likwid.die = likwid.numa + 1
+likwid.socket = likwid.die + 1
+likwid.node = likwid.socket + 1
+likwid.nvidia_gpu = likwid.node + 1
+likwid.amd_gpu = likwid.nvidia_gpu + 1
+likwid.intel_gpu = likwid.amd_gpu + 1
 
 local function getopt(args, ostrlist)
     local arg, place,placeend = nil, 0, 0;
@@ -199,7 +284,7 @@ local function getopt(args, ostrlist)
                     if #args[1] == 2 then -- found "--"
                         place = 0
                         table.remove(args, 1)
-                        return args[1], nil
+                        return "-", nil
                     end
                     place = place + 1
                 end
@@ -372,6 +457,9 @@ end
 likwid.calculate_metric = calculate_metric
 
 local function printtable(tab)
+    printline = function(linetab)
+        print("| " .. table.concat(linetab, " | ") .. " |")
+    end
     local nr_columns = tablelength(tab)
     if nr_columns == 0 then
         print("Table has no columns. Empty table?")
@@ -404,23 +492,25 @@ local function printtable(tab)
     end
     hline = hline .. "+"
     print(hline)
-    str = "| "
+    local strtab = {}
     for i=1,nr_columns do
         front, back = get_spaces(tostring(tab[i][1]), min_lengths[i], max_lengths[i])
-        str = str .. front.. tostring(tab[i][1]) ..back
-        if i<nr_columns then
-            str = str .. " | "
-        else
-            str = str .. " |"
-        end
+        table.insert(strtab, front.. tostring(tab[i][1]) ..back)
     end
-    print(str)
+    printline(strtab)
     print(hline)
     for j=2,nr_lines do
         str = "| "
         for i=1,nr_columns do
+            local mtab = getmetatable(tab[i])
             front, back = get_spaces(tostring(tab[i][j]), min_lengths[i],max_lengths[i])
-            str = str .. front.. tostring(tab[i][j]) ..back
+            if mtab and mtab["align"] and mtab["align"] == "left" then
+                str = str .. tostring(tab[i][j]) .. back .. front
+            elseif mtab and mtab["align"] and mtab["align"] == "right" then
+                str = str .. front .. back .. tostring(tab[i][j])
+            else
+                str = str .. front.. tostring(tab[i][j]) ..back
+            end
             if i<nr_columns then
                 str = str .. " | "
             else
@@ -712,6 +802,7 @@ local function min_max_avg(values)
 end
 
 local function tableMinMaxAvgSum(inputtable, skip_cols, skip_lines)
+    local function isSpecial(s) return s == "nan" or s ~= "-" or s == "int" end
     local outputtable = {}
     local nr_columns = #inputtable
     if nr_columns == 0 then
@@ -734,14 +825,12 @@ local function tableMinMaxAvgSum(inputtable, skip_cols, skip_lines)
     for j=skip_cols+1,nr_columns do
         for i=skip_lines+1, nr_lines do
             local res = tonumber(inputtable[j][i])
-            if inputtable[j][i] ~= "nan" and inputtable[j][i] ~= "-" then
-                if res ~= nil then
-                    minOfLine[i-skip_lines+1] = math.min(res, minOfLine[i-skip_lines+1])
-                    maxOfLine[i-skip_lines+1] = math.max(res, maxOfLine[i-skip_lines+1])
-                    sumOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1] + res
-                end
-                avgOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1]/(nr_columns-skip_cols)
+            if res ~= nil then
+                minOfLine[i-skip_lines+1] = math.min(res, minOfLine[i-skip_lines+1])
+                maxOfLine[i-skip_lines+1] = math.max(res, maxOfLine[i-skip_lines+1])
+                sumOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1] + res
             end
+            avgOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1]/(nr_columns-skip_cols)
         end
     end
     for i=2,#minOfLine do
@@ -797,7 +886,7 @@ local function printOutput(results, metrics, cpulist, region, stats)
             infotab[1] = {"Region Info","RDTSC Runtime [s]","call count"}
             for c, cpu in pairs(cur_cpulist) do
                 local tmpList = {}
-                table.insert(tmpList, "Core "..tostring(cpu))
+                table.insert(tmpList, "HWThread "..tostring(cpu))
                 table.insert(tmpList, string.format("%.6f", likwid.markerRegionTime(region, c)))
                 table.insert(tmpList, tostring(likwid.markerRegionCount(region, c)))
                 table.insert(infotab, tmpList)
@@ -820,7 +909,7 @@ local function printOutput(results, metrics, cpulist, region, stats)
             table.insert(firsttab_combined[2], countername)
         end
         for c, cpu in pairs(cur_cpulist) do
-            local tmpList = {"Core "..tostring(cpu)}
+            local tmpList = {"HWThread "..tostring(cpu)}
             if likwid.getNumberOfMetrics(g) == 0 then
                 if region == nil then
                     table.insert(tmpList, string.format("%e", runtime))
@@ -845,7 +934,7 @@ local function printOutput(results, metrics, cpulist, region, stats)
                 table.insert(secondtab_combined[1], likwid.getNameOfMetric(g, m).." STAT" )
             end
             for c, cpu in pairs(cur_cpulist) do
-                local tmpList = {"Core "..tostring(cpu)}
+                local tmpList = {"HWThread "..tostring(cpu)}
                 for m=1, likwid.getNumberOfMetrics(g) do
                     local tmp = tostring(likwid.num2str(metrics[g][m][c]))
                     table.insert(tmpList, tmp)
@@ -1186,10 +1275,26 @@ end
 
 likwid.gethostname = gethostname
 
+local function get_perf_event_paranoid()
+    local f = io.open("/proc/sys/kernel/perf_event_paranoid")
+    if f then
+        value = f:read("*l")
+        f:close()
+        return tonumber(value)
+    end
+    return 4
+end
+
+likwid.perf_event_paranoid = get_perf_event_paranoid
+
 local function getjid()
-    local jid = os.getenv("PBS_JOBID")
-    if jid == nil then
-        jid = "X"
+    jid = "X"
+    for _, v in pairs({"PBS_JOBID", "SLURM_JOB_ID", "SLURM_JOBID", "LOADL_STEP_ID", "LSB_JOBID" }) do
+        x = os.getenv(v)
+        if x then
+            jid = x
+            break
+        end
     end
     return jid
 end
@@ -1197,11 +1302,12 @@ end
 likwid.getjid = getjid
 
 local function getMPIrank()
-    local rank = os.getenv("PMI_RANK")
-    if rank == nil then
-        rank = os.getenv("OMPI_COMM_WORLD_RANK")
-        if rank == nil then
-            rank = "X"
+    rank = "X"
+    for _, v in pairs({"PMI_RANK", "OMPI_COMM_WORLD_RANK", "SLURM_PROCID"}) do
+        x = os.getenv(v)
+        if x then
+            rank = x
+            break
         end
     end
     return rank
@@ -1261,4 +1367,475 @@ end
 
 likwid.getArch = llikwid_getArch
 
+
+
+local function printOutputCuda(results, metrics, gpulist, region, stats)
+    local maxLineFields = 0
+    local gputopo = likwid.getCudaTopology()
+    local regionName = likwid.nvMarkerRegionTag(region)
+    local regionGPUs = likwid.nvMarkerRegionGpus(region)
+    local cur_gpulist = gpulist
+    if region ~= nil then
+        cur_gpulist = likwid.nvMarkerRegionGpulist(region)
+    end
+
+    for g, group in pairs(results) do
+        local infotab = {}
+        local firsttab = {}
+        local firsttab_combined = {}
+        local secondtab = {}
+        local secondtab_combined = {}
+        local runtime = likwid.nvMarkerRegionTime(g, 0)
+        local groupName = likwid.nvGetNameOfGroup(g)
+        if region ~= nil then
+            infotab[1] = {"Region Info","RDTSC Runtime [s]","call count"}
+            for c, gpu in pairs(cur_gpulist) do
+                local tmpList = {}
+                table.insert(tmpList, "GPU "..tostring(gpu))
+                table.insert(tmpList, string.format("%.6f", likwid.nvMarkerRegionTime(region, c)))
+                table.insert(tmpList, tostring(likwid.nvMarkerRegionCount(region, c)))
+                table.insert(infotab, tmpList)
+            end
+        end
+        firsttab[1] = {"Event"}
+        firsttab_combined[1] = {"Event"}
+        firsttab[2] = {"Counter"}
+        firsttab_combined[2] = {"Counter"}
+        if likwid.nvMarkerRegionMetrics(g) == 0 then
+            table.insert(firsttab[1],"Runtime (RDTSC) [s]")
+            table.insert(firsttab[2],"TSC")
+        end
+        for e, event in pairs(group) do
+            eventname = likwid.nvGetNameOfEvent(g, e)
+            countername = likwid.nvGetNameOfCounter(g, e)
+            table.insert(firsttab[1], eventname)
+            table.insert(firsttab[2], countername)
+            table.insert(firsttab_combined[1], eventname .. " STAT")
+            table.insert(firsttab_combined[2], countername)
+        end
+        for c, gpu in pairs(cur_gpulist) do
+            local tmpList = {"GPU "..tostring(gpu)}
+            if likwid.nvMarkerRegionMetrics(g) == 0 then
+                if region == nil then
+                    table.insert(tmpList, string.format("%e", runtime))
+                else
+                    table.insert(tmpList, string.format("%e", likwid.nvMarkerRegionTime(region, c)))
+                end
+            end
+            for e, event in pairs(group) do
+                local tmp = tostring(likwid.num2str(event[c]))
+                table.insert(tmpList, tmp)
+            end
+            table.insert(firsttab, tmpList)
+        end
+        if #gpulist > 1 or stats == true then
+            firsttab_combined = tableMinMaxAvgSum(firsttab, 2, 1)
+        end
+        if likwid.nvMarkerRegionMetrics(g) > 0 then
+            secondtab[1] = {"Metric"}
+            secondtab_combined[1] = {"Metric"}
+            for m=1, likwid.nvMarkerRegionMetrics(g) do
+                local mname = likwid.nvGetNameOfMetric(g, m)
+                table.insert(secondtab[1], mname)
+                table.insert(secondtab_combined[1], mname .." STAT" )
+            end
+            for c, gpu in pairs(cur_gpulist) do
+                local tmpList = {"GPU "..tostring(gpu)}
+                for m=1, likwid.nvMarkerRegionMetrics(g) do
+                    local tmp = tostring(likwid.num2str(metrics[g][m][c]))
+                    table.insert(tmpList, tmp)
+                end
+                table.insert(secondtab, tmpList)
+            end
+            if #gpulist > 1 or stats == true  then
+                secondtab_combined = tableMinMaxAvgSum(secondtab, 1, 1)
+            end
+        end
+        maxLineFields = math.max(#firsttab, #firsttab_combined,
+                                 #secondtab, #secondtab_combined)
+        if use_csv then
+--            print(string.format("STRUCT,Info,3%s",string.rep(",",maxLineFields-3)))
+--            print(string.format("GPU name:,%s%s", cpuinfo["osname"],string.rep(",",maxLineFields-2)))
+--            print(string.format("CPU type:,%s%s", cpuinfo["name"],string.rep(",",maxLineFields-2)))
+--            print(string.format("CPU clock:,%s GHz%s", clock*1.E-09,string.rep(",",maxLineFields-2)))
+            if region == nil then
+                print(string.format("TABLE,Group %d Raw,%s,%d%s",g,groupName,#firsttab[1]-1,string.rep(",",maxLineFields-4)))
+            else
+                print(string.format("TABLE,Region %s,Group %d Raw,%s,%d%s",regionName,g,groupName,#firsttab[1]-1,string.rep(",",maxLineFields-5)))
+            end
+            if #infotab > 0 then
+                likwid.printcsv(infotab, maxLineFields)
+            end
+            likwid.printcsv(firsttab, maxLineFields)
+        else
+            if outfile ~= nil then
+                print(likwid.hline)
+--                print(string.format("CPU name:\t%s",cpuinfo["osname"]))
+--                print(string.format("CPU type:\t%s",cpuinfo["name"]))
+--                print(string.format("CPU clock:\t%3.2f GHz",clock * 1.E-09))
+                print(likwid.hline)
+            end
+            if region == nil then
+                print("Group "..tostring(g)..": "..groupName)
+            else
+                print("Region "..regionName..", Group "..tostring(g)..": "..groupName)
+            end
+            if #infotab > 0 then
+                likwid.printtable(infotab)
+            end
+            likwid.printtable(firsttab)
+        end
+        if #cur_gpulist > 1 or stats == true then
+            if use_csv then
+                if region == nil then
+                    print(string.format("TABLE,Group %d Raw STAT,%s,%d%s",g,groupName,#firsttab_combined[1]-1,string.rep(",",maxLineFields-4)))
+                else
+                    print(string.format("TABLE,Region %s,Group %d Raw STAT,%s,%d%s",regionName, g,groupName,#firsttab_combined[1]-1,string.rep(",",maxLineFields-5)))
+                end
+                likwid.printcsv(firsttab_combined, maxLineFields)
+            else
+                likwid.printtable(firsttab_combined)
+            end
+        end
+        if likwid.nvMarkerRegionMetrics(g) > 0 then
+            if use_csv then
+                if region == nil then
+                    print(string.format("TABLE,Group %d Metric,%s,%d%s",g,groupName,#secondtab[1]-1,string.rep(",",maxLineFields-4)))
+                else
+                    print(string.format("TABLE,Region %s,Group %d Metric,%s,%d%s",regionName,g,groupName,#secondtab[1]-1,string.rep(",",maxLineFields-5)))
+                end
+                likwid.printcsv(secondtab, maxLineFields)
+            else
+                likwid.printtable(secondtab)
+            end
+            if #cur_gpulist > 1 or stats == true then
+                if use_csv then
+                    if region == nil then
+                        print(string.format("TABLE,Group %d Metric STAT,%s,%d%s",g,groupName,#secondtab_combined[1]-1,string.rep(",",maxLineFields-4)))
+                    else
+                        print(string.format("TABLE,Region %s,Group %d Metric STAT,%s,%d%s",regionName,g,groupName,#secondtab_combined[1]-1,string.rep(",",maxLineFields-5)))
+                    end
+                    likwid.printcsv(secondtab_combined, maxLineFields)
+                else
+                    likwid.printtable(secondtab_combined)
+                end
+            end
+        end
+    end
+end
+
+likwid.printOutputCuda = printOutputCuda
+
+local function getMarkerResultsCuda(filename, gpulist, nan2value)
+    local gputopo = likwid.getCudaTopology()
+    local ret = likwid.readNvMarkerFile(filename)
+    if ret < 0 then
+        return nil, nil
+    elseif ret == 0 then
+        return {}, {}
+    end
+    if not nan2value then
+        nan2value = '-'
+    end
+    results = {}
+    metrics = {}
+    for i=1, likwid.nvMarkerNumRegions() do
+        local regionName = likwid.nvMarkerRegionTag(i)
+        local groupID = likwid.nvMarkerRegionGroup(i)
+        local regionThreads = likwid.nvMarkerRegionGpus(i)
+        results[i] = {}
+        metrics[i] = {}
+        results[i][groupID] = {}
+        metrics[i][groupID] = {}
+        for k=1, likwid.nvMarkerRegionEvents(i) do
+            local eventName = likwid.getNameOfEvent(groupID, k)
+            local counterName = likwid.getNameOfCounter(groupID, k)
+            results[i][groupID][k] = {}
+            for j=1, regionThreads do
+                results[i][groupID][k][j] = likwid.nvMarkerRegionResult(i,k,j)
+                if results[i][groupID][k][j] ~= results[i][groupID][k][j] then
+                    results[i][groupID][k][j] = nan2value
+                end
+            end
+        end
+        if likwid.nvMarkerRegionMetrics(groupID) > 0 then
+            for k=1, likwid.nvMarkerRegionMetrics(groupID) do
+                local metricName = likwid.getNameOfMetric(groupID, k)
+                metrics[i][likwid.nvMarkerRegionGroup(i)][k] = {}
+                for j=1, regionThreads do
+                    metrics[i][groupID][k][j] = likwid.nvMarkerRegionMetric(i,k,j)
+                    if metrics[i][groupID][k][j] ~= metrics[i][groupID][k][j] then
+                        metrics[i][groupID][k][j] = nan2value
+                    end
+                end
+            end
+        end
+    end
+    return results, metrics
+end
+
+likwid.getMarkerResultsCuda = getMarkerResultsCuda
+
+local function getMarkerResultsRocm(filename, gpulist, nan2value)
+    local gputopo = likwid.getGpuTopology_rocm()
+    local ret = likwid.readMarkerFileRocm(filename)
+    if ret < 0 then
+        return nil, nil
+    elseif ret == 0 then
+        return {}, {}
+    end
+    if not nan2value then
+        nan2value = '-'
+    end
+    results = {}
+    metrics = {}
+    for i=1, likwid.markerNumRegionsRocm() do
+        local regionName = likwid.markerRegionTagRocm(i)
+        local groupID = likwid.markerRegionGroupRocm(i)
+        local regionThreads = likwid.markerRegionGpusRocm(i)
+        results[i] = {}
+        metrics[i] = {}
+        results[i][groupID] = {}
+        metrics[i][groupID] = {}
+        for k=1, likwid.markerRegionEventsRocm(i) do
+            local eventName = likwid.getNameOfEventRocm(groupID, k)
+            local counterName = likwid.getNameOfCounterRocm(groupID, k)
+            results[i][groupID][k] = {}
+            for j=1, regionThreads do
+                results[i][groupID][k][j] = likwid.markerRegionResultRocm(i,k,j)
+                if results[i][groupID][k][j] ~= results[i][groupID][k][j] then
+                    results[i][groupID][k][j] = nan2value
+                end
+            end
+        end
+        if likwid.markerRegionMetricsRocm(groupID) > 0 then
+            for k=1, likwid.markerRegionMetricsRocm(groupID) do
+                local metricName = likwid.getNameOfMetricRocm(groupID, k)
+                metrics[i][likwid.markerRegionGroupRocm(i)][k] = {}
+                for j=1, regionThreads do
+                    metrics[i][groupID][k][j] = likwid.markerRegionMetricRocm(i,k,j)
+                    if metrics[i][groupID][k][j] ~= metrics[i][groupID][k][j] then
+                        metrics[i][groupID][k][j] = nan2value
+                    end
+                end
+            end
+        end
+    end
+    return results, metrics
+end
+
+likwid.getMarkerResultsRocm = getMarkerResultsRocm
+
+local function printOutputRocm(results, metrics, gpulist, region, stats)
+    local maxLineFields = 0
+    local gputopo = likwid.getGpuTopology_rocm()
+    local regionName = likwid.markerRegionTagRocm(region)
+    local regionGPUs = likwid.markerRegionGpusRocm(region)
+    local cur_gpulist = gpulist
+    if region ~= nil then
+        cur_gpulist = likwid.markerRegionGpulistRocm(region)
+    end
+
+    for g, group in pairs(results) do
+        local infotab = {}
+        local firsttab = {}
+        local firsttab_combined = {}
+        local secondtab = {}
+        local secondtab_combined = {}
+        local runtime = likwid.markerRegionTimeRocm(g, 0)
+        local groupName = likwid.getNameOfGroupRocm(g)
+        if region ~= nil then
+            infotab[1] = {"Region Info","RDTSC Runtime [s]","call count"}
+            for c, gpu in pairs(cur_gpulist) do
+                local tmpList = {}
+                table.insert(tmpList, "GPU "..tostring(gpu))
+                table.insert(tmpList, string.format("%.6f", likwid.markerRegionTimeRocm(region, c)))
+                table.insert(tmpList, tostring(likwid.markerRegionCountRocm(region, c)))
+                table.insert(infotab, tmpList)
+            end
+        end
+        firsttab[1] = {"Event"}
+        firsttab_combined[1] = {"Event"}
+        firsttab[2] = {"Counter"}
+        firsttab_combined[2] = {"Counter"}
+        if likwid.markerRegionMetricsRocm(g) == 0 then
+            table.insert(firsttab[1],"Runtime (RDTSC) [s]")
+            table.insert(firsttab[2],"TSC")
+        end
+        for e, event in pairs(group) do
+            eventname = likwid.getNameOfEventRocm(g, e)
+            countername = likwid.getNameOfCounterRocm(g, e)
+            table.insert(firsttab[1], eventname)
+            table.insert(firsttab[2], countername)
+            table.insert(firsttab_combined[1], eventname .. " STAT")
+            table.insert(firsttab_combined[2], countername)
+        end
+        for c, gpu in pairs(cur_gpulist) do
+            local tmpList = {"GPU "..tostring(gpu)}
+            if likwid.markerRegionMetricsRocm(g) == 0 then
+                if region == nil then
+                    table.insert(tmpList, string.format("%e", runtime))
+                else
+                    table.insert(tmpList, string.format("%e", likwid.markerRegionTimeRocm(region, c)))
+                end
+            end
+            for e, event in pairs(group) do
+                local tmp = tostring(likwid.num2str(event[c]))
+                table.insert(tmpList, tmp)
+            end
+            table.insert(firsttab, tmpList)
+        end
+        if #gpulist > 1 or stats == true then
+            firsttab_combined = tableMinMaxAvgSum(firsttab, 2, 1)
+        end
+        if likwid.markerRegionMetricsRocm(g) > 0 then
+            secondtab[1] = {"Metric"}
+            secondtab_combined[1] = {"Metric"}
+            for m=1, likwid.markerRegionMetricsRocm(g) do
+                local mname = likwid.getNameOfMetricRocm(g, m)
+                table.insert(secondtab[1], mname)
+                table.insert(secondtab_combined[1], mname .." STAT" )
+            end
+            for c, gpu in pairs(cur_gpulist) do
+                local tmpList = {"GPU "..tostring(gpu)}
+                for m=1, likwid.markerRegionMetricsRocm(g) do
+                    local tmp = tostring(likwid.num2str(metrics[g][m][c]))
+                    table.insert(tmpList, tmp)
+                end
+                table.insert(secondtab, tmpList)
+            end
+            if #gpulist > 1 or stats == true  then
+                secondtab_combined = tableMinMaxAvgSum(secondtab, 1, 1)
+            end
+        end
+        maxLineFields = math.max(#firsttab, #firsttab_combined,
+                                 #secondtab, #secondtab_combined)
+        if use_csv then
+--            print(string.format("STRUCT,Info,3%s",string.rep(",",maxLineFields-3)))
+--            print(string.format("GPU name:,%s%s", cpuinfo["osname"],string.rep(",",maxLineFields-2)))
+--            print(string.format("CPU type:,%s%s", cpuinfo["name"],string.rep(",",maxLineFields-2)))
+--            print(string.format("CPU clock:,%s GHz%s", clock*1.E-09,string.rep(",",maxLineFields-2)))
+            if region == nil then
+                print(string.format("TABLE,Group %d Raw,%s,%d%s",g,groupName,#firsttab[1]-1,string.rep(",",maxLineFields-4)))
+            else
+                print(string.format("TABLE,Region %s,Group %d Raw,%s,%d%s",regionName,g,groupName,#firsttab[1]-1,string.rep(",",maxLineFields-5)))
+            end
+            if #infotab > 0 then
+                likwid.printcsv(infotab, maxLineFields)
+            end
+            likwid.printcsv(firsttab, maxLineFields)
+        else
+            if outfile ~= nil then
+                print(likwid.hline)
+--                print(string.format("CPU name:\t%s",cpuinfo["osname"]))
+--                print(string.format("CPU type:\t%s",cpuinfo["name"]))
+--                print(string.format("CPU clock:\t%3.2f GHz",clock * 1.E-09))
+                print(likwid.hline)
+            end
+            if region == nil then
+                print("Group "..tostring(g)..": "..groupName)
+            else
+                print("Region "..regionName..", Group "..tostring(g)..": "..groupName)
+            end
+            if #infotab > 0 then
+                likwid.printtable(infotab)
+            end
+            likwid.printtable(firsttab)
+        end
+        if #cur_gpulist > 1 or stats == true then
+            if use_csv then
+                if region == nil then
+                    print(string.format("TABLE,Group %d Raw STAT,%s,%d%s",g,groupName,#firsttab_combined[1]-1,string.rep(",",maxLineFields-4)))
+                else
+                    print(string.format("TABLE,Region %s,Group %d Raw STAT,%s,%d%s",regionName, g,groupName,#firsttab_combined[1]-1,string.rep(",",maxLineFields-5)))
+                end
+                likwid.printcsv(firsttab_combined, maxLineFields)
+            else
+                likwid.printtable(firsttab_combined)
+            end
+        end
+        if likwid.markerRegionMetricsRocm(g) > 0 then
+            if use_csv then
+                if region == nil then
+                    print(string.format("TABLE,Group %d Metric,%s,%d%s",g,groupName,#secondtab[1]-1,string.rep(",",maxLineFields-4)))
+                else
+                    print(string.format("TABLE,Region %s,Group %d Metric,%s,%d%s",regionName,g,groupName,#secondtab[1]-1,string.rep(",",maxLineFields-5)))
+                end
+                likwid.printcsv(secondtab, maxLineFields)
+            else
+                likwid.printtable(secondtab)
+            end
+            if #cur_gpulist > 1 or stats == true then
+                if use_csv then
+                    if region == nil then
+                        print(string.format("TABLE,Group %d Metric STAT,%s,%d%s",g,groupName,#secondtab_combined[1]-1,string.rep(",",maxLineFields-4)))
+                    else
+                        print(string.format("TABLE,Region %s,Group %d Metric STAT,%s,%d%s",regionName,g,groupName,#secondtab_combined[1]-1,string.rep(",",maxLineFields-5)))
+                    end
+                    likwid.printcsv(secondtab_combined, maxLineFields)
+                else
+                    likwid.printtable(secondtab_combined)
+                end
+            end
+        end
+    end
+end
+
+likwid.printOutputRocm = printOutputRocm
+
+local function printTextTable(header, line, print_header)
+    local linelength = 80
+    local headerlength = 0
+    local out = {}
+    local headcount = 0
+    for _, h in pairs(header) do
+        headerlength = headerlength + string.len(h)
+        headcount = headcount + 1
+    end
+    hspaces = math.floor((linelength - headerlength)/(headcount-1))
+    if print_header and print_header == true then
+        table.insert(out, table.concat(header, string.rep(" ",hspaces)))
+    end
+    ltab = {}
+    for i, l in pairs(line) do
+        hlength = string.len(header[i]) + hspaces
+        v = l .. string.rep(" ",hlength - string.len(l))
+        table.insert(ltab, v)
+    end
+    table.insert(out, table.concat(ltab, ""))
+    return table.concat(out, "\n")
+end
+
+likwid.printTextTable = printTextTable
+
+local function perfctr_checkpid(pid)
+    local fname = string.format("/proc/%d/status", pid)
+    local f = io.open(fname, "r")
+    if f ~= nil then
+        f:close()
+        return true
+    end
+    return false
+end
+likwid.perfctr_checkpid = perfctr_checkpid
+
+local function perfctr_pid_cpulist(pid)
+    local cpulist = nil
+    local fname = string.format("/proc/%d/status", pid)
+    local f = io.open(fname, "r")
+    if f ~= nil then
+        local l = f:read("*line")
+        while l do
+            if l:match("Cpus_allowed_list:.*") then
+                cpulist = l:match("Cpus_allowed_list:%s+([0-9%-,]+)")
+                break
+            end
+            l = f:read("*line")
+        end
+        f:close()
+    end
+    return cpulist
+end
+likwid.perfctr_pid_cpulist = perfctr_pid_cpulist
+
 return likwid
