@@ -5,13 +5,13 @@
  *
  *      Description:  Parser for CPU selection strings
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -65,16 +65,27 @@ cpulist_sort(int* incpus, int* outcpus, int length)
     {
         return -1;
     }
-    int inner_loop = ceil((double)length/cpuid_topology->numThreadsPerCore);
-    for (int off = 0; off < cpuid_topology->numThreadsPerCore; off++)
+    for (int k = 0; k < cpuid_topology->numThreadsPerCore; k++)
     {
-        for (int i = 0; i < inner_loop; i++)
+        for (int i = 0; i < length; i++)
         {
-            outcpus[insert] = incpus[(i*cpuid_topology->numThreadsPerCore)+off];
-            insert++;
+            int idx = -1;
+            for (int j = 0; j < cpuid_topology->numHWThreads; j++)
+            {
+                if (cpuid_topology->threadPool[j].apicId == incpus[i])
+                {
+                    idx = j;
+                    break;
+                }
+            }
+            if (idx >= 0 && cpuid_topology->threadPool[idx].threadId == k)
+            {
+                outcpus[insert] = incpus[i];
+                insert++;
+            }
+            if (insert == length) break;
         }
-        if (insert == length)
-            break;
+        if (insert == length) break;
     }
     return insert;
 }
@@ -111,13 +122,13 @@ cpu_in_domain(int domainidx, int cpu)
 }
 
 static int
-get_domain_idx(bstring bdomain)
+get_domain_idx(char* bdomain)
 {
     affinity_init();
     AffinityDomains_t affinity = get_affinityDomains();
     for (int i=0;i<affinity->numberOfAffinityDomains; i++)
     {
-        if (bstrcmp(affinity->domains[i].tag, bdomain) == BSTR_OK)
+        if (strncmp(affinity->domains[i].tag, bdomain, strlen(affinity->domains[i].tag)) == BSTR_OK)
         {
             return i;
         }
@@ -144,7 +155,7 @@ cpuexpr_to_list(bstring bcpustr, bstring prefix, int* list, int length)
         oldinsert = insert;
         for (int j = 0; j < affinity->numberOfAffinityDomains; j++)
         {
-            if (bstrcmp(affinity->domains[j].tag, newstr) == 0)
+            if (strncmp(affinity->domains[j].tag, bdata(newstr), strlen(affinity->domains[j].tag)) == 0)
             {
                 tmp = check_and_atoi(bdata(strlist->entry[i]));
                 if (tmp >= 0)
@@ -202,7 +213,7 @@ cpustr_to_cpulist_method(bstring bcpustr, int* cpulist, int length)
         }
         for (int i=0; i<affinity->numberOfAffinityDomains; i++)
         {
-            if (binstr(affinity->domains[i].tag, 0, parts->entry[0]) != BSTR_ERR &&
+            if (strstr(affinity->domains[i].tag, bdata(parts->entry[0])) != NULL &&
                 affinity->domains[i].numberOfProcessors > 0)
             {
                 suitable[suitidx] = i;
@@ -376,12 +387,12 @@ cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
     CpuTopology_t cpuid_topology = get_cpuTopology();
     affinity_init();
     AffinityDomains_t affinity = get_affinityDomains();
-    bstring bdomain;
     int domainidx = -1;
     int count = 0;
     int stride = 0;
     int chunk = 0;
     int off = 0;
+    bstring bdomain;
     if (bstrchrp(bcpustr, 'E', 0) != 0)
     {
         fprintf(stderr, "Not a valid CPU expression\n");
@@ -425,7 +436,7 @@ cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
         bstrListDestroy(strlist);
         return 0;
     }
-    domainidx = get_domain_idx(bdomain);
+    domainidx = get_domain_idx(bdata(bdomain));
     if (domainidx < 0)
     {
         fprintf(stderr, "Cannot find domain %s\n", bdata(bdomain));
@@ -494,7 +505,7 @@ cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
     bdomain = bstrcpy(strlist->entry[1]);
     blist = bstrcpy(strlist->entry[2]);
     bstrListDestroy(strlist);
-    domainidx = get_domain_idx(bdomain);
+    domainidx = get_domain_idx(bdata(bdomain));
     if (domainidx < 0)
     {
         fprintf(stderr, "ERROR: Cannot find domain %s\n", bdata(bdomain));
@@ -554,8 +565,8 @@ cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
     {
         fprintf(stderr,
                 "WARN: Selected affinity domain %s has only %d hardware threads, but selection string evaluates to %d threads.\n",
-                bdata(affinity->domains[domainidx].tag), ret, require);
-        fprintf(stderr, "     This results in multiple threads on the same hardware thread.\n");
+                affinity->domains[domainidx].tag, ret, require);
+        fprintf(stderr, "      This results in multiple threads on the same hardware thread.\n");
         return 0;
     }
 logical_redo:
@@ -669,7 +680,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
         bdomain = bformat("N");
         blist = bstrcpy(bcpustr);
     }
-    domainidx = get_domain_idx(bdomain);
+    domainidx = get_domain_idx(bdata(bdomain));
     if (domainidx < 0)
     {
         fprintf(stderr, "Cannot find domain %s\n", bdata(bdomain));
@@ -677,7 +688,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
         bdestroy(blist);
         return 0;
     }
-    bstring domtag = affinity->domains[domainidx].tag;
+    char* domtag = affinity->domains[domainidx].tag;
 
     strlist = bsplit(blist, ',');
     int insert = 0;
@@ -720,7 +731,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
                                 notInCpuSet = 1;
                             }
                         }
-                        fprintf(stderr, "CPU %d not in domain %s.", j, bdata(domtag));
+                        fprintf(stderr, "CPU %d not in domain %s.", j, domtag);
                         if (notInCpuSet)
                         {
                             fprintf(stderr, " It is not in the given cpuset\n");
@@ -756,7 +767,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
                                 notInCpuSet = 1;
                             }
                         }
-                        fprintf(stderr, "CPU %d not in domain %s.", j, bdata(domtag));
+                        fprintf(stderr, "CPU %d not in domain %s.", j, domtag);
                         if (notInCpuSet)
                         {
                             fprintf(stderr, " It is not in the given cpuset\n");
@@ -797,7 +808,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
                         notInCpuSet = 1;
                     }
                 }
-                fprintf(stderr, "CPU %d not in domain %s.", cpu, bdata(domtag));
+                fprintf(stderr, "CPU %d not in domain %s.", cpu, domtag);
                 if (notInCpuSet)
                 {
                     fprintf(stderr, " It is not in the given cpuset\n");
@@ -864,7 +875,9 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
         else if ((bstrchrp(strlist->entry[i], 'N', 0) == 0) ||
                  (bstrchrp(strlist->entry[i], 'S', 0) == 0) ||
                  (bstrchrp(strlist->entry[i], 'C', 0) == 0) ||
-                 (bstrchrp(strlist->entry[i], 'M', 0) == 0))
+                 (bstrchrp(strlist->entry[i], 'M', 0) == 0) ||
+                 (bstrchrp(strlist->entry[i], 'D', 0) == 0) ||
+                 (bstrchrp(strlist->entry[i], 'G', 0) == 0))
         {
             if (bstrchrp(strlist->entry[i], ':', 0) != BSTR_ERR)
             {
@@ -876,7 +889,7 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
             }
             else
             {
-                int dom = get_domain_idx(strlist->entry[i]);
+                int dom = get_domain_idx(bdata(strlist->entry[i]));
                 if (dom >= 0)
                 {
                     AffinityDomains_t affinity = get_affinityDomains();
@@ -919,8 +932,8 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
     }
     free(tmpList);
     bdestroy(bcpustr);
-    bdestroy(balancedcheck);
     bdestroy(scattercheck);
+    bdestroy(balancedcheck);
     bstrListDestroy(strlist);
     return insert;
 }
@@ -949,4 +962,114 @@ sockstr_to_socklist(const char* sockstr, int* sockets, int length)
     return ret;
 }
 
+#ifdef LIKWID_WITH_NVMON
 
+static int valid_gpu_cuda(CudaTopology_t topo, int id)
+{
+    for (int i = 0; i < topo->numDevices; i++)
+    {
+        if (topo->devices[i].devid == id)
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int
+gpustr_to_gpulist_cuda(const char* gpustr, int* gpulist, int length)
+{
+    int insert = 0;
+    topology_cuda_init();
+    CudaTopology_t gpu_topology = get_cudaTopology();
+    bstring bgpustr = bfromcstr(gpustr);
+    struct bstrList* commalist = bsplit(bgpustr, ',');
+    for (int i = 0; i < commalist->qty; i++)
+    {
+        if (bstrchrp(commalist->entry[i], '-', 0) != BSTR_ERR)
+        {
+            struct bstrList* indexlist = bsplit(commalist->entry[i], '-');
+            int start = check_and_atoi(bdata(indexlist->entry[0]));
+            int end = check_and_atoi(bdata(indexlist->entry[1]));
+            if (start <= end)
+            {
+                for (int k = start; k <= end; k++)
+                {
+                    if (valid_gpu_cuda(gpu_topology, k) && insert < length)
+                    {
+                        gpulist[insert] = k;
+                        insert++;
+                    }
+                }
+            }
+        }
+        else
+        {
+            int id = check_and_atoi(bdata(commalist->entry[i]));
+            if (valid_gpu_cuda(gpu_topology, id) && insert < length)
+            {
+                gpulist[insert] = id;
+                insert++;
+            }
+        }
+    }
+    return insert;
+}
+
+#endif /* LIKWID_WITH_NVMON */
+
+#ifdef LIKWID_WITH_ROCMON
+
+static int valid_gpu_rocm(RocmTopology_t topo, int id)
+{
+    for (int i = 0; i < topo->numDevices; i++)
+    {
+        if (topo->devices[i].devid == id)
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int
+gpustr_to_gpulist_rocm(const char* gpustr, int* gpulist, int length)
+{
+    int insert = 0;
+    topology_rocm_init();
+    RocmTopology_t gpu_topology = get_rocmTopology();
+    bstring bgpustr = bfromcstr(gpustr);
+    struct bstrList* commalist = bsplit(bgpustr, ',');
+    for (int i = 0; i < commalist->qty; i++)
+    {
+        if (bstrchrp(commalist->entry[i], '-', 0) != BSTR_ERR)
+        {
+            struct bstrList* indexlist = bsplit(commalist->entry[i], '-');
+            int start = check_and_atoi(bdata(indexlist->entry[0]));
+            int end = check_and_atoi(bdata(indexlist->entry[1]));
+            if (start <= end)
+            {
+                for (int k = start; k <= end; k++)
+                {
+                    if (valid_gpu_rocm(gpu_topology, k) && insert < length)
+                    {
+                        gpulist[insert] = k;
+                        insert++;
+                    }
+                }
+            }
+        }
+        else
+        {
+            int id = check_and_atoi(bdata(commalist->entry[i]));
+            if (valid_gpu_rocm(gpu_topology, id) && insert < length)
+            {
+                gpulist[insert] = id;
+                insert++;
+            }
+        }
+    }
+    return insert;
+}
+
+#endif /* LIKWID_WITH_ROCMON */
