@@ -6,14 +6,14 @@
  *      Description:  Module implementing an interface for frequency manipulation
  *                    Module for manipuating CPU frequencies
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *                Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -319,7 +319,7 @@ freq_client_startDaemon()
     if (access(exeprog, X_OK))
     {
         fprintf(stderr, "Failed to find the daemon '%s'\n", exeprog);
-        exit(EXIT_FAILURE);
+        return -1;
     }
     DEBUG_PRINT(DEBUGLEV_INFO, Starting daemon %s, exeprog);
     pid = fork();
@@ -341,7 +341,7 @@ freq_client_startDaemon()
         {
             //ERRNO_PRINT;
             fprintf(stderr, "Failed to execute the daemon '%s'\n", exeprog);
-            exit(EXIT_FAILURE);
+            return -1;
         }
     }
     else if (pid < 0)
@@ -350,12 +350,27 @@ freq_client_startDaemon()
         return pid;
     }
 
-    EXIT_IF_ERROR(socket_fd = socket(AF_LOCAL, SOCK_STREAM, 0), socket() failed);
+    socket_fd = socket(AF_LOCAL, SOCK_STREAM, 0);
+    if (socket_fd < 0)
+    {
+        ERROR_PRINT(socket() failed);
+        return -1;
+    }
 
     address.sun_family = AF_LOCAL;
     address_length = sizeof(address);
     snprintf(address.sun_path, sizeof(address.sun_path), TOSTRING(LIKWIDSOCKETBASE) "-freq-%d", pid);
     filepath = strdup(address.sun_path);
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Waiting for socket file %s, address.sun_path);
+    while (access(address.sun_path, F_OK) && timeout > 0)
+    {
+        usleep(1000);
+        timeout--;
+    }
+    if (!access(address.sun_path, F_OK))
+    {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Socket file %s exists, address.sun_path);
+    }
 
     res = connect(socket_fd, (struct sockaddr *) &address, address_length);
     while (res && timeout > 0)
@@ -379,7 +394,10 @@ freq_client_startDaemon()
         fprintf(stderr, "opened within 10 seconds. Consult the error message above\n");
         fprintf(stderr, "this to find out why. If the error is 'no such file or directoy',\n");
         fprintf(stderr, "it usually means that likwid-accessD just failed to start.\n");
-        exit(EXIT_FAILURE);
+        free(filepath);
+        close(socket_fd);
+        socket_fd = -1;
+        return -1;
     }
     DEBUG_PRINT(DEBUGLEV_DEVELOP, Successfully opened socket %s to daemon, filepath);
     free(filepath);
@@ -1271,3 +1289,8 @@ int freq_setTurbo(const int cpu_id, const int turbo)
         return setIntelTurbo(cpu_id, turbo);
     return 1;
 }
+
+/*void __attribute__((destructor (104))) close_frequency_cpu(void)*/
+/*{*/
+/*    _freqFinalize();*/
+/*}*/
