@@ -5,13 +5,13 @@
  *
  *      Description:  Header File HPM access Module
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -30,6 +30,9 @@
 #ifndef ACCESS_H
 #define ACCESS_H
 
+#include <stdint.h>
+#include <pci_types.h>
+
 void HPMmode(int mode);
 int HPMinit(void);
 int HPMinitialized(void);
