@@ -5,14 +5,14 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@gmail.com
+ *                Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,21 +34,23 @@
 #include <types.h>
 #include <likwid.h>
 
-int socket_lock[MAX_NUM_NODES];
-int core_lock[MAX_NUM_THREADS];
-int tile_lock[MAX_NUM_THREADS];
-int numa_lock[MAX_NUM_THREADS];
-int sharedl2_lock[MAX_NUM_THREADS];
-int sharedl3_lock[MAX_NUM_THREADS];
+extern int *socket_lock;
+extern int *core_lock;
+extern int *tile_lock;
+extern int *numa_lock;
+extern int *die_lock;
+extern int *sharedl2_lock;
+extern int *sharedl3_lock;
 
 extern AffinityDomains affinityDomains;
 
 extern int *affinity_thread2core_lookup;
 extern int *affinity_thread2socket_lookup;
 extern int *affinity_thread2numa_lookup;
+extern int *affinity_thread2die_lookup;
 extern int *affinity_thread2sharedl3_lookup;
 extern int affinity_processGetProcessorId();
 extern int affinity_threadGetProcessorId();
-extern const AffinityDomain* affinity_getDomain(bstring domain);
+extern const AffinityDomain* affinity_getDomain(char* domain);
 
 #endif /*AFFINITY_H*/
