@@ -5,14 +5,14 @@
  *
  *      Description:  Central error handling macros
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@gmail.com
+ *                Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
 #define ERROR_H
 
 #include <likwid.h>
+#include <stdio.h>
 
 #define str(x) #x
 
@@ -45,16 +46,16 @@
    fprintf(stderr,  "ERROR - [%s:%s:%d] " str(msg) "\n", __FILE__, __func__,__LINE__);
 
 #define ERROR_PRINT(fmt, ...) \
-   fprintf(stderr,  "ERROR - [%s:%s:%d] %s.\n" str(fmt) "\n", __FILE__,  __func__,__LINE__, strerror(errno), __VA_ARGS__);
+   fprintf(stderr,  "ERROR - [%s:%s:%d] %s.\n" str(fmt) "\n", __FILE__,  __func__,__LINE__, strerror(errno), ##__VA_ARGS__);
 
 #define CHECK_ERROR(func, msg)  \
     if ((func) < 0) { \
-        fprintf(stderr, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno));  \
+        ERROR_PRINT(msg); \
     }
 
 #define CHECK_AND_RETURN_ERROR(func, msg)  \
     if ((func) < 0) { \
-        fprintf(stderr, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno));  \
+        ERROR_PRINT(msg); \
         return errno; \
     }
 
@@ -83,7 +84,19 @@
 
 #define DEBUG_PRINT(lev, fmt, ...) \
     if ((lev >= 0) && (lev <= perfmon_verbosity)) { \
-        fprintf(stdout, "DEBUG - [%s:%d] " str(fmt) "\n", __func__, __LINE__,__VA_ARGS__); \
+        fprintf(stdout, "DEBUG - [%s:%d] " str(fmt) "\n", __func__, __LINE__,##__VA_ARGS__); \
+        fflush(stdout); \
+    }
+
+#define GPUDEBUG_PRINT(lev, fmt, ...) \
+    if ((lev >= 0) && (lev <= likwid_nvmon_verbosity)) { \
+        fprintf(stdout, "DEBUG - [%s:%d] " str(fmt) "\n", __func__, __LINE__,##__VA_ARGS__); \
+        fflush(stdout); \
+    }
+
+#define ROCMON_DEBUG_PRINT(lev, fmt, ...) \
+    if ((lev >= 0) && (lev <= likwid_rocmon_verbosity)) { \
+        fprintf(stdout, "ROCMON DEBUG - [%s:%d] " str(fmt) "\n", __func__, __LINE__,##__VA_ARGS__); \
         fflush(stdout); \
     }
 
@@ -93,10 +106,32 @@
         fflush(stdout); \
     }
 
+#define INFO_PRINT(fmt, ...) \
+    if (perfmon_verbosity >= DEBUGLEV_INFO) \
+    { \
+        fprintf(stdout, "INFO - " STRINGIFY(fmt) "\n", ##__VA_ARGS__); \
+    }
+#define GPUINFO_PRINT(fmt, ...) \
+    if (likwid_nvmon_verbosity >= DEBUGLEV_INFO) \
+    { \
+        fprintf(stdout, "INFO - " STRINGIFY(fmt) "\n", ##__VA_ARGS__); \
+    }
+#define ROCMON_INFO_PRINT(fmt, ...) \
+    if (likwid_rocmon_verbosity >= DEBUGLEV_INFO) \
+    { \
+        fprintf(stdout, "ROCMON INFO - " STRINGIFY(fmt) "\n", ##__VA_ARGS__); \
+    }
+
+#define TODO_PRINT(fmt, ...)  \
+    fprintf(stdout, "TODO - " STRINGIFY(fmt) "\n", ##__VA_ARGS__);
+
+
 #define CHECK_MSR_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, MSR write operation failed);
 #define CHECK_MSR_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, MSR read operation failed);
 #define CHECK_PCI_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, PCI write operation failed);
 #define CHECK_PCI_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, PCI read operation failed);
+#define CHECK_MMIO_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, MMIO write operation failed);
+#define CHECK_MMIO_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, MMIO read operation failed);
 #define CHECK_POWER_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, Power register read operation failed);
 #define CHECK_TEMP_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, Temperature register read operation failed);
 
