@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for libperfctr module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,6 +31,7 @@
 #define LIBPERFCTR_H
 
 #include <bstrlib.h>
+#include <map.h>
 
 typedef enum LikwidThreadStates {
     MARKER_STATE_NEW,
@@ -52,6 +53,25 @@ typedef struct LikwidThreadResults{
     LikwidThreadStates state;
 } LikwidThreadResults;
 
+typedef struct {
+    int thread_id;
+    int cpu_id;
+    uint64_t last;
+    LikwidThreadResults* last_res;
+    Map_t regions;
+    uint64_t _padding[4];
+} GroupThreadsMap;
+
+typedef struct {
+    int numberOfThreads;
+    GroupThreadsMap *threads;
+} GroupThreads;
+
+typedef struct {
+    int numberOfGroups;
+    GroupThreads *groups;
+} MarkerGroups;
+
 typedef struct {
     bstring  tag;
     int groupID;
