@@ -5,13 +5,13 @@
  *
  *      Description:  Header File Locking primitive Module
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,10 +37,9 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <types.h>
 
 #define LOCK_INIT -1
-#define STRINGIFY(x) #x
-#define TOSTRING(x) STRINGIFY(x)
 
 static inline int
 lock_acquire(int* var, int newval)
