@@ -6,13 +6,13 @@
  *      Description:  Header File NUMA module for internal use. External functions are
  *                    defined in likwid.h
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,14 +38,20 @@
 #include <likwid.h>
 #include <numa_hwloc.h>
 #include <numa_proc.h>
+#include <numa_virtual.h>
 
 extern int numaInitialized;
 
 extern int str2int(const char* str);
 
+extern uint64_t proc_getFreeSysMem(void);
+
+extern uint64_t proc_getTotalSysMem(void);
+
 struct numa_functions {
     int (*numa_init) (void);
     void (*numa_setInterleaved) (const int*, int);
+    void (*numa_setMembind) (const int*, int);
     void (*numa_membind) (void*, size_t, int);
 };
 
