@@ -5,13 +5,13 @@
  *
  *      Description:  Header file of perfmon module for Intel Xeon Phi (Knights Landing)
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -1035,7 +1035,6 @@ int perfmon_stopCountersThread_knl(int thread_id, PerfmonEventSet* eventSet)
                     break;
             }
             eventSet->events[i].threadCounter[thread_id].counterData = field64(counter_result, 0, box_map[type].regWidth);
-            eventSet->events[i].threadCounter[thread_id].init = FALSE;
         }
     }
     return 0;
