@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of perfmon module for Intel Sandy Bridge.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -64,7 +64,7 @@ int perfmon_init_sandybridge(int cpu_id)
     lock_acquire((int*) &tile_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_PEBS_ENABLE, 0x0ULL));
     
-    if ((cpuid_info.model == SANDYBRIDGE_EP))
+    if (cpuid_info.model == SANDYBRIDGE_EP)
     {
         sandy_cbox_setup = snbep_cbox_setup;
         snb_did_cbox_test = 1;
@@ -868,6 +868,11 @@ int perfmon_setupCounterThread_sandybridge(
                 break;
 
             case MBOX0:
+                if (!cpuid_info.supportClientmem)
+                {
+                    snb_mbox_setup(cpu_id, index, event);
+                }
+                break;
             case MBOX1:
             case MBOX2:
             case MBOX3:
@@ -1070,6 +1075,25 @@ int perfmon_startCountersThread_sandybridge(int thread_id, PerfmonEventSet* even
                     break;
 
                 case MBOX0:
+                    if (!cpuid_info.supportClientmem)
+                    {
+                        if (haveLock && HPMcheck(dev, cpu_id))
+                        {
+                            CHECK_PCI_WRITE_ERROR(HPMwrite(cpu_id, dev, counter1, 0x0ULL));
+                            CHECK_PCI_WRITE_ERROR(HPMwrite(cpu_id, dev, counter2, 0x0ULL));
+                        }
+                    }
+                    else
+                    {
+                        if (haveLock)
+                        {
+                            uint64_t tmp = 0x0;
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &tmp));
+                            eventSet->events[i].threadCounter[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST tmp, READ_MBOX);
+                        }
+                    }
+                    break;
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
@@ -1341,8 +1365,22 @@ int perfmon_stopCountersThread_sandybridge(int thread_id, PerfmonEventSet* event
                     break;
 
                 case MBOX0:
-                    SNB_READ_PCI_BOX(MBOX0, dev, counter1, counter2);
-                    SNB_CHECK_OVERFLOW;
+                    if (!cpuid_info.supportClientmem)
+                    {
+                        SNB_READ_PCI_BOX(MBOX0, dev, counter1, counter2);
+                        SNB_CHECK_OVERFLOW;
+                    }
+                    else if (haveLock)
+                    {
+                        CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &counter_result));
+                        if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                        {
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                            eventSet->events[i].threadCounter[thread_id].overflows++;
+                        }
+                        *current = counter_result;
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
                     break;
 
                 case MBOX1:
@@ -1639,8 +1677,22 @@ int perfmon_readCountersThread_sandybridge(int thread_id, PerfmonEventSet* event
                     break;
 
                 case MBOX0:
-                    SNB_READ_PCI_BOX(MBOX0, dev, counter1, counter2);
-                    SNB_CHECK_OVERFLOW;
+                    if (!cpuid_info.supportClientmem)
+                    {
+                        SNB_READ_PCI_BOX(MBOX0, dev, counter1, counter2);
+                        SNB_CHECK_OVERFLOW;
+                    }
+                    else if (haveLock)
+                    {
+                        CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &counter_result));
+                        if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                        {
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                            eventSet->events[i].threadCounter[thread_id].overflows++;
+                        }
+                        *current = counter_result;
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
                     break;
 
                 case MBOX1:
@@ -1900,7 +1952,7 @@ int perfmon_finalizeCountersThread_sandybridge(int thread_id, PerfmonEventSet* e
                 break;
         }
         if ((reg) &&
-            (((type == PMC)||(type == FIXED)) || ((type >= UNCORE) && (haveLock) && (HPMcheck(dev, cpu_id)))))
+            (((type == PMC)||(type == FIXED)) || ((type >= UNCORE && type < NUM_UNITS) && (haveLock) && (HPMcheck(dev, cpu_id)))))
         {
             VERBOSEPRINTPCIREG(cpu_id, dev, reg, 0x0ULL, CLEAR_CTL);
             CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, dev, reg, 0x0ULL));
