@@ -7,14 +7,14 @@
  *                    Configures and reads out performance counters
  *                    on x86 based architectures. Supports multi threading.
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -88,9 +88,10 @@ typedef enum {
     EVENT_OPTION_PERF_FLAGS, /*!< \brief FLAGS parameters to use in the perf_event_open call */
 #endif
 #ifdef _ARCH_PPC
-    EVENT_OPTION_PMC,
+    EVENT_OPTION_PMC, /*!< \brief Specify which PMC counter should be used */
     EVENT_OPTION_PMCXSEL,
-    EVENT_OPTION_UNCORE_CONFIG, /*!< \brief Configuration bitmask for NEST (uncore) counters */
+    EVENT_OPTION_UNCORE_CONFIG, /*!< \brief Configuration bitmask for event ID for NEST (Uncore) measurements */
+
 #endif
     NUM_EVENT_OPTIONS /*!< \brief Amount of defined options */
 } EventOptionType;
@@ -241,6 +242,8 @@ typedef struct {
     uint64_t              regTypeMask2; /*!< \brief Bitmask2 for easy checks which types are included in the eventSet */
     uint64_t              regTypeMask3; /*!< \brief Bitmask3 for easy checks which types are included in the eventSet */
     uint64_t              regTypeMask4; /*!< \brief Bitmask4 for easy checks which types are included in the eventSet */
+    uint64_t              regTypeMask5; /*!< \brief Bitmask5 for easy checks which types are included in the eventSet */
+    uint64_t              regTypeMask6; /*!< \brief Bitmask6 for easy checks which types are included in the eventSet */
     GroupState            state; /*!< \brief Current state of the event group (configured, started, none) */
     GroupInfo             group; /*!< \brief Structure holding the performance group information */
 } PerfmonEventSet;
@@ -280,5 +283,13 @@ extern int perfmon_numCoreCounters;
 extern int perfmon_numUncoreCounters;
 extern int perfmon_numArchEvents;
 
+/*! \brief Structure holding information for Intel's uncore discovery mechanism introduced with SapphireRapids */
+typedef struct {
+    char* name; /*!< \brief Name of unit */
+    int discovery_type; /*!< \brief Intel's device discovery type */
+    int max_devices; /*!< \brief Maximal amont of devices for this discovery type */
+    PciDeviceIndex base_device; /*!< \brief LIKWID unit device base */
+} PerfmonUncoreDiscovery;
+
 
 #endif /*PERFMON_TYPES_H*/
