@@ -5,13 +5,13 @@
  *
  *      Description:  Header file of perfmon module for AMD Family 17 (ZEN)
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -42,6 +42,7 @@ int perfmon_init_zen(int cpu_id)
     lock_acquire((int*) &core_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &numa_lock[affinity_thread2numa_lookup[cpu_id]], cpu_id);
+    lock_acquire((int*) &die_lock[affinity_thread2die_lookup[cpu_id]], cpu_id);
     return 0;
 }
 
@@ -157,7 +158,7 @@ int k17_uncore_setup(int cpu_id, RegisterIndex index, PerfmonEvent* event)
 {
     uint64_t flags = 0x0ULL;
 
-    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] != cpu_id)
+    if (die_lock[affinity_thread2die_lookup[cpu_id]] != cpu_id)
     {
         return 0;
     }
@@ -225,7 +226,7 @@ int perfmon_startCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     int haveSLock = 0;
     int haveL3Lock = 0;
     int haveCLock = 0;
-    int haveMLock = 0;
+    int haveDLock = 0;
     uint64_t flags = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
@@ -233,9 +234,9 @@ int perfmon_startCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     {
         haveSLock = 1;
     }
-    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
     {
-        haveMLock = 1;
+        haveDLock = 1;
     }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
@@ -262,7 +263,7 @@ int perfmon_startCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
             eventSet->events[i].threadCounter[thread_id].startData = 0;
             eventSet->events[i].threadCounter[thread_id].counterData = 0;
             if ((type == PMC) ||
-                ((type == MBOX0) && (haveMLock)) ||
+                ((type == MBOX0) && (haveDLock)) ||
                 ((type == CBOX0) && (haveL3Lock)))
             {
                 VERBOSEPRINTREG(cpu_id, counter, LLU_CAST 0x0ULL, RESET_CTR);
@@ -301,7 +302,7 @@ int perfmon_stopCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     int haveSLock = 0;
     int haveL3Lock = 0;
     int haveCLock = 0;
-    int haveMLock = 0;
+    int haveDLock = 0;
     uint64_t counter_result = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
@@ -309,9 +310,9 @@ int perfmon_stopCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     {
         haveSLock = 1;
     }
-    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
     {
-        haveMLock = 1;
+        haveDLock = 1;
     }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
@@ -336,7 +337,7 @@ int perfmon_stopCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
             uint32_t reg = counter_map[index].configRegister;
             uint32_t counter = counter_map[index].counterRegister;
             if ((type == PMC) ||
-                ((type == MBOX0) && (haveMLock)) ||
+                ((type == MBOX0) && (haveDLock)) ||
                 ((type == CBOX0) && (haveL3Lock)))
             {
                 CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, reg, &flags));
@@ -390,7 +391,7 @@ int perfmon_readCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     int haveSLock = 0;
     int haveL3Lock = 0;
     int haveCLock = 0;
-    int haveMLock = 0;
+    int haveDLock = 0;
     uint64_t counter_result = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
@@ -398,9 +399,9 @@ int perfmon_readCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     {
         haveSLock = 1;
     }
-    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
     {
-        haveMLock = 1;
+        haveDLock = 1;
     }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
@@ -425,7 +426,7 @@ int perfmon_readCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
             uint32_t counter = counter_map[index].counterRegister;
             uint64_t* current = &(eventSet->events[i].threadCounter[thread_id].counterData);
             if ((type == PMC) ||
-                ((type == MBOX0) && (haveMLock)) ||
+                ((type == MBOX0) && (haveDLock)) ||
                 ((type == CBOX0) && (haveL3Lock)))
             {
                 CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, counter, &counter_result));
@@ -472,16 +473,16 @@ int perfmon_finalizeCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
 {
     int haveSLock = 0;
     int haveL3Lock = 0;
-    int haveMLock = 0;
+    int haveDLock = 0;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
     if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
     {
         haveSLock = 1;
     }
-    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
     {
-        haveMLock = 1;
+        haveDLock = 1;
     }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
@@ -497,7 +498,7 @@ int perfmon_finalizeCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
         }
         RegisterIndex index = eventSet->events[i].index;
         if ((type == PMC) ||
-            ((type == MBOX0) && (haveMLock)) ||
+            ((type == MBOX0) && (haveDLock)) ||
             ((type == CBOX0) && (haveL3Lock)))
         {
             if (counter_map[index].configRegister != 0x0)
