@@ -6,14 +6,14 @@
  *      Description:  Header File Power Module
  *                    Implements Intel RAPL Interface.
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -95,7 +95,7 @@ power_start(PowerData* data, int cpuId, PowerType type)
             uint64_t result = 0;
             data->before = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, power_regs[type], &result))
-            data->before = field64(result, 0, 32);
+            data->before = field64(result, 0, power_info.statusRegWidth);
             data->domain = type;
             return 0;
         }
@@ -122,7 +122,7 @@ power_stop(PowerData* data, int cpuId, PowerType type)
             uint64_t result = 0;
             data->after = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, power_regs[type], &result))
-            data->after = field64(result, 0, 32);
+            data->after = field64(result, 0, power_info.statusRegWidth);
             data->domain = type;
             return 0;
         }
@@ -147,7 +147,7 @@ power_read(int cpuId, uint64_t reg, uint32_t *data)
 
     if (power_info.hasRAPL)
     {
-        for (i = 0; i < NUM_POWER_DOMAINS; i++)
+        for (i = 0; i < power_info.numDomains; i++)
         {
             if (reg == power_regs[i])
             {
@@ -160,7 +160,7 @@ power_read(int cpuId, uint64_t reg, uint32_t *data)
             uint64_t result = 0;
             *data = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, reg, &result))
-            *data = field64(result, 0, 32);
+            *data = field64(result, 0, power_info.statusRegWidth);
             return 0;
         }
         else
@@ -196,7 +196,7 @@ power_tread(int socket_fd, int cpuId, uint64_t reg, uint32_t *data)
             uint64_t result = 0;
             *data = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, reg, &result))
-            *data = field64(result, 0, 32);
+            *data = field64(result, 0, power_info.statusRegWidth);
             return 0;
         }
         else
