@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of topology backend using cpuid instruction.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,9 +33,9 @@
 
 #include <sched.h>
 
-void cpuid_init_cpuInfo(cpu_set_t cpuSet);
-void cpuid_init_cpuFeatures(void);
-void cpuid_init_nodeTopology(cpu_set_t cpuSet);
-void cpuid_init_cacheTopology(void);
+int cpuid_init_cpuInfo(cpu_set_t cpuSet);
+int cpuid_init_cpuFeatures(void);
+int cpuid_init_nodeTopology(cpu_set_t cpuSet);
+int cpuid_init_cacheTopology(void);
 
 #endif /* TOPOLOGY_CPUID_H */
