@@ -5,13 +5,13 @@
  *
  *      Description:  Header File for hardcoded cache information
  *
- *      Version:   5.2.2
- *      Released:  26.07.2022
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -42,5 +42,10 @@ CacheLevel a64fx_caches[2] = {
     {2, DATACACHE, 16, 2048, 256, 8388608, 12, 1},
 };
 
+CacheLevel apple_m1_caches[2] = {
+    {1, DATACACHE, 4, 64, 256, 131072, 1, 1},
+    {2, DATACACHE, 16, 2048, 256, 12582912, 4, 1},
+};
+
 
 #endif
