@@ -5,14 +5,14 @@
  *
  *      Description:  Global  Types file
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -68,10 +68,23 @@ typedef struct {
 
 #define LLU_CAST  (unsigned long long)
 
+// borrowed from https://www.scs.stanford.edu/~dm/blog/va-opt.html
+#define PARENS ()
+#define EXPAND(...) EXPAND3(EXPAND3(EXPAND3(__VA_ARGS__)))
+#define EXPAND3(...) EXPAND2(EXPAND2(EXPAND2(__VA_ARGS__)))
+#define EXPAND2(...) EXPAND1(EXPAND1(EXPAND1(__VA_ARGS__)))
+#define EXPAND1(...) __VA_ARGS__
+#define FOR_EACH(macro, ...)                                                  \
+      __VA_OPT__(EXPAND(FOR_EACH_HELPER(macro, __VA_ARGS__)))
+#define FOR_EACH_HELPER(macro, a1, ...)                                       \
+      macro(a1) __VA_OPT__(FOR_EACH_AGAIN PARENS (macro, __VA_ARGS__))
+#define FOR_EACH_AGAIN() FOR_EACH_HELPER
 #define STRINGIFY(x) #x
-#define TOSTRING(x) STRINGIFY(x)
+#define TOSTRING(...) FOR_EACH(STRINGIFY, __VA_ARGS__)
 
 #define likely(x)       __builtin_expect(!!(x), 1)
 #define unlikely(x)     __builtin_expect(!!(x), 0)
 
+#define ARRAY_COUNT(arr) (sizeof(arr) / sizeof((arr)[0]))
+
 #endif /*TYPES_H*/
