@@ -5,14 +5,14 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -69,6 +69,7 @@ static int registered_cpus = 0;
 static pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
 static int use_locks = 0;
 static pthread_mutex_t threadLocks[MAX_NUM_THREADS] = { [ 0 ... (MAX_NUM_THREADS-1)] = PTHREAD_MUTEX_INITIALIZER};
+static int maxRegionNameLength = 100;
 
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
@@ -113,6 +114,10 @@ calculateMarkerResult(RegisterIndex index, uint64_t start, uint64_t stop, int ov
 {
     double result = 0.0;
     uint64_t maxValue = 0ULL;
+    if (start > stop)
+    {
+        overflows++;
+    }
     if (overflows == 0)
     {
         result = (double) (stop - start);
@@ -154,6 +159,7 @@ likwid_markerInit(void)
     bstring bEventStr;
     struct bstrList* threadTokens;
     struct bstrList* eventStrings;
+    cpu_set_t cpuset;
     char* modeStr = getenv("LIKWID_MODE");
     char* eventStr = getenv("LIKWID_EVENTS");
     char* cThreadStr = getenv("LIKWID_THREADS");
@@ -183,7 +189,54 @@ likwid_markerInit(void)
     if (!lock_check())
     {
         fprintf(stderr,"Access to performance counters is locked.\n");
-        exit(EXIT_FAILURE);
+        return;
+    }
+
+    if (debugStr != NULL)
+    {
+        verbosity = atoi(debugStr);
+        perfmon_setVerbosity(verbosity);
+    }
+
+    bThreadStr = bfromcstr(cThreadStr);
+    threadTokens = bsplit(bThreadStr,',');
+    num_cpus = threadTokens->qty;
+    for (i = 0; i < num_cpus; i++)
+    {
+        threads2Cpu[i] = ownatoi(bdata(threadTokens->entry[i]));
+    }
+    bdestroy(bThreadStr);
+    bstrListDestroy(threadTokens);
+
+    CPU_ZERO(&cpuset);
+    if (pinStr != NULL)
+    {
+        // If LIKWID should pin the threads, we check whether we are already
+        // in a restricted cpuset with a single hwthread. This happens
+        // when an OpenMP region is executed before LIKWID_MARKER_INIT.
+        // When the first thread starts up, LIKWID pins the master process
+        // and the thread. But we want to get the topology without the restriction
+        // here. We don't need to set it back because the first operation
+        // after getting the topology is the pinning of the master process.
+        int err = sched_getaffinity(0, sizeof(cpu_set_t), &cpuset);
+        if (err == 0)
+        {
+            int count = CPU_COUNT(&cpuset);
+            if (count == 1 && count != num_cpus)
+            {
+                cpu_set_t newcpuset;
+                CPU_ZERO(&newcpuset);
+                for (i = 0; i < num_cpus; i++)
+                {
+                    CPU_SET(threads2Cpu[i], &newcpuset);
+                }
+                err = sched_setaffinity(0, sizeof(cpu_set_t), &newcpuset);
+            }
+        }
+        else
+        {
+            fprintf(stderr,"Failed to get cpuset for master thread.\n");
+        }
     }
 
     topology_init();
@@ -194,21 +247,6 @@ likwid_markerInit(void)
 #ifndef LIKWID_USE_PERFEVENT
     HPMmode(atoi(modeStr));
 #endif
-    if (debugStr != NULL)
-    {
-        perfmon_verbosity = atoi(debugStr);
-        verbosity = perfmon_verbosity;
-    }
-
-    bThreadStr = bfromcstr(cThreadStr);
-    threadTokens = bsplit(bThreadStr,',');
-    num_cpus = threadTokens->qty;
-    for (i=0; i<num_cpus; i++)
-    {
-        threads2Cpu[i] = ownatoi(bdata(threadTokens->entry[i]));
-    }
-    bdestroy(bThreadStr);
-    bstrListDestroy(threadTokens);
 
     if (pinStr != NULL)
     {
@@ -256,7 +294,9 @@ likwid_markerInit(void)
     {
         fprintf(stderr,"Cannot allocate space for group handling.\n");
         bstrListDestroy(eventStrings);
-        exit(EXIT_FAILURE);
+        bdestroy(bEventStr);
+        perfmon_finalize();
+        return;
     }
     for (i=0; i<eventStrings->qty; i++)
     {
@@ -490,8 +530,134 @@ likwid_markerClose(void)
     {
         free(results);
     }
-    likwid_init = 0;
+    perfmon_finalize();
     HPMfinalize();
+    likwid_init = 0;
+}
+
+int
+likwid_markerWriteFile(const char* markerfile)
+{
+    if (markerfile == NULL)
+    {
+        fprintf(stderr, "File can not be NULL.\n");
+        return -EFAULT;
+    }
+
+    FILE *file = NULL;
+    int* validRegions = NULL;
+
+    int numberOfThreads = perfmon_getNumberOfThreads();
+    int numberOfRegions = perfmon_getNumberOfRegions();
+
+    if ( ! likwid_init )
+    {
+        return -EFAULT;;
+    }
+    if ((numberOfThreads == 0)||(numberOfRegions == 0))
+    {
+        fprintf(stderr, "No threads or regions defined in hash table\n");
+        return -EFAULT;
+    }
+
+    file = fopen(markerfile,"w");
+    if (file != NULL)
+    {
+        int newNumberOfRegions = 0;
+        int newRegionID = 0;
+        validRegions = (int*)malloc(numberOfRegions*sizeof(int));
+        if (!validRegions)
+        {
+            return -EFAULT;
+        }
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            validRegions[i] = 0;
+        }
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            for (int j=0; j<numberOfThreads; j++)
+            {
+                validRegions[i] += perfmon_getCountOfRegion(i, j);
+            }
+            if (validRegions[i] > 0)
+                newNumberOfRegions++;
+            else
+                fprintf(stderr, "WARN: Skipping region %s for evaluation.\n", perfmon_getTagOfRegion(i));
+        }
+        if (newNumberOfRegions < numberOfRegions)
+        {
+            fprintf(stderr, "WARN: Regions are skipped because:\n");
+            fprintf(stderr, "      - The region was only registered\n");
+            fprintf(stderr, "      - The region was started but never stopped\n");
+            fprintf(stderr, "      - The region was never started but stopped\n");
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP,
+                Creating Marker file %s with %d regions %d groups and %d threads,
+                markerfile, newNumberOfRegions, numberOfGroups, numberOfThreads);
+        bstring thread_regs_grps = bformat("%d %d %d", numberOfThreads, newNumberOfRegions, numberOfGroups);
+        fprintf(file,"%s\n", bdata(thread_regs_grps));
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(thread_regs_grps));
+        bdestroy(thread_regs_grps);
+
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            if (validRegions[i] == 0)
+                continue;
+            bstring tmp = bformat("%d:%s", newRegionID, perfmon_getTagOfRegion(i));
+            fprintf(file,"%s\n", bdata(tmp));
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(tmp));
+            bdestroy(tmp);
+            newRegionID++;
+        }
+        int *cpulist = (int*) malloc(numberOfThreads * sizeof(int));
+        if (cpulist == NULL)
+        {
+            fprintf(stderr, "Failed to allocate %lu bytes for the cpulist storage\n",
+                        numberOfThreads * sizeof(int));
+            free(validRegions);
+            return -EFAULT;
+        }
+        newRegionID = 0;
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            if (validRegions[i] == 0)
+                continue;
+            int groupID = perfmon_getGroupOfRegion(i);
+            int nevents = groupSet->groups[groupID].numberOfEvents;
+            int cpulist_size = perfmon_getCpulistOfRegion(i, numberOfThreads, cpulist);
+            for (int j=0; j<numberOfThreads; j++)
+            {
+                int count = perfmon_getCountOfRegion(i, j);
+                double time = perfmon_getTimeOfRegion(i, j);
+                bstring l = bformat("%d %d %d %u %e %d ", newRegionID,
+                                                          groupID,
+                                                          cpulist[j],
+                                                          count,
+                                                          time,
+                                                          nevents);
+
+                for (int k=0; k < MIN(nevents, NUM_PMC); k++)
+                {
+                    bstring tmp = bformat("%e ", perfmon_getResultOfRegionThread(i, k, j));
+                    bconcat(l, tmp);
+                    bdestroy(tmp);
+                }
+                fprintf(file,"%s\n", bdata(l));
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(l));
+                bdestroy(l);
+            }
+            newRegionID++;
+        }
+        fclose(file);
+        free(validRegions);
+        free(cpulist);
+    }
+    else
+    {
+        fprintf(stderr, "Cannot open file %s\n", markerfile);
+        fprintf(stderr, "%s", strerror(errno));
+    }
 }
 
 int
@@ -504,13 +670,15 @@ likwid_markerRegisterRegion(const char* regionTag)
     TimerData timer;
     int ret = 0;
     uint64_t tmp = 0x0ULL;
-    bstring tag = bfromcstralloc(100, regionTag);
-    LikwidThreadResults* results;
-    char groupSuffix[10];
-    sprintf(groupSuffix, "-%d", groupSet->activeGroup);
-    bcatcstr(tag, groupSuffix);
+    LikwidThreadResults* results = NULL;
+    bstring tag = bformat("%.*s-%d", 100, regionTag, groupSet->activeGroup);
     int cpu_id = hashTable_get(tag, &results);
     bdestroy(tag);
+    if (!results)
+    {
+        fprintf(stderr, "ERROR: Failed to get thread data for tag %s\n", regionTag);
+        return -EFAULT;
+    }
 
 #ifndef LIKWID_USE_PERFEVENT
     // Add CPU to access layer if ACCESSMODE is direct or accessdaemon
@@ -525,6 +693,7 @@ likwid_markerRegisterRegion(const char* regionTag)
 int
 likwid_markerStartRegion(const char* regionTag)
 {
+    LikwidThreadResults* results = NULL;
     if ( ! likwid_init )
     {
         return -EFAULT;
@@ -535,13 +704,13 @@ likwid_markerStartRegion(const char* regionTag)
         return -EFAULT;
     }
 
-    bstring tag = bfromcstralloc(100, regionTag);
-    LikwidThreadResults* results;
-    char groupSuffix[10];
-    sprintf(groupSuffix, "-%d", groupSet->activeGroup);
-    bcatcstr(tag, groupSuffix);
-
+    bstring tag = bformat("%.*s-%d", 100, regionTag, groupSet->activeGroup);
     int cpu_id = hashTable_get(tag, &results);
+    if (!results)
+    {
+        fprintf(stderr, "ERROR: Failed to get thread data for tag %s\n", regionTag);
+        return -EFAULT;
+    }
     int thread_id = getThreadID(cpu_id);
     if (results->state == MARKER_STATE_START)
     {
@@ -584,6 +753,7 @@ likwid_markerStopRegion(const char* regionTag)
     }
 
     TimerData timestamp;
+    LikwidThreadResults* results = NULL;
     timer_stop(&timestamp);
     double result = 0.0;
     int cpu_id;
@@ -593,24 +763,29 @@ likwid_markerStopRegion(const char* regionTag)
         return -EFAULT;
     }
     int thread_id;
-    bstring tag = bfromcstr(regionTag);
-    char groupSuffix[100];
-    LikwidThreadResults* results;
-    sprintf(groupSuffix, "-%d", groupSet->activeGroup);
-    bcatcstr(tag, groupSuffix);
+    bstring tag = bformat("%.*s-%d", 100, regionTag, groupSet->activeGroup);
     if (use_locks == 1)
     {
         pthread_mutex_lock(&threadLocks[myCPU]);
     }
 
     cpu_id = hashTable_get(tag, &results);
+    if (!results)
+    {
+        fprintf(stderr, "ERROR: Failed to get thread data for tag %s\n", regionTag);
+        if (use_locks == 1)
+        {
+            pthread_mutex_unlock(&threadLocks[myCPU]);
+        }
+        return -EFAULT;
+    }
     thread_id = getThreadID(cpu_id);
     if (results->state != MARKER_STATE_START)
     {
         fprintf(stderr, "WARN: Stopping an unknown/not-started region %s\n", regionTag);
         if (use_locks == 1)
         {
-            pthread_mutex_lock(&threadLocks[myCPU]);
+            pthread_mutex_unlock(&threadLocks[myCPU]);
         }
         return -EFAULT;
     }
@@ -626,12 +801,12 @@ likwid_markerStopRegion(const char* regionTag)
     {
         if (groupSet->groups[groupSet->activeGroup].events[i].type != NOTYPE)
         {
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu, regionTag, thread_id, cpu_id, i,
-                            LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
             result = calculateMarkerResult(groupSet->groups[groupSet->activeGroup].events[i].index, results->StartPMcounters[i],
                                             groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
                                             groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows -
                                             results->StartOverflows[i]);
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu DIFF %f, regionTag, thread_id, cpu_id, i,
+                            LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData, result);
             if ((counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL) &&
                 (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != VOLTAGE) &&
                 (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != MBOX0TMP))
@@ -675,13 +850,15 @@ likwid_markerGetRegion(
     int cpu_id;
     int myCPU = likwid_getProcessorId();
     int thread_id;
-    bstring tag = bfromcstr(regionTag);
-    char groupSuffix[100];
-    LikwidThreadResults* results;
-    sprintf(groupSuffix, "-%d", groupSet->activeGroup);
-    bcatcstr(tag, groupSuffix);
+    LikwidThreadResults* results = NULL;
+    bstring tag = bformat("%.*s-%d", 100, regionTag, groupSet->activeGroup);
 
     cpu_id = hashTable_get(tag, &results);
+    if (!results)
+    {
+        fprintf(stderr, "ERROR: Failed to get thread data for tag %s\n", regionTag);
+        return;
+    }
     thread_id = getThreadID(myCPU);
     if (count != NULL)
     {
@@ -708,6 +885,7 @@ likwid_markerGetRegion(
 int
 likwid_markerResetRegion(const char* regionTag)
 {
+    LikwidThreadResults* results = NULL;
     if (! likwid_init)
     {
         return -EFAULT;
@@ -718,13 +896,14 @@ likwid_markerResetRegion(const char* regionTag)
     {
         return -EFAULT;
     }
-    bstring tag = bfromcstr(regionTag);
-    char groupSuffix[100];
-    LikwidThreadResults* results;
-    sprintf(groupSuffix, "-%d", groupSet->activeGroup);
-    bcatcstr(tag, groupSuffix);
+    bstring tag = bformat("%.*s-%d", 100, regionTag, groupSet->activeGroup);
 
     cpu_id = hashTable_get(tag, &results);
+    if (!results)
+    {
+        fprintf(stderr, "ERROR: Failed to get thread data for tag %s\n", regionTag);
+        return -EFAULT;
+    }
     if (results->state != MARKER_STATE_STOP)
     {
         fprintf(stderr, "ERROR: Can only reset stopped regions\n");
