@@ -5,14 +5,14 @@
  *
  *      Description: F90 interface for marker API
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com,
  *               Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,24 @@ likwid_markerclose_(void)
     likwid_markerClose();
 }
 
+void __attribute__ ((visibility ("default") ))
+likwid_markerwritefile_(char* markerfile, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, markerfile, len * sizeof(char));
+
+    for (int i = (len - 1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i + 1] = 0;
+            break;
+        }
+    }
+
+    likwid_markerWriteFile(tmp);
+    free(tmp);
+}
+
 void __attribute__ ((visibility ("default") ))
 likwid_markernextgroup_(void)
 {
@@ -162,20 +180,20 @@ likwid_markerresetregion_(char* regionTag, int len)
 void __attribute__ ((visibility ("default") ))
 likwid_nvmarkerinit_(void)
 {
-    likwid_gpuMarkerInit();
+    nvmon_markerInit();
 }
 
 
 void __attribute__ ((visibility ("default") ))
 likwid_nvmarkerclose_(void)
 {
-    likwid_gpuMarkerClose();
+    nvmon_markerClose();
 }
 
 void __attribute__ ((visibility ("default") ))
 likwid_nvmarkernextgroup_(void)
 {
-    likwid_markerNextGroup();
+    nvmon_markerNextGroup();
 }
 
 void __attribute__ ((visibility ("default") ))
@@ -192,7 +210,7 @@ likwid_nvmarkerregisterregion_(char* regionTag, int len)
         }
     }
 
-    likwid_gpuMarkerRegisterRegion( tmp );
+    nvmon_markerRegisterRegion( tmp );
     free(tmp);
 }
 
@@ -210,7 +228,7 @@ likwid_nvmarkerstartregion_(char* regionTag, int len)
         }
     }
 
-    likwid_gpuMarkerStartRegion( tmp );
+    nvmon_markerStartRegion( tmp );
     free(tmp);
 }
 
@@ -228,7 +246,7 @@ likwid_nvmarkerstopregion_(char* regionTag, int len)
         }
     }
 
-    likwid_gpuMarkerStopRegion( tmp );
+    nvmon_markerStopRegion( tmp );
     free(tmp);
 }
 
@@ -245,7 +263,133 @@ likwid_nvmarkerresetregion_(char* regionTag, int len)
             break;
         }
     }
-    likwid_gpuMarkerResetRegion( tmp);
+    nvmon_markerResetRegion( tmp);
     free(tmp);
 }
-#endif
\ No newline at end of file
+
+int __attribute__ ((visibility ("default") ))
+likwid_nvmarkerwritefile_(char* filename, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, filename, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+    nvmon_markerWriteFile(tmp);
+    free(tmp);
+}
+#endif
+
+#ifdef LIKWID_WITH_ROCMON
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerinit_(void)
+{
+    rocmon_markerInit();
+}
+
+
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerclose_(void)
+{
+    rocmon_markerClose();
+}
+
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkernextgroup_(void)
+{
+    rocmon_markerNextGroup();
+}
+
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerregisterregion_(char* regionTag, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char) );
+    strncpy(tmp, regionTag, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+
+    rocmon_markerRegisterRegion( tmp );
+    free(tmp);
+}
+
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerstartregion_(char* regionTag, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char) );
+    strncpy(tmp, regionTag, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+
+    rocmon_markerStartRegion( tmp );
+    free(tmp);
+}
+
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerstopregion_(char* regionTag, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, regionTag, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+
+    rocmon_markerStopRegion( tmp );
+    free(tmp);
+}
+
+void __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerresetregion_(char* regionTag, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, regionTag, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+    rocmon_markerResetRegion( tmp);
+    free(tmp);
+}
+
+int __attribute__ ((visibility ("default") ))
+likwid_rocmmarkerwritefile_(char* filename, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, filename, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+    rocmon_markerWriteFile(tmp);
+    free(tmp);
+}
+#endif /* LIKWID_WITH_ROCMON */
