@@ -5,13 +5,13 @@
  *
  *      Description:  Interface to hwloc for NUMA topology
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
- *      Author:  Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:  Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -73,46 +73,19 @@ getFreeNodeMem(int nodeId)
                  free = str2int(bdata(subtokens->entry[0]));
                  bdestroy(tmp);
                  bstrListDestroy(subtokens);
+                 break;
             }
         }
         bstrListDestroy(tokens);
         bdestroy(src);
         fclose(fp);
     }
-    else if (!access("/proc/meminfo", R_OK))
-    {
-        bdestroy(filename);
-        filename = bfromcstr("/proc/meminfo");
-        if (NULL != (fp = fopen (bdata(filename), "r")))
-        {
-            bstring src = bread ((bNread) fread, fp);
-            struct bstrList* tokens = bsplit(src,(char) '\n');
-            for (i=0;i<tokens->qty;i++)
-            {
-                if (binstr(tokens->entry[i],0,freeString) != BSTR_ERR)
-                {
-                     bstring tmp = bmidstr (tokens->entry[i], 10, blength(tokens->entry[i])-10  );
-                     bltrimws(tmp);
-                     struct bstrList* subtokens = bsplit(tmp,(char) ' ');
-                     free = str2int(bdata(subtokens->entry[0]));
-                     bdestroy(tmp);
-                     bstrListDestroy(subtokens);
-                }
-            }
-            bstrListDestroy(tokens);
-            bdestroy(src);
-            fclose(fp);
-        }
-    }
-    else
-    {
-        bdestroy(freeString);
-        bdestroy(filename);
-        ERROR;
-    }
+
     bdestroy(freeString);
     bdestroy(filename);
-    return free;
+
+    /* Fallback to system-wide free memory */
+    return (free > 0 ? free : proc_getFreeSysMem());
 }
 
 uint64_t
@@ -141,47 +114,20 @@ getTotalNodeMem(int nodeId)
                  total = str2int(bdata(subtokens->entry[0]));
                  bdestroy(tmp);
                  bstrListDestroy(subtokens);
+                 break;
             }
         }
         bstrListDestroy(tokens);
         bdestroy(src);
         fclose(fp);
     }
-    else if (!access(sptr, R_OK))
-    {
-        if (NULL != (fp = fopen (bdata(procfilename), "r")))
-        {
-            bstring src = bread ((bNread) fread, fp);
-            struct bstrList* tokens = bsplit(src,(char) '\n');
-            for (i=0;i<tokens->qty;i++)
-            {
-                if (binstr(tokens->entry[i],0,totalString) != BSTR_ERR)
-                {
-                     bstring tmp = bmidstr (tokens->entry[i], 10, blength(tokens->entry[i])-10  );
-                     bltrimws(tmp);
-                     struct bstrList* subtokens = bsplit(tmp,(char) ' ');
-                     total = str2int(bdata(subtokens->entry[0]));
-                     bdestroy(tmp);
-                     bstrListDestroy(subtokens);
-                }
-            }
-            bstrListDestroy(tokens);
-            bdestroy(src);
-            fclose(fp);
-        }
-    }
-    else
-    {
-        bdestroy(totalString);
-        bdestroy(sysfilename);
-        bdestroy(procfilename);
-        ERROR;
-    }
 
     bdestroy(totalString);
     bdestroy(sysfilename);
     bdestroy(procfilename);
-    return total;
+
+    /* Fallback to system-wide total memory */
+    return (total > 0 ? total : proc_getTotalSysMem());
 }
 
 int
@@ -189,11 +135,11 @@ likwid_hwloc_findProcessor(int nodeID, int cpuID)
 {
     hwloc_obj_t obj;
     int i;
-    int pu_count = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
+    int pu_count = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, HWLOC_OBJ_PU);
 
     for (i=0; i<pu_count; i++)
     {
-        obj = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PU, i);
+        obj = LIKWID_HWLOC_NAME(get_obj_by_type)(hwloc_topology, HWLOC_OBJ_PU, i);
         if (!obj)
         {
             continue;
@@ -216,68 +162,44 @@ hwloc_numa_init(void)
 {
     int errno;
     uint32_t i;
-    int d;
+    int d, j;
     int depth;
     int cores_per_socket;
     int numPUs = 0;
     hwloc_obj_t obj;
     const struct hwloc_distances_s* distances;
+    struct hwloc_distances_s* dists = NULL;
+    unsigned dist_count = 1;
+#if HWLOC_API_VERSION > 0x00020000
+    hwloc_obj_type_t hwloc_type = HWLOC_OBJ_NUMANODE;
+#else
     hwloc_obj_type_t hwloc_type = HWLOC_OBJ_NODE;
+#endif
     if (numaInitialized > 0 || numa_info.numberOfNodes > 0)
         return 0;
 
     if (!hwloc_topology)
     {
-        likwid_hwloc_topology_init(&hwloc_topology);
-        likwid_hwloc_topology_load(hwloc_topology);
+        LIKWID_HWLOC_NAME(topology_init)(&hwloc_topology);
+#if HWLOC_API_VERSION > 0x00020000
+        LIKWID_HWLOC_NAME(topology_set_flags)(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM );
+#else
+        LIKWID_HWLOC_NAME(topology_set_flags)(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM|HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
+#endif
+        LIKWID_HWLOC_NAME(topology_load)(hwloc_topology);
     }
-
-    numa_info.numberOfNodes = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, hwloc_type);
-    numPUs = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
-
+#if defined(__x86_64) || defined(__i386__)
+    numa_info.numberOfNodes = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, hwloc_type);
+#endif
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A__)
+    numa_info.numberOfNodes = 0;
+#endif
+    numPUs = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, HWLOC_OBJ_PU);
     /* If the amount of NUMA nodes == 0, there is actually no NUMA node, hence
        aggregate all sockets in the system into the single virtually created NUMA node */
     if (numa_info.numberOfNodes == 0)
     {
-        hwloc_type = HWLOC_OBJ_SOCKET;
-        numa_info.numberOfNodes = 1;
-
-        numa_info.nodes = (NumaNode*) malloc(sizeof(NumaNode));
-        if (!numa_info.nodes)
-        {
-            fprintf(stderr,"No memory to allocate %ld byte for nodes array\n",sizeof(NumaNode));
-            return -1;
-        }
-        numa_info.nodes[0].id = 0;
-        numa_info.nodes[0].numberOfProcessors = 0;
-        numa_info.nodes[0].totalMemory = getTotalNodeMem(0);
-        numa_info.nodes[0].freeMemory = getFreeNodeMem(0);
-        numa_info.nodes[0].processors = (uint32_t*) malloc(numPUs * sizeof(uint32_t));
-        if (!numa_info.nodes[0].processors)
-        {
-            fprintf(stderr,"No memory to allocate %ld byte for processors array of NUMA node %d\n",
-                    numPUs * sizeof(uint32_t),0);
-            return -1;
-        }
-        numa_info.nodes[0].distances = (uint32_t*) malloc(sizeof(uint32_t));
-        if (!numa_info.nodes[0].distances)
-        {
-            fprintf(stderr,"No memory to allocate %ld byte for distances array of NUMA node %d\n",
-                    sizeof(uint32_t),0);
-            return -1;
-        }
-        numa_info.nodes[0].distances[0] = 10;
-        numa_info.nodes[0].numberOfDistances = 1;
-        cores_per_socket = cpuid_topology.numHWThreads/cpuid_topology.numSockets;
-
-        for (d=0; d<likwid_hwloc_get_nbobjs_by_type(hwloc_topology, hwloc_type); d++)
-        {
-            obj = likwid_hwloc_get_obj_by_type(hwloc_topology, hwloc_type, d);
-            /* depth is here used as index in the processors array */
-            depth = d * cores_per_socket;
-            numa_info.nodes[0].numberOfProcessors += likwid_hwloc_record_objs_of_type_below_obj(
-                    likwid_hwloc_topology, obj, HWLOC_OBJ_PU, &depth, &numa_info.nodes[0].processors);
-        }
+        return virtual_numa_init();
     }
     else
     {
@@ -288,14 +210,33 @@ hwloc_numa_init(void)
                     numa_info.numberOfNodes * sizeof(NumaNode));
             return -1;
         }
-        depth = likwid_hwloc_get_type_depth(hwloc_topology, hwloc_type);
-        distances = likwid_hwloc_get_whole_distance_matrix_by_type(hwloc_topology, hwloc_type);
+        depth = LIKWID_HWLOC_NAME(get_type_depth)(hwloc_topology, hwloc_type);
+#if HWLOC_API_VERSION > 0x00020000
+        errno = LIKWID_HWLOC_NAME(distances_get_by_type)(hwloc_topology, hwloc_type, &dist_count, &dists, HWLOC_DISTANCES_KIND_FROM_OS, 0);
+#else
+        distances = LIKWID_HWLOC_NAME(get_whole_distance_matrix_by_type)(hwloc_topology, hwloc_type);
+#endif
         for (i=0; i<numa_info.numberOfNodes; i++)
         {
-            obj = likwid_hwloc_get_obj_by_depth(hwloc_topology, depth, i);
+            obj = LIKWID_HWLOC_NAME(get_obj_by_depth)(hwloc_topology, depth, i);
 
             numa_info.nodes[i].id = obj->os_index;
-
+#if HWLOC_API_VERSION > 0x00020000
+            if (obj->attr->numanode.local_memory != 0)
+            {
+                numa_info.nodes[i].totalMemory = (uint64_t)(obj->attr->numanode.local_memory/1024);
+            }
+            else if (obj->attr->numanode.page_types_len != 0)
+            {
+                numa_info.nodes[i].totalMemory = 0;
+                for (int k = 0; k < obj->attr->numanode.page_types_len; k++)
+                {
+                    uint64_t size = obj->attr->numanode.page_types[k].size;
+                    uint64_t count = obj->attr->numanode.page_types[k].count;
+                    numa_info.nodes[i].totalMemory += (uint64_t)((size*count)/1024);
+                }
+            }
+#else
             if (obj->memory.local_memory != 0)
             {
                 numa_info.nodes[i].totalMemory = (uint64_t)(obj->memory.local_memory/1024);
@@ -304,6 +245,7 @@ hwloc_numa_init(void)
             {
                 numa_info.nodes[i].totalMemory = (uint64_t)(obj->memory.total_memory/1024);
             }
+#endif
             else
             {
                 numa_info.nodes[i].totalMemory = getTotalNodeMem(numa_info.nodes[i].id);
@@ -318,8 +260,33 @@ hwloc_numa_init(void)
                 return -1;
             }
             d = 0;
+            j = 0;
+            // call before hwloc update
+
+#if HWLOC_API_VERSION > 0x00020000
+            for (d = 0; d < cpuid_topology.numHWThreads; d++)
+            {
+                HWThread *t = &cpuid_topology.threadPool[d];
+                if (LIKWID_HWLOC_NAME(bitmap_isset)(obj->cpuset, t->apicId) && t->threadId == 0)
+                {
+                    numa_info.nodes[i].processors[j] = t->apicId;
+                    j++;
+                    for (int k = 0; k < cpuid_topology.numHWThreads; k++)
+                    {
+                        HWThread *x = &cpuid_topology.threadPool[k];
+                        if (t->coreId == x->coreId && t->threadId != x->threadId && LIKWID_HWLOC_NAME(bitmap_isset)(obj->cpuset, x->apicId))
+                        {
+                            numa_info.nodes[i].processors[j] = x->apicId;
+                            j++;
+                        }
+                    }
+                }
+            }
+            numa_info.nodes[i].numberOfProcessors = j;
+#else
             numa_info.nodes[i].numberOfProcessors = likwid_hwloc_record_objs_of_type_below_obj(
-                    hwloc_topology, obj, HWLOC_OBJ_PU, &d, &numa_info.nodes[i].processors);
+                        hwloc_topology, obj, HWLOC_OBJ_PU, &d, &numa_info.nodes[i].processors);
+#endif
             numa_info.nodes[i].distances = (uint32_t*) malloc(numa_info.numberOfNodes * sizeof(uint32_t));
             if (!numa_info.nodes[i].distances)
             {
@@ -327,6 +294,19 @@ hwloc_numa_init(void)
                         numa_info.numberOfNodes*sizeof(uint32_t),i);
                 return -1;
             }
+#if HWLOC_API_VERSION > 0x00020000
+            if (dists)
+            {
+                int base = LIKWID_HWLOC_NAME(distances_obj_index)(dists, obj);
+                for (d = 0; d < dists->nbobjs; d++)
+                {
+                    hwloc_obj_t dobj = dists->objs[d];
+                    int idx = LIKWID_HWLOC_NAME(distances_obj_index)(dists, dobj);
+                    numa_info.nodes[i].distances[idx] = dists->values[(base*dists->nbobjs)+idx];
+                }
+                numa_info.nodes[i].numberOfDistances = numa_info.numberOfNodes;
+            }
+#else
             if (distances)
             {
                 numa_info.nodes[i].numberOfDistances = distances->nbobjs;
@@ -335,16 +315,22 @@ hwloc_numa_init(void)
                     numa_info.nodes[i].distances[d] = distances->latency[i*distances->nbobjs + d] * distances->latency_base;
                 }
             }
+#endif
             else
             {
                 numa_info.nodes[i].numberOfDistances = numa_info.numberOfNodes;
-                for(d=0;d<numa_info.numberOfNodes;d++)
+                for(d = 0; d < numa_info.numberOfNodes; d++)
                 {
                     numa_info.nodes[i].distances[d] = 10;
                 }
             }
-
         }
+#if HWLOC_API_VERSION > 0x00020000
+        if (dists)
+        {
+            LIKWID_HWLOC_NAME(distances_release_remove)(hwloc_topology, dists);
+        }
+#endif
     }
 
     if (numa_info.nodes[0].numberOfProcessors == 0)
@@ -362,12 +348,18 @@ void
 hwloc_numa_membind(void* ptr, size_t size, int domainId)
 {
     int ret = 0;
-    hwloc_membind_flags_t flags = HWLOC_MEMBIND_STRICT|HWLOC_MEMBIND_PROCESS;
-    hwloc_nodeset_t nodeset = likwid_hwloc_bitmap_alloc();
-    likwid_hwloc_bitmap_zero(nodeset);
-    likwid_hwloc_bitmap_set(nodeset, domainId);
-    ret = likwid_hwloc_set_area_membind_nodeset(hwloc_topology, ptr, size, nodeset, HWLOC_MEMBIND_BIND, flags);
-    likwid_hwloc_bitmap_free(nodeset);
+    if (!ptr || size == 0 || domainId < 0 || domainId >= numa_info.numberOfNodes)
+        return;
+    hwloc_membind_flags_t flags = HWLOC_MEMBIND_STRICT|HWLOC_MEMBIND_PROCESS|HWLOC_MEMBIND_BYNODESET;
+    hwloc_nodeset_t nodeset = LIKWID_HWLOC_NAME(bitmap_alloc)();
+    LIKWID_HWLOC_NAME(bitmap_zero)(nodeset);
+    LIKWID_HWLOC_NAME(bitmap_set)(nodeset, domainId);
+#if HWLOC_API_VERSION > 0x00020000
+    ret = LIKWID_HWLOC_NAME(set_area_membind)(hwloc_topology, ptr, size, nodeset, HWLOC_MEMBIND_BIND, flags);
+#else
+    ret = LIKWID_HWLOC_NAME(set_area_membind_nodeset)(hwloc_topology, ptr, size, nodeset, HWLOC_MEMBIND_BIND, flags);
+#endif
+    LIKWID_HWLOC_NAME(bitmap_free)(nodeset);
 
     if (ret < 0)
     {
@@ -380,21 +372,21 @@ hwloc_numa_setInterleaved(int* processorList, int numberOfProcessors)
 {
     int i,j;
     int ret = 0;
-    likwid_hwloc_cpuset_t cpuset = hwloc_bitmap_alloc();
-    likwid_hwloc_membind_flags_t flags = HWLOC_MEMBIND_STRICT|HWLOC_MEMBIND_PROCESS;
-    likwid_hwloc_bitmap_zero(cpuset);
+    LIKWID_HWLOC_NAME(cpuset_t) cpuset = LIKWID_HWLOC_NAME(bitmap_alloc)();
+    LIKWID_HWLOC_NAME(membind_flags_t) flags = HWLOC_MEMBIND_STRICT|HWLOC_MEMBIND_PROCESS;
+    LIKWID_HWLOC_NAME(bitmap_zero)(cpuset);
     for (i=0; i<numa_info.numberOfNodes; i++)
     {
         for (j=0; j<numberOfProcessors; j++)
         {
             if (likwid_hwloc_findProcessor(i,processorList[j]))
             {
-                likwid_hwloc_bitmap_set(cpuset, i);
+                LIKWID_HWLOC_NAME(bitmap_set)(cpuset, i);
             }
         }
     }
-    ret = likwid_hwloc_set_membind(hwloc_topology, cpuset, HWLOC_MEMBIND_INTERLEAVE, flags);
-    likwid_hwloc_bitmap_free(cpuset);
+    ret = LIKWID_HWLOC_NAME(set_membind)(hwloc_topology, cpuset, HWLOC_MEMBIND_INTERLEAVE, flags);
+    LIKWID_HWLOC_NAME(bitmap_free)(cpuset);
     if (ret < 0)
     {
         ERROR;
