@@ -5,14 +5,14 @@
  *
  *      Description:  Get NUMA topology from procfs and sysfs
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -106,6 +106,55 @@ setConfiguredNodes(void)
     return maxIdConfiguredNode;
 }
 
+static int
+get_numaNodes(int* array, int maxlen)
+{
+    DIR *dir = NULL;
+    struct dirent *de = NULL;
+    int count = 0;
+
+    dir = opendir("/sys/devices/system/node");
+
+    if (!dir)
+    {
+        count = 0;
+    }
+    else
+    {
+        while ((de = readdir(dir)) != NULL)
+        {
+            if (strncmp(de->d_name, "node", 4))
+            {
+                continue;
+            }
+            if (array && count < maxlen)
+            {
+                int nd = str2int(de->d_name+4);
+                array[count] = nd;
+            }
+            count++;
+        }
+    }
+    if (array && count > 0)
+    {
+        int i = 0;
+        int j = 0;
+        while (i < count)
+        {
+            j = i;
+            while (j > 0 && array[j-1] > array[j])
+            {
+                int tmp = array[j];
+                array[j] = array[j-1];
+                array[j-1] = tmp;
+                j--;
+            }
+            i++;
+        }
+    }
+    return count;
+}
+
 static void
 nodeMeminfo(int node, uint64_t* totalMemory, uint64_t* freeMemory)
 {
@@ -116,7 +165,6 @@ nodeMeminfo(int node, uint64_t* totalMemory, uint64_t* freeMemory)
     int i;
 
     filename = bformat("/sys/devices/system/node/node%d/meminfo", node);
-
     if (NULL != (fp = fopen (bdata(filename), "r")))
     {
         bstring src = bread ((bNread) fread, fp);
@@ -216,7 +264,7 @@ nodeProcessorList(int node, uint32_t** list)
                         }
                         else
                         {
-                            ERROR_PRINT(Number Of threads %d too large,count);
+                            ERROR_PRINT(Number Of threads %d too large for NUMA node %d, count, node);
                             return -EFAULT;
                         }
                         count++;
@@ -300,12 +348,18 @@ int proc_numa_init(void)
 
     if (get_mempolicy(NULL, NULL, 0, 0, 0) < 0 && errno == ENOSYS)
     {
-        numa_info.numberOfNodes = 0;
-        numa_info.nodes = NULL;
-        return -1;
+        /* Allocate a virtual node instead and bail out */
+        return virtual_numa_init();
     }
     /* First determine maximum number of nodes */
-    numa_info.numberOfNodes = setConfiguredNodes()+1;
+    //numa_info.numberOfNodes = setConfiguredNodes()+1;
+    numa_info.numberOfNodes= get_numaNodes(NULL, 10000);
+    int* nodes = malloc(numa_info.numberOfNodes*sizeof(int));
+    if (!nodes)
+    {
+        return -ENOMEM;
+    }
+    numa_info.numberOfNodes = get_numaNodes(nodes, numa_info.numberOfNodes);
     numa_info.nodes = (NumaNode*) malloc(numa_info.numberOfNodes * sizeof(NumaNode));
     if (!numa_info.nodes)
     {
@@ -314,16 +368,22 @@ int proc_numa_init(void)
 
     for (i=0; i<numa_info.numberOfNodes; i++)
     {
-        numa_info.nodes[i].id = i;
-        nodeMeminfo(i, &numa_info.nodes[i].totalMemory, &numa_info.nodes[i].freeMemory);
-        numa_info.nodes[i].numberOfProcessors = nodeProcessorList(i,&numa_info.nodes[i].processors);
-        nrCPUs += numa_info.nodes[i].numberOfProcessors;
-        if (numa_info.nodes[i].numberOfProcessors == 0 && nrCPUs != cpuid_topology.activeHWThreads)
-        {
-            err = -EFAULT;
-            break;
-        }
-        numa_info.nodes[i].numberOfDistances = nodeDistanceList(i, numa_info.numberOfNodes, &numa_info.nodes[i].distances);
+        int id = nodes[i];
+        numa_info.nodes[i].id = id;
+        nodeMeminfo(id, &numa_info.nodes[i].totalMemory, &numa_info.nodes[i].freeMemory);
+        numa_info.nodes[i].numberOfProcessors = nodeProcessorList(id, &numa_info.nodes[i].processors);
+/*        int check = 0;*/
+/*        for (int j = 0; j < numa_info.nodes[i].numberOfProcessors; j++)*/
+/*        {*/
+/*            int id = numa_info.nodes[i].processors[j];*/
+/*            if (cpuid_topology.threadPool[id].inCpuSet == 1)*/
+/*            {*/
+/*                numa_info.nodes[i].processors[check++] = numa_info.nodes[i].processors[j];*/
+/*            }*/
+/*        }*/
+/*        if (check < numa_info.nodes[i].numberOfProcessors)*/
+/*            numa_info.nodes[i].numberOfProcessors = check;*/
+        numa_info.nodes[i].numberOfDistances = nodeDistanceList(id, numa_info.numberOfNodes, &numa_info.nodes[i].distances);
         if (numa_info.nodes[i].numberOfDistances == 0)
         {
             err = -EFAULT;
@@ -332,8 +392,9 @@ int proc_numa_init(void)
     }
     for (; i<numa_info.numberOfNodes; i++)
     {
+        int id = nodes[i];
         numa_info.nodes[i].numberOfProcessors = 0;
-        numa_info.nodes[i].numberOfDistances = nodeDistanceList(i, numa_info.numberOfNodes, &numa_info.nodes[i].distances);
+        numa_info.nodes[i].numberOfDistances = nodeDistanceList(id, numa_info.numberOfNodes, &numa_info.nodes[i].distances);
     }
     if (err == 0)
         numaInitialized = 1;
@@ -369,6 +430,35 @@ proc_numa_setInterleaved(const int* processorList, int numberOfProcessors)
     }
 }
 
+void
+proc_numa_setMembind(const int* processorList, int numberOfProcessors)
+{
+    long i;
+    int j;
+    int ret=0;
+    unsigned long numberOfNodes = 65;
+    unsigned long mask = 0UL;
+
+    for (i=0; i<numa_info.numberOfNodes; i++)
+    {
+        for (j=0; j<numberOfProcessors; j++)
+        {
+            if (proc_findProcessor(i,processorList[j]))
+            {
+                mask |= (1UL<<i);
+                break;
+            }
+        }
+    }
+
+    ret = set_mempolicy(MPOL_BIND, &mask, numberOfNodes);
+
+    if (ret < 0)
+    {
+        ERROR;
+    }
+}
+
 void
 proc_numa_membind(void* ptr, size_t size, int domainId)
 {
