@@ -6,13 +6,13 @@
  *      Description:  Main implementation of the performance monitoring module
  *                    for NVIDIA GPUs
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Author:   Thomas Gruber (tg), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -74,7 +74,6 @@ int likwid_nvmon_verbosity = DEBUGLEV_ONLY_ERROR;
 #include <nvmon_nvml.h>
 #include <nvmon_perfworks.h>
 
-
 LikwidNvResults* gMarkerResults = NULL;
 int gMarkerRegions = 0;
 
@@ -86,7 +85,7 @@ nvmon_init(int nrGpus, const int* gpuIds)
 {
     int idx = 0;
     int ret = 0;
-    GpuTopology_t gtopo = NULL;
+    CudaTopology_t gtopo = NULL;
     if (nvmon_initialized == 1)
     {
         return 0;
@@ -109,16 +108,16 @@ nvmon_init(int nrGpus, const int* gpuIds)
         return -EEXIST;
     }
 
-    ret = topology_gpu_init();
+    ret = topology_cuda_init();
     if (ret != EXIT_SUCCESS)
     {
         return -ENODEV;
     }
-    gtopo = get_gpuTopology();
+    gtopo = get_cudaTopology();
 
     init_configuration();
 
-    nvGroupSet = (NvmonGroupSet*) malloc(sizeof(NvmonGroupSet));
+    nvGroupSet = calloc(1, sizeof(NvmonGroupSet));
     if (nvGroupSet == NULL)
     {
         ERROR_PLAIN_PRINT(Cannot allocate group descriptor);
@@ -138,6 +137,8 @@ nvmon_init(int nrGpus, const int* gpuIds)
     nvGroupSet->groups = NULL;
     nvGroupSet->activeGroup = -1;
     nvGroupSet->numberOfBackends = 0;
+    nvGroupSet->numGroupSources = 0;
+    nvGroupSet->groupSources = NULL;
 
 #ifdef LIKWID_NVMON_CUPTI_H
     nvGroupSet->backends[LIKWID_NVMON_CUPTI_BACKEND] = &nvmon_cupti_functions;
@@ -184,6 +185,9 @@ nvmon_init(int nrGpus, const int* gpuIds)
             perfworkscount++;
         }
         device->deviceId = gpuIds[i];
+        device->timeStart = 0;
+        device->timeStop = 0;
+        device->timeRead = 0;
         if (cputicount > 0 && perfworkscount > 0)
         {
             ERROR_PRINT(Cannot use GPUs with different backends in a session, gpuIds[i]);
@@ -194,21 +198,29 @@ nvmon_init(int nrGpus, const int* gpuIds)
         }
 
         NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-        if (funcs)
+        if (funcs && funcs->createDevice)
         {
-            if (funcs->createDevice)
-                ret = funcs->createDevice(device->deviceId, device);
-        }
-        if (ret < 0)
-        {
-            ERROR_PRINT(Cannot create device %d, device->deviceId);
-            free(nvGroupSet->gpus);
-            free(nvGroupSet);
-            nvGroupSet = NULL;
-            return -ENOMEM;
+            ret = funcs->createDevice(device->deviceId, device);
+            if (ret < 0)
+            {
+                ERROR_PRINT(Cannot create device %d (error: %d), device->deviceId, ret);
+                free(nvGroupSet->gpus);
+                free(nvGroupSet);
+                nvGroupSet = NULL;
+                return -ENOMEM;
+            }
         }
     }
 
+    ret = nvml_init();
+    if (ret < 0)
+    {
+        free(nvGroupSet->gpus);
+        free(nvGroupSet);
+        nvGroupSet = NULL;
+        return ret;
+    }
+
     nvmon_initialized = 1;
     return 0;
 }
@@ -220,6 +232,7 @@ nvmon_finalize(void)
 {
     if (nvmon_initialized && nvGroupSet)
     {
+        nvml_finalize();
         for (int i = 0; i < nvGroupSet->numberOfGPUs; i++)
         {
             NvmonDevice_t device = &nvGroupSet->gpus[i];
@@ -227,6 +240,12 @@ nvmon_finalize(void)
             if (funcs && funcs->freeDevice)
                 funcs->freeDevice(&nvGroupSet->gpus[i]);
         }
+        for (int i = 0; i < nvGroupSet->numGroupSources; i++)
+        {
+            free(nvGroupSet->groupSources[i].sourceTypes);
+            free(nvGroupSet->groupSources[i].sourceIds);
+        }
+        free(nvGroupSet->groupSources);
         free(nvGroupSet->gpus);
         free(nvGroupSet);
         nvGroupSet = NULL;
@@ -253,22 +272,30 @@ static int concatNvmonEventLists(NvmonEventList_t base, NvmonEventList_t new)
     for (int i = 0; i < new->numEvents; i++)
     {
         int bidx = base->numEvents + i;
-        base->events[bidx].name = malloc((strlen(new->events[i].name)+1)*sizeof(char));
+        int len = strlen(new->events[i].name);
+        base->events[bidx].name = malloc((len+1)*sizeof(char));
         if (base->events[bidx].name)
         {
-            strncpy(base->events[bidx].name, new->events[i].name, strlen(new->events[i].name));
+            strncpy(base->events[bidx].name, new->events[i].name, len);
+            base->events[bidx].name[len] = '\0';
         }
-        base->events[bidx].desc = malloc((strlen(new->events[i].desc)+1)*sizeof(char));
+        len = strlen(new->events[i].desc);
+        base->events[bidx].desc = malloc((len+1)*sizeof(char));
         if (base->events[bidx].desc)
         {
-            strncpy(base->events[bidx].desc, new->events[i].desc, strlen(new->events[i].desc));
+            strncpy(base->events[bidx].desc, new->events[i].desc, len);
+            base->events[bidx].desc[len] = '\0';
         }
-        base->events[bidx].limit = malloc((strlen(new->events[i].limit)+1)*sizeof(char));
+        len = strlen(new->events[i].limit);
+        base->events[bidx].limit = malloc((len+1)*sizeof(char));
         if (base->events[bidx].limit)
         {
-            strncpy(base->events[bidx].limit, new->events[i].limit, strlen(new->events[i].limit));
+            strncpy(base->events[bidx].limit, new->events[i].limit, len);
+            base->events[bidx].limit[len] = '\0';
         }
     }
+
+    base->numEvents = totalevents;
     return 0;
 }
 
@@ -278,12 +305,12 @@ int
 nvmon_getEventsOfGpu(int gpuId, NvmonEventList_t* list)
 {
     int err = 0;
-    int ret = topology_gpu_init();
+    int ret = topology_cuda_init();
     if (ret != EXIT_SUCCESS)
     {
         return -ENODEV;
     }
-    GpuTopology_t gtopo = get_gpuTopology();
+    CudaTopology_t gtopo = get_cudaTopology();
     int available = -1;
     for (int i = 0; i < gtopo->numDevices; i++)
     {
@@ -293,26 +320,47 @@ nvmon_getEventsOfGpu(int gpuId, NvmonEventList_t* list)
             break;
         }
     }
-    if (available >= 0)
+    if (available < 0)
     {
-        //err = nvmon_cupti_functions.getEventList(available, list);
-        if (gtopo->devices[available].ccapMajor < 7)
+        return -EINVAL;
+    }
+
+    if (gtopo->devices[available].ccapMajor < 7)
+    {
+        if (nvmon_cupti_functions.getEventList)
         {
-            if (nvmon_cupti_functions.getEventList)
-            {
-                err = nvmon_cupti_functions.getEventList(available, list);
-            }
+            err = nvmon_cupti_functions.getEventList(available, list);
         }
-        else
+    }
+    else
+    {
+        if (nvmon_perfworks_functions.getEventList)
         {
-            if (nvmon_perfworks_functions.getEventList)
-            {
-                err = nvmon_perfworks_functions.getEventList(available, list);
-            }
+            err = nvmon_perfworks_functions.getEventList(available, list);
         }
     }
-    return err;
 
+    // Get nvml events and merge lists
+    NvmonEventList_t nvmlList = NULL;
+    err = nvml_getEventsOfGpu(gpuId, &nvmlList);
+    if (err < 0)
+    {
+        nvmon_returnEventsOfGpu(*list);
+        *list = NULL;
+        return err;
+    }
+    err = concatNvmonEventLists(*list, nvmlList);
+    if (err < 0)
+    {
+        ERROR_PLAIN_PRINT(Failed to concatenate event lists);
+        nvmon_returnEventsOfGpu(*list);
+        nvml_returnEventsOfGpu(nvmlList);
+        *list = NULL;
+        return err;
+    }
+    nvml_returnEventsOfGpu(nvmlList);
+
+    return 0;
 }
 
 void nvmon_returnEventsOfGpu(NvmonEventList_t list)
@@ -339,12 +387,136 @@ void nvmon_returnEventsOfGpu(NvmonEventList_t list)
 }
 
 
+static int
+nvmon_splitEventSet(GroupInfo* backendEvents, GroupInfo* nvmlEvents, int gid)
+{
+    int ret;
+
+    // Initialize groups
+    perfgroup_new(backendEvents);
+    perfgroup_new(nvmlEvents);
+
+    // Sort events (shallow copy)
+    GroupInfo* info = &nvGroupSet->groups[gid];
+    for (int i = 0; i < info->nevents; i++)
+    {
+        if (nvGroupSet->groupSources[gid].sourceTypes[i] == NVMON_SOURCE_NVML)
+        {
+            ret = perfgroup_addEvent(nvmlEvents, info->counters[i], info->events[i]);
+            if (ret < 0)
+            {
+                ERROR_PRINT(Failed to add event while splitting);
+                return ret;
+            }
+        }
+        else
+        {
+            ret = perfgroup_addEvent(backendEvents, info->counters[i], info->events[i]);
+            if (ret < 0)
+            {
+                ERROR_PRINT(Failed to add event while splitting);
+                return ret;
+            }
+        }
+    }
+
+    return 0;
+}
+
+
+static void
+nvmon_returnSplitEventSet(GroupInfo* backendEvents, GroupInfo* nvmlEvents)
+{
+    if (backendEvents != NULL)
+    {
+        perfgroup_returnGroup(backendEvents);
+    }
+    if (nvmlEvents != NULL)
+    {
+        perfgroup_returnGroup(nvmlEvents);
+    }
+}
+
+
+static int
+nvmon_initEventSourceLookupMaps(int gid, int gpuId)
+{
+    int ret;
+    GroupInfo* group = &nvGroupSet->groups[gid];
+
+    // Allocate memory
+    if (gid >= nvGroupSet->numGroupSources)
+    {
+        int* tmpSourceTypes = (int*) malloc(group->nevents * sizeof(int));
+        if (tmpSourceTypes == NULL)
+        {
+            ERROR_PLAIN_PRINT(Failed to allocate source type map);
+            return -ENOMEM;
+        }
+        int* tmpSourceIds = (int*) malloc(group->nevents * sizeof(int));
+        if (tmpSourceIds == NULL)
+        {
+            ERROR_PLAIN_PRINT(Failed to allocate source id map);
+            free(tmpSourceTypes);
+            return -ENOMEM;
+        }
+        NvmonGroupSourceInfo* tmpInfo = (NvmonGroupSourceInfo*) realloc(nvGroupSet->groupSources, (gid+1) * sizeof(NvmonGroupSourceInfo));
+        if (tmpInfo == NULL)
+        {
+            ERROR_PLAIN_PRINT(Failed to allocate source infos);
+            free(tmpSourceTypes);
+            free(tmpSourceIds);
+            return -ENOMEM;
+        }
+
+        nvGroupSet->groupSources = tmpInfo;
+        nvGroupSet->groupSources[gid].numEvents = group->nevents;
+        nvGroupSet->groupSources[gid].sourceTypes = tmpSourceTypes;
+        nvGroupSet->groupSources[gid].sourceIds = tmpSourceIds;
+        nvGroupSet->numGroupSources++;
+    }
+
+    // Get list of nvml events for sorting
+    NvmonEventList_t nvmlList;
+    ret = nvml_getEventsOfGpu(gpuId, &nvmlList);
+    if (ret < 0) return ret;
+
+    // Sort events
+    int nvmlId = 0;
+    int backendId = 0;
+    for (int i = 0; i < group->nevents; i++)
+    {
+        // Check if event is in nvml list
+        int isNvmlEvent = 0;
+        for (int j = 0; j < nvmlList->numEvents; j++)
+        {
+            if (strcmp(group->events[i], nvmlList->events[j].name) == 0)
+            {
+                isNvmlEvent = 1;
+                break;
+            }
+        }
+        if (isNvmlEvent)
+        {
+            nvGroupSet->groupSources[gid].sourceTypes[i] = NVMON_SOURCE_NVML;
+            nvGroupSet->groupSources[gid].sourceIds[i] = nvmlId++;
+        }
+        else
+        {
+            nvGroupSet->groupSources[gid].sourceTypes[i] = NVMON_SOURCE_BACKEND;
+            nvGroupSet->groupSources[gid].sourceIds[i] = backendId++;
+        }
+    }
+
+    nvml_returnEventsOfGpu(nvmlList);
+    return 0;
+}
+
 
 int
 nvmon_addEventSet(const char* eventCString)
 {
     int i = 0, j = 0, k = 0, l = 0, m = 0, err = 0;
-    int isPerfGroup = 0;
 
     int devId = 0;
     int configuredEvents = 0;
@@ -440,37 +612,83 @@ nvmon_addEventSet(const char* eventCString)
             ERROR_PRINT(Cannot read performance group %s, eventCString);
             return err;
         }
-        isPerfGroup = 1;
     }
+
+    // Build source lookup
+    err = nvmon_initEventSourceLookupMaps(nvGroupSet->numberOfGroups - 1, nvGroupSet->gpus[0].deviceId); // it is assumed that all gpus split the same
+    if (err < 0)
+    {
+        ERROR_PRINT(Failed to init source lookup for group %d, nvGroupSet->numberOfGroups - 1);
+        return err;
+    }
+
+    // Split events into nvml and backend events
+    GroupInfo backendEvents;
+    GroupInfo nvmlEvents;
+    err = nvmon_splitEventSet(&backendEvents, &nvmlEvents, nvGroupSet->numberOfGroups-1);
+    if (err < 0)
+    {
+        ERROR_PRINT(Failed to split events);
+        return -1;
+    }
+
+    // Add nvml events
+    err = nvml_addEventSet(nvmlEvents.events, nvmlEvents.nevents);
+    if (err < 0)
+    {
+        ERROR_PRINT(Failed to add nvml events);
+        return -1;
+    }
+
     bdestroy(eventBString);
-    char * evstr = perfgroup_getEventStr(&nvGroupSet->groups[nvGroupSet->numberOfGroups-1]);
+    char * evstr = perfgroup_getEventStr(&backendEvents);
 /*    eventBString = bfromcstr(evstr);*/
     GPUDEBUG_PRINT(DEBUGLEV_DEVELOP, EventStr %s, evstr);
 /*    eventtokens = bsplit(eventBString, ',');*/
 /*    bdestroy(eventBString);*/
 
+    // Event string is null when there are no events for backend
     for (devId = 0; devId < nvGroupSet->numberOfGPUs; devId++)
     {
-        int err = 0;
         device = &nvGroupSet->gpus[devId];
-        NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-        if (!funcs)
+        if (evstr != NULL)
         {
-            ERROR_PRINT(Backend functions undefined?);
-        }
-        if (funcs->addEvents)
-        {
-            GPUDEBUG_PRINT(DEBUGLEV_DEVELOP, Calling addevents);
-            err = funcs->addEvents(device, evstr);
-            if (err < 0)
+            int err = 0;
+            NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
+            if (!funcs)
             {
-                errno = -err;
-                ERROR_PRINT(Failed to add event set for GPU %d, devId);
-                return err;
+                ERROR_PRINT(Backend functions undefined?);
             }
+            if (funcs->addEvents)
+            {
+                GPUDEBUG_PRINT(DEBUGLEV_DEVELOP, Calling addevents);
+                err = funcs->addEvents(device, evstr);
+                if (err < 0)
+                {
+                    errno = -err;
+                    ERROR_PRINT(Failed to add event set for GPU %d, devId);
+                    return err;
+                }
+            }
+        }   
+        else
+        {
+            // Add empty event set 
+            NvmonEventSet* tmpEventSet = realloc(device->nvEventSets, (device->numNvEventSets+1)*sizeof(NvmonEventSet));
+            if (!tmpEventSet)
+            {
+                ERROR_PRINT(Cannot enlarge GPU %d eventSet list, device->deviceId);
+                return -ENOMEM;
+            }
+            device->nvEventSets = tmpEventSet;
+            NvmonEventSet* newEventSet = &device->nvEventSets[device->numNvEventSets];
+            memset(newEventSet, 0, sizeof(NvmonEventSet));
         }
     }
 
+    // Cleanup
+    nvmon_returnSplitEventSet(&backendEvents, &nvmlEvents);
+
     // Check whether group has any event in any device
     nvGroupSet->numberOfActiveGroups++;
     return (nvGroupSet->numberOfActiveGroups-1);
@@ -498,11 +716,12 @@ int nvmon_setupCounters(int gid)
             NvmonEventSet* devEventSet = &device->nvEventSets[gid];
 
             NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-            if (funcs->setupCounters)
+            if (devEventSet->numberOfEvents > 0 && funcs->setupCounters)
             {
                 err = funcs->setupCounters(device, devEventSet);
             }
         }
+        nvml_setupCounters(gid);
     }
     nvGroupSet->activeGroup = gid;
 
@@ -523,14 +742,20 @@ nvmon_startCounters(void)
     for (i = 0; i < nvGroupSet->numberOfGPUs; i++)
     {
         NvmonDevice_t device = &nvGroupSet->gpus[i];
+        NvmonEventSet* devEventSet = &device->nvEventSets[nvGroupSet->activeGroup];
         NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-        if (funcs->startCounters)
+        if (devEventSet->numberOfEvents > 0 && funcs->startCounters)
         {
             err = funcs->startCounters(device);
+            if (err < 0) return err;
         }
     }
 
-    return err;
+    // Start nvml counters
+    err = nvml_startCounters();
+    if (err < 0) return err;
+
+    return 0;
 }
 
 int
@@ -546,13 +771,19 @@ nvmon_stopCounters(void)
     for (i = 0; i < nvGroupSet->numberOfGPUs; i++)
     {
         NvmonDevice_t device = &nvGroupSet->gpus[i];
+        NvmonEventSet* devEventSet = &device->nvEventSets[nvGroupSet->activeGroup];
         NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-        if (funcs->stopCounters)
+        if (devEventSet->numberOfEvents > 0 && funcs->stopCounters)
         {
             err = funcs->stopCounters(device);
+            if (err < 0) return err;
         }
     }
 
+    // Stop nvml counters
+    err = nvml_stopCounters();
+    if (err < 0) return err;
+
     return 0;
 }
 
@@ -569,13 +800,19 @@ int nvmon_readCounters(void)
     for (i = 0; i < nvGroupSet->numberOfGPUs; i++)
     {
         NvmonDevice_t device = &nvGroupSet->gpus[i];
+        NvmonEventSet* devEventSet = &device->nvEventSets[nvGroupSet->activeGroup];
         NvmonFunctions* funcs = nvGroupSet->backends[device->backend];
-        if (funcs->readCounters)
+        if (devEventSet->numberOfEvents > 0 && funcs->readCounters)
         {
             err = funcs->readCounters(device);
+            if (err < 0) return err;
         }
-
     }
+    
+    // Read nvml counters
+    err = nvml_readCounters();
+    if (err < 0) return err;
+
     return 0;
 }
 
@@ -589,17 +826,31 @@ double nvmon_getResult(int groupId, int eventId, int gpuId)
     {
         return -EFAULT;
     }
-    NvmonDevice *device = &nvGroupSet->gpus[gpuId];
-    if (groupId < 0 || groupId >= device->numNvEventSets)
+    if (groupId < 0 || groupId >= nvGroupSet->numGroupSources)
     {
         return -EFAULT;
     }
-    NvmonEventSet* evset = &device->nvEventSets[groupId];
-    if (eventId < 0 || eventId >= evset->numberOfEvents)
+    NvmonGroupSourceInfo* info = &nvGroupSet->groupSources[groupId];
+    if (eventId < 0 || eventId >= info->numEvents)
     {
         return -EFAULT;
     }
-    return evset->results[eventId].fullValue;
+
+    // Get value from respective source
+    if (info->sourceTypes[eventId] == NVMON_SOURCE_NVML)
+    {
+        return nvml_getResult(gpuId, groupId, info->sourceIds[eventId]);
+    }
+    else
+    {
+        NvmonDevice *device = &nvGroupSet->gpus[gpuId];
+        if (groupId < 0 || groupId >= device->numNvEventSets)
+        {
+            return -EFAULT;
+        }
+        NvmonEventSet* evset = &device->nvEventSets[groupId];
+        return evset->results[info->sourceIds[eventId]].fullValue;
+    }
 }
 
 
@@ -622,17 +873,31 @@ double nvmon_getLastResult(int groupId, int eventId, int gpuId)
     {
         return -EFAULT;
     }
-    NvmonDevice *device = &nvGroupSet->gpus[gpuId];
-    if (groupId < 0 || groupId >= device->numNvEventSets)
+    if (groupId < 0 || groupId >= nvGroupSet->numGroupSources)
     {
         return -EFAULT;
     }
-    NvmonEventSet* evset = &device->nvEventSets[groupId];
-    if (eventId < 0 || eventId >= evset->numberOfEvents)
+    NvmonGroupSourceInfo* info = &nvGroupSet->groupSources[groupId];
+    if (eventId < 0 || eventId >= info->numEvents)
     {
         return -EFAULT;
     }
-    return evset->results[eventId].lastValue;
+
+    // Get value from respective source
+    if (info->sourceTypes[eventId] == NVMON_SOURCE_NVML)
+    {
+        return nvml_getLastResult(gpuId, groupId, info->sourceIds[eventId]);
+    }
+    else
+    {
+        NvmonDevice *device = &nvGroupSet->gpus[gpuId];
+        if (groupId < 0 || groupId >= device->numNvEventSets)
+        {
+            return -EFAULT;
+        }
+        NvmonEventSet* evset = &device->nvEventSets[groupId];
+        return evset->results[info->sourceIds[eventId]].lastValue;
+    }
 }
 
 
@@ -678,35 +943,91 @@ int nvmon_getNumberOfEvents(int groupId)
 
 double nvmon_getTimeOfGroup(int groupId)
 {
-    int i = 0;
-    double t = 0;
     if ((!nvGroupSet) || (!nvmon_initialized) || (groupId < 0) || groupId >= nvGroupSet->numberOfActiveGroups)
     {
         return -EFAULT;
     }
-    for (i = 0; i < nvGroupSet->numberOfGPUs; i++)
+
+    // Get largest time from backend measurings
+    double backendTime = 0;
+    for (int i = 0; i < nvGroupSet->numberOfGPUs; i++)
     {
-        t = MAX(t, (double)(nvGroupSet->gpus[i].timeStop - nvGroupSet->gpus[i].timeStart));
+        // timeStart and timeStop are zero if no backend event is registered
+        backendTime = MAX(backendTime, (double)(nvGroupSet->gpus[i].timeStop - nvGroupSet->gpus[i].timeStart));
     }
-    return t*1E-9;
+    backendTime *= 1E-9; // ns to seconds
+
+    // Get largest time of nvml events
+    double nvmlTime = 0;
+    nvmlTime = nvml_getTimeOfGroup(groupId);
+    if (nvmlTime < 0)
+    {
+        return nvmlTime;
+    }
+
+    // Return largest time
+    return MAX(backendTime, nvmlTime);
 }
 
 
 double nvmon_getLastTimeOfGroup(int groupId)
 {
-    int i = 0;
-    double t = 0;
     if ((!nvGroupSet) || (!nvmon_initialized) || (groupId < 0) || groupId >= nvGroupSet->numberOfActiveGroups)
     {
         return -EFAULT;
     }
-    for (i = 0; i < nvGroupSet->numberOfGPUs; i++)
+
+    // Get largest time from backend measurings
+    double backendTime = 0;
+    for (int i = 0; i < nvGroupSet->numberOfGPUs; i++)
     {
-        t = MAX(t, (double)(nvGroupSet->gpus[i].timeStop - nvGroupSet->gpus[i].timeRead));
+        // timeStart and timeStop are zero if no backend event is registered
+        backendTime = MAX(backendTime, (double)(nvGroupSet->gpus[i].timeStop - nvGroupSet->gpus[i].timeRead));
     }
-    return t*1E-9;
+    backendTime *= 1E-9; // ns to seconds
+
+    // Get largest time of nvml events
+    double nvmlTime = 0;
+    nvmlTime = nvml_getLastTimeOfGroup(groupId);
+    if (nvmlTime < 0)
+    {
+        return nvmlTime;
+    }
+
+    // Return largest time
+    return MAX(backendTime, nvmlTime);
 }
 
+
+double nvmon_getTimeToLastReadOfGroup(int groupId)
+{
+    if ((!nvGroupSet) || (!nvmon_initialized) || (groupId < 0) || groupId >= nvGroupSet->numberOfActiveGroups)
+    {
+        return -EFAULT;
+    }
+
+    // Get largest time from backend measurings
+    double backendTime = 0;
+    for (int i = 0; i < nvGroupSet->numberOfGPUs; i++)
+    {
+        // timeStart and timeStop are zero if no backend event is registered
+        backendTime = MAX(backendTime, (double)(nvGroupSet->gpus[i].timeRead - nvGroupSet->gpus[i].timeStart));
+    }
+    backendTime *= 1E-9; // ns to seconds
+
+    // Get largest time of nvml events
+    double nvmlTime = 0;
+    nvmlTime = nvml_getTimeToLastReadOfGroup(groupId);
+    if (nvmlTime < 0)
+    {
+        return nvmlTime;
+    }
+
+    // Return largest time
+    return MAX(backendTime, nvmlTime);
+}
+
+
 char* nvmon_getEventName(int groupId, int eventId)
 {
     if ((!nvGroupSet) || (!nvmon_initialized) || (groupId < 0) || groupId >= nvGroupSet->numberOfActiveGroups)
@@ -786,12 +1107,12 @@ int nvmon_getGroups(int gpuId, char*** groups, char*** shortinfos, char*** longi
     init_configuration();
     int ccapMajor = 0;
     Configuration_t config = get_configuration();
-    ret = topology_gpu_init();
+    ret = topology_cuda_init();
     if (ret != EXIT_SUCCESS)
     {
         return -ENODEV;
     }
-    GpuTopology_t gtopo = get_gpuTopology();
+    CudaTopology_t gtopo = get_cudaTopology();
     if (gpuId < 0 || gpuId >= gtopo->numDevices)
     {
         return -ENODEV;
@@ -810,6 +1131,7 @@ int nvmon_getGroups(int gpuId, char*** groups, char*** shortinfos, char*** longi
 int nvmon_returnGroups(int nrgroups, char** groups, char** shortinfos, char** longinfos)
 {
     perfgroup_returnGroups(nrgroups, groups, shortinfos, longinfos);
+    return 0;
 }
 
 int nvmon_getNumberOfMetrics(int groupId)
@@ -1025,7 +1347,7 @@ nvmon_readMarkerFile(const char* filename)
         return -ENOMEM;
     }
     gMarkerRegions = regions;
-    for ( uint32_t i=0; i < regions; i++ )
+    for (int i = 0; i < regions; i++)
     {
         regionGPUs[i] = 0;
         gMarkerResults[i].gpuCount = gpus;
@@ -1120,7 +1442,7 @@ nvmon_readMarkerFile(const char* filename)
             }
         }
     }
-    for ( uint32_t i=0; i < regions; i++ )
+    for (int i = 0; i < regions; i++)
     {
         gMarkerResults[i].gpuCount = regionGPUs[i];
     }
@@ -1386,5 +1708,3 @@ double nvmon_getMetricOfRegionGpu(int region, int metricId, int gpuId)
     destroy_clist(&clist);
     return result;
 }
-
-
