@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the cpuid based topology backend
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -41,7 +41,9 @@
 #include <bitUtil.h>
 #include <tlb-info.h>
 #include <topology.h>
+#if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A)
 #include <cpuid.h>
+#endif
 #include <affinity.h>
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
@@ -324,7 +326,7 @@ cpuid_set_osname(void)
     fclose(fp);
 }
 
-void
+int
 cpuid_init_cpuInfo(cpu_set_t cpuSet)
 {
     int cpus_in_set = 0;
@@ -358,10 +360,10 @@ cpuid_init_cpuInfo(cpu_set_t cpuSet)
                             cpuid_info.isIntel,
                             cpuid_topology.numHWThreads,
                             cpuid_topology.activeHWThreads)
-    return;
+    return 0;
 }
 
-void
+int
 cpuid_init_cpuFeatures(void)
 {
     eax = 0x01;
@@ -369,6 +371,10 @@ cpuid_init_cpuFeatures(void)
 
     cpuid_info.featureFlags = 0;
     cpuid_info.features = (char*) malloc(MAX_FEATURE_STRING_LENGTH*sizeof(char));
+    if (!cpuid_info.features)
+    {
+        return -ENOMEM;
+    }
     cpuid_info.features[0] = '\0';
     if (ecx & (1<<0))
     {
@@ -516,10 +522,10 @@ cpuid_init_cpuFeatures(void)
         }
     }
 
-    return;
+    return 0;
 }
 
-void
+int
 cpuid_init_nodeTopology(cpu_set_t cpuSet)
 {
     uint32_t apicId;
@@ -535,6 +541,10 @@ cpuid_init_nodeTopology(cpu_set_t cpuSet)
     int maxNumCores;
     int width;
     hwThreadPool = (HWThread*) malloc(cpuid_topology.numHWThreads * sizeof(HWThread));
+    if (!hwThreadPool)
+    {
+        return -ENOMEM;
+    }
     /* check if 0x0B cpuid leaf is supported */
     if (largest_function >= 0x0B)
     {
@@ -766,10 +776,10 @@ cpuid_init_nodeTopology(cpu_set_t cpuSet)
         }
     }
     cpuid_topology.threadPool = hwThreadPool;
-    return;
+    return 0;
 }
 
-void
+int
 cpuid_init_cacheTopology(void)
 {
     int maxNumLevels=0;
@@ -777,6 +787,7 @@ cpuid_init_cacheTopology(void)
     CacheLevel* cachePool = NULL;
     CacheType type = DATACACHE;
 
+
     switch ( cpuid_info.family )
     {
         case MIC_FAMILY:
@@ -796,7 +807,10 @@ cpuid_init_cacheTopology(void)
         case K8_FAMILY:
             maxNumLevels = 2;
             cachePool = (CacheLevel*) malloc(maxNumLevels * sizeof(CacheLevel));
-
+            if (!cachePool)
+            {
+                return -ENOMEM;
+            }
             eax = 0x80000005;
             CPUID(eax, ebx, ecx, edx);
             cachePool[0].level = 1;
@@ -835,7 +849,10 @@ cpuid_init_cacheTopology(void)
              */
             maxNumLevels = 3;
             cachePool = (CacheLevel*) malloc(maxNumLevels * sizeof(CacheLevel));
-
+            if (!cachePool)
+            {
+                return -ENOMEM;
+            }
             eax = 0x80000005;
             CPUID(eax, ebx, ecx, edx);
             cachePool[0].level = 1;
@@ -898,7 +915,10 @@ cpuid_init_cacheTopology(void)
 
             maxNumLevels = 0;
             cachePool = (CacheLevel*) malloc(3 * sizeof(CacheLevel));
-
+            if (!cachePool)
+            {
+                return -ENOMEM;
+            }
             while (type)
             {
                 ecx = id;
@@ -929,6 +949,14 @@ cpuid_init_cacheTopology(void)
 
     cpuid_topology.numCacheLevels = maxNumLevels;
     cpuid_topology.cacheLevels = cachePool;
-    return;
+    return 0;
 }
 
+
+int cpuid_get_hwthread_data(int cpu_id, HWThread* hwt)
+{
+    eax = 0x01;
+    CPUID(eax, ebx, ecx, edx);
+    hwt->apicId = extractBitField(ebx,8,24);
+    return 0;
+}
