@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the hwloc based topology backend
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Authors:  Thomas Gruber (tr), thomas.roehl@googlemail.com
  *
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -63,54 +63,61 @@ int parse_cpuinfo(uint32_t* count, uint32_t* family, uint32_t* variant, uint32_t
     uint32_t c = 0;
     int (*ownatoi)(const char*);
     ownatoi = &atoi;
+    struct tagbstring familyString = bsStatic("CPU architecture");
+    struct tagbstring variantString = bsStatic("CPU variant");
+    struct tagbstring steppingString = bsStatic("CPU revision");
+    struct tagbstring partString = bsStatic("CPU part");
+    struct tagbstring vendString = bsStatic("CPU implementer");
+    struct tagbstring procString = bsStatic("processor");
 
     if (NULL != (fp = fopen ("/proc/cpuinfo", "r")))
     {
-        const_bstring familyString = bformat("CPU architecture");
-        const_bstring variantString = bformat("CPU variant");
-        const_bstring steppingString = bformat("CPU revision");
-        const_bstring partString = bformat("CPU part");
-        const_bstring vendString = bformat("CPU implementer");
-        const_bstring procString = bformat("processor");
         bstring src = bread ((bNread) fread, fp);
         struct bstrList* tokens = bsplit(src,(char) '\n');
         bdestroy(src);
         fclose(fp);
         for (i=0;i<tokens->qty;i++)
         {
-            if ((f == 0) && (binstr(tokens->entry[i],0,procString) != BSTR_ERR))
+            if ((binstr(tokens->entry[i],0,&procString) != BSTR_ERR))
             {
-                c++;
+                struct bstrList* subtokens = bsplit(tokens->entry[i],(char) ':');
+                bltrimws(subtokens->entry[1]);
+                uint32_t tmp = ownatoi(bdata(subtokens->entry[1]));
+                if (tmp > c)
+                {
+                    c = tmp;
+                }
+                bstrListDestroy(subtokens);
             }
-            else if ((f == 0) && (binstr(tokens->entry[i],0,familyString) != BSTR_ERR))
+            else if ((f == 0) && (binstr(tokens->entry[i],0,&familyString) != BSTR_ERR))
             {
                 struct bstrList* subtokens = bsplit(tokens->entry[i],(char) ':');
                 bltrimws(subtokens->entry[1]);
                 f = ownatoi(bdata(subtokens->entry[1]));
                 bstrListDestroy(subtokens);
             }
-            else if ((s == 0) && (binstr(tokens->entry[i],0,steppingString) != BSTR_ERR))
+            else if ((s == 0) && (binstr(tokens->entry[i],0,&steppingString) != BSTR_ERR))
             {
                 struct bstrList* subtokens = bsplit(tokens->entry[i],(char) ':');
                 bltrimws(subtokens->entry[1]);
                 s = ownatoi(bdata(subtokens->entry[1]));
                 bstrListDestroy(subtokens);
             }
-            else if ((v == 0) && (binstr(tokens->entry[i],0,variantString) != BSTR_ERR))
+            else if ((v == 0) && (binstr(tokens->entry[i],0,&variantString) != BSTR_ERR))
             {
                 struct bstrList* subtokens = bsplit(tokens->entry[i],(char) ':');
                 bltrimws(subtokens->entry[1]);
                 v = strtol(bdata(subtokens->entry[1]), NULL, 0);
                 bstrListDestroy(subtokens);
             }
-            else if ((p == 0) && (binstr(tokens->entry[i],0,partString) != BSTR_ERR))
+            else if ((p == 0) && (binstr(tokens->entry[i],0,&partString) != BSTR_ERR))
             {
                 struct bstrList* subtokens = bsplit(tokens->entry[i],(char) ':');
                 bltrimws(subtokens->entry[1]);
                 p = strtol(bdata(subtokens->entry[1]), NULL, 0);
                 bstrListDestroy(subtokens);
             }
-            else if ((p == 0) && (binstr(tokens->entry[i],0,vendString) != BSTR_ERR))
+            else if ((vend == 0) && (binstr(tokens->entry[i],0,&vendString) != BSTR_ERR))
             {
                 struct bstrList* subtokens = bsplit(tokens->entry[i],(char) ':');
                 bltrimws(subtokens->entry[1]);
@@ -119,9 +126,6 @@ int parse_cpuinfo(uint32_t* count, uint32_t* family, uint32_t* variant, uint32_t
             }
         }
         bstrListDestroy(tokens);
-        /*bdestroy(familyString);
-        bdestroy(variantString);
-        bdestroy(steppingString);*/
     }
     else
     {
@@ -132,7 +136,7 @@ int parse_cpuinfo(uint32_t* count, uint32_t* family, uint32_t* variant, uint32_t
     *stepping = s;
     *part = p;
     *vendor = vend;
-    *count = c;
+    *count = c + 1;
     return 0;
 }
 
@@ -235,7 +239,7 @@ likwid_hwloc_record_objs_of_type_below_obj(
 
 
 
-void
+int
 hwloc_init_cpuInfo(cpu_set_t cpuSet)
 {
     int i;
@@ -269,7 +273,9 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
     cpuid_info.osname[0] = '\0';
     if (!obj)
     {
-        return;
+        free(cpuid_info.osname);
+        cpuid_info.osname = NULL;
+        return -EFAULT;
     }
 
     const char * info;
@@ -284,25 +290,16 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
         cpuid_info.stepping = atoi(info);
     snprintf(cpuid_info.architecture, 19, "x86_64");
 #endif
-#ifdef __ARM_ARCH_7A__
-    if ((info = LIKWID_HWLOC_NAME(obj_get_info_by_name)(obj, "CPUArchitecture")))
-       cpuid_info.family = atoi(info);
-    if ((info = LIKWID_HWLOC_NAME(obj_get_info_by_name)(obj, "CPURevision")))
-        cpuid_info.model = atoi(info);
-    if (cpuid_info.family == 0 || cpuid_info.model == 0)
-    {
-        uint32_t part = 0;
-        parse_cpuinfo(&count, &cpuid_info.family, &cpuid_info.model, &cpuid_info.stepping, &cpuid_info.part, &cpuid_info.vendor);
-        parse_cpuname(cpuid_info.osname);
-    }
-    snprintf(cpuid_info.architecture, 19, "armv7");
-#endif
 #if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
-    uint32_t part = 0;
     parse_cpuinfo(&count, &cpuid_info.family, &cpuid_info.model, &cpuid_info.stepping, &cpuid_info.part, &cpuid_info.vendor);
     parse_cpuname(cpuid_info.osname);
+#ifdef __ARM_ARCH_7A__
+    snprintf(cpuid_info.architecture, 19, "armv7");
+#endif
+#ifdef __ARM_ARCH_8A
     snprintf(cpuid_info.architecture, 19, "armv8");
 #endif
+#endif
 
 #ifndef _ARCH_PPC
     if ((info = LIKWID_HWLOC_NAME(obj_get_info_by_name)(obj, "CPUModel")))
@@ -345,13 +342,22 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
     cpuid_topology.numHWThreads = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, HWLOC_OBJ_PU);
 #if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
     if (count > cpuid_topology.numHWThreads)
+    {
         cpuid_topology.numHWThreads = count;
+    }
 #endif
-    count = likwid_sysfs_list_len("/sys/devices/system/cpu/online");
-    if (count > cpuid_topology.numHWThreads)
-        cpuid_topology.numHWThreads = count;
-    if (cpuid_topology.activeHWThreads > cpuid_topology.numHWThreads)
-        cpuid_topology.numHWThreads = cpuid_topology.activeHWThreads;
+    if (!getenv("HWLOC_FSROOT"))
+    {
+        count = likwid_sysfs_list_len("/sys/devices/system/cpu/online");
+        if (count > cpuid_topology.numHWThreads)
+        {
+            cpuid_topology.numHWThreads = count;
+        }
+        if (cpuid_topology.activeHWThreads > cpuid_topology.numHWThreads)
+        {
+            cpuid_topology.numHWThreads = cpuid_topology.activeHWThreads;
+        }
+    }
     DEBUG_PRINT(DEBUGLEV_DEVELOP, HWLOC CpuInfo Family %d Model %d Stepping %d Vendor 0x%X Part 0x%X isIntel %d numHWThreads %d activeHWThreads %d,
                             cpuid_info.family,
                             cpuid_info.model,
@@ -361,10 +367,10 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
                             cpuid_info.isIntel,
                             cpuid_topology.numHWThreads,
                             cpuid_topology.activeHWThreads)
-    return;
+    return 0;
 }
 
-void
+int
 hwloc_init_nodeTopology(cpu_set_t cpuSet)
 {
     HWThread* hwThreadPool = NULL;
@@ -381,6 +387,7 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
     int consecutive_cores = -1;
     int from_file = (getenv("HWLOC_FSROOT") != NULL);
     hwloc_obj_type_t socket_type = HWLOC_OBJ_SOCKET;
+    hwloc_obj_type_t die_type = HWLOC_OBJ_DIE;
     if (!from_file)
     {
         for (uint32_t i=0;i<cpuid_topology.numHWThreads;i++)
@@ -412,9 +419,14 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
     {
         socket_type = HWLOC_OBJ_NODE;
     }
+    if (LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, die_type) == 0)
+    {
+        die_type = socket_type;
+    }
+    
 
     maxNumSockets = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, socket_type);
-    maxNumDies = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, HWLOC_OBJ_DIE);
+    maxNumDies = LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, die_type);
     obj = LIKWID_HWLOC_NAME(get_obj_by_type)(hwloc_topology, socket_type, 0);
 
     if (obj)
@@ -508,34 +520,6 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
             }
         }
 #endif
-        if (maxNumSockets < maxNumDies)
-        {
-            while (obj->type != HWLOC_OBJ_DIE) {
-                obj = obj->parent;
-                if (!obj)
-                {
-                    skip = 1;
-                    break;
-                }
-            }
-            if (skip)
-            {
-                hwThreadPool[id].dieId = 0;
-                continue;
-            }
-            if (obj->type == HWLOC_OBJ_DIE)
-            {
-                hwThreadPool[id].dieId = obj->os_index;
-            }
-            else
-            {
-                hwThreadPool[id].dieId = 0;
-            }
-        }
-        else
-        {
-            hwThreadPool[id].dieId = 0;
-        }
         while (obj->type != socket_type) {
             obj = obj->parent;
             if (!obj)
@@ -561,6 +545,40 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
 #else
         hwThreadPool[id].packageId = obj->os_index;
 #endif
+        if (maxNumSockets < maxNumDies)
+        {
+            printf("Searching for socket of die\n");
+            hwloc_obj_t t = obj->parent;
+            while (t->type != die_type) {
+                t = t->parent;
+                if (!t)
+                {
+                    printf("Cannot find type %d\n", die_type);
+                    skip = 1;
+                    break;
+                }
+            }
+            if (skip)
+            {
+                printf("setting die id = 0\n");
+                hwThreadPool[id].dieId = 0;
+                continue;
+            }
+            if (t->type == die_type)
+            {
+                printf("setting die id = %d\n", t->os_index);
+                hwThreadPool[id].dieId = t->os_index;
+            }
+            else
+            {
+                printf("setting die id = 0 (else)\n");
+                hwThreadPool[id].dieId = 0;
+            }
+        }
+        else
+        {
+            hwThreadPool[id].dieId = 0;
+        }
         DEBUG_PRINT(DEBUGLEV_DEVELOP, HWLOC Thread Pool PU %d Thread %d Core %d Die %d Socket %d inCpuSet %d,
                             hwThreadPool[id].apicId,
                             hwThreadPool[id].threadId,
@@ -572,6 +590,7 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
 
     int socket_nums[MAX_NUM_NODES];
     int num_sockets = 0;
+    int has_zero_id = 0;
     for (uint32_t i=0; i< cpuid_topology.numHWThreads; i++)
     {
         int found = 0;
@@ -588,14 +607,22 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
             socket_nums[num_sockets] = hwThreadPool[i].packageId;
             num_sockets++;
         }
-    }
-    for (uint32_t i=0; i< cpuid_topology.numHWThreads; i++)
-    {
-        for (uint32_t j=0; j < num_sockets; j++)
+        if (hwThreadPool[i].packageId == 0)
         {
-            if (hwThreadPool[i].packageId == socket_nums[j])
+            has_zero_id = 1;
+        }
+    }
+    if (!has_zero_id)
+    {
+        for (uint32_t i=0; i< cpuid_topology.numHWThreads; i++)
+        {
+            HWThread* t = &hwThreadPool[i];
+            for (uint32_t j=0; j < num_sockets; j++)
             {
-                hwThreadPool[i].packageId = j;
+                if (hwThreadPool[i].packageId == socket_nums[j])
+                {
+                    hwThreadPool[i].packageId = j;
+                }
             }
         }
     }
@@ -604,7 +631,7 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
     cpuid_topology.numCoresPerSocket = maxNumCoresPerSocket;
     cpuid_topology.numSockets = maxNumSockets;
     cpuid_topology.numDies = maxNumDies;
-    return;
+    return 0;
 }
 
 void hwloc_split_llc_check(CacheLevel* llc_cache)
@@ -647,7 +674,7 @@ void hwloc_split_llc_check(CacheLevel* llc_cache)
     return;
 }
 
-void
+int
 hwloc_init_cacheTopology(void)
 {
     int maxNumLevels=0;
@@ -680,7 +707,7 @@ hwloc_init_cacheTopology(void)
     {
         cpuid_topology.numCacheLevels = 0;
         cpuid_topology.cacheLevels = NULL;
-        return;
+        return -ENOMEM;
     }
     /* Start at the bottom of the tree to get all cache levels in order */
     depth = LIKWID_HWLOC_NAME(topology_get_depth)(hwloc_topology);
@@ -806,7 +833,7 @@ hwloc_init_cacheTopology(void)
 
     cpuid_topology.numCacheLevels = maxNumLevels;
     cpuid_topology.cacheLevels = cachePool;
-    return;
+    return 0;
 }
 
 void
@@ -819,23 +846,23 @@ hwloc_close(void)
     }
 }
 #else
-void
+int
 hwloc_init_cpuInfo(void)
 {
     return;
 }
 
-void hwloc_init_cpuFeatures(void)
+int hwloc_init_cpuFeatures(void)
 {
     return;
 }
 
-void hwloc_init_nodeTopology(void)
+int hwloc_init_nodeTopology(void)
 {
     return;
 }
 
-void hwloc_init_cacheTopology(void)
+int hwloc_init_cacheTopology(void)
 {
     return;
 }
