@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the procfs/sysfs based topology backend
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.4.0
+ *      Released:  15.11.2024
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2024 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -180,7 +180,7 @@ static int readCacheInclusiveAMD(int level)
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void
+int
 proc_init_cpuInfo(cpu_set_t cpuSet)
 {
     int i = 0;
@@ -213,12 +213,6 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
     const_bstring vendorString = bformat("CPU implementer\t:");
     const_bstring familyString = bformat("CPU architecture");
 #endif
-    cpuid_info.osname = malloc(MAX_MODEL_STRING_LENGTH * sizeof(char));
-    if (!cpuid_info.osname)
-    {
-        return;
-    }
-    cpuid_info.osname[0] = '\0';
 
     const_bstring countString = bformat("processor\t:");
 
@@ -232,7 +226,8 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
 #endif
     cpuid_info.stepping = 0;
     cpuid_topology.numHWThreads = 0;
-    
+    cpuid_info.osname = malloc(MAX_MODEL_STRING_LENGTH * sizeof(char));
+    cpuid_info.osname[0] = '\0';
 
     if (NULL != (fp = fopen ("/proc/cpuinfo", "r")))
     {
@@ -340,11 +335,11 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
                             cpuid_info.isIntel,
                             cpuid_topology.numHWThreads)
     }
-    return;
+    return 0;
 }
 
 
-void
+int
 proc_init_cpuFeatures(void)
 {
     int ret = 0;
@@ -354,26 +349,30 @@ proc_init_cpuFeatures(void)
     char delimiter[] = " ";
     char* cptr;
 #ifdef _ARCH_PPC
-    return;
+    return 0;
 #endif
-    const_bstring flagString = bformat("flags");
-    const_bstring featString = bformat("Features");
+    struct tagbstring flagString = bsStatic ("flags");
+    struct tagbstring featString = bsStatic ("Features");
     bstring flagline = bfromcstr("");
 
     bstring cpuinfo = read_file("/proc/cpuinfo");
+    if (blength(cpuinfo) == 0)
+    {
+        return -EFAULT;
+    }
     struct bstrList* cpulines = bsplit(cpuinfo, '\n');
     bdestroy(cpuinfo);
     for (int i = 0; i < cpulines->qty; i++)
     {
 #if defined(__x86_64__) || defined(__i386__)
-        if (bstrncmp(cpulines->entry[i], flagString, 5) == BSTR_OK ||
-            bstrncmp(cpulines->entry[i], featString, 8) == BSTR_OK)
+        if (bstrncmp(cpulines->entry[i], &flagString, 5) == BSTR_OK ||
+            bstrncmp(cpulines->entry[i], &featString, 8) == BSTR_OK)
 #endif
 #if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A__) || defined(__ARM_ARCH_8A)
-        if (bstrncmp(cpulines->entry[i], featString, 8) == BSTR_OK)
+        if (bstrncmp(cpulines->entry[i], &featString, 8) == BSTR_OK)
 #endif
 #ifdef _ARCH_PPC
-    if (ret != 1)
+	if (ret != 1)
 #endif
         {
             bdestroy(flagline);
@@ -384,6 +383,7 @@ proc_init_cpuFeatures(void)
     bstrListDestroy(cpulines);
 
     struct bstrList* flaglist = bsplit(flagline, ' ');
+    bdestroy(flagline);
     bstring bfeatures = bfromcstr("");
 
     cpuid_info.featureFlags = 0;
@@ -570,7 +570,7 @@ proc_init_cpuFeatures(void)
             setBit(cpuid_info.featureFlags, PMULL);
             bcatcstr(bfeatures, "PMULL ");
         }
-        else if (bisstemeqblk(flaglist->entry[i], "sve", 3) == 1)
+        else if (bisstemeqblk(flaglist->entry[i], "sve", 3) == 1 && (!testBit(cpuid_info.featureFlags, SVE)))
         {
             setBit(cpuid_info.featureFlags, SVE);
             bcatcstr(bfeatures, "SVE ");
@@ -584,6 +584,12 @@ proc_init_cpuFeatures(void)
     }
 
     cpuid_info.features = (char*) malloc((blength(bfeatures)+2)*sizeof(char));
+    if (!cpuid_info.features)
+    {
+        bdestroy(bfeatures);
+        bstrListDestroy(flaglist);
+        return -ENOMEM;
+    }
     ret = snprintf(cpuid_info.features, blength(bfeatures)+1, "%s", bdata(bfeatures));
     if (ret > 0)
     {
@@ -593,10 +599,10 @@ proc_init_cpuFeatures(void)
     bstrListDestroy(flaglist);
 
     get_cpu_perf_data();
-    return;
+    return 0;
 }
 
-void
+int
 proc_init_nodeTopology(cpu_set_t cpuSet)
 {
     HWThread* hwThreadPool;
@@ -606,6 +612,7 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
     int (*ownatoi)(const char*);
     ownatoi = &atoi;
     int last_socket = -1;
+    int last_coreid = -1;
     int num_sockets = 0;
     int num_cores_per_socket = 0;
     int num_threads_per_core = 0;
@@ -635,6 +642,7 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
             {
                 num_sockets++;
                 last_socket = packageId;
+                last_coreid = -1;
             }
             fclose(fp);
         }
@@ -643,7 +651,7 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
         if (NULL != (fp = fopen (bdata(file), "r")))
         {
             bstring src = bread ((bNread) fread, fp);
-            hwThreadPool[i].coreId = ownatoi(bdata(src));
+            hwThreadPool[i].coreId = (++last_coreid);
             if (hwThreadPool[i].packageId == 0)
             {
                 num_cores_per_socket++;
@@ -720,7 +728,7 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
     int* helper = malloc(cpuid_topology.numHWThreads * sizeof(int));
     if (!helper)
     {
-        return;
+        return -ENOMEM;
     }
     cpuid_topology.threadPool = hwThreadPool;
     int hidx = 0;
@@ -739,9 +747,88 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
         if (!found)
         {
             helper[hidx++] = pid;
+        }
+    }
+    int tmp_numSockets = hidx;
+    hidx = 0;
+    for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+    {
+        int pid = hwThreadPool[i].dieId;
+        int found = 0;
+        for (int j = 0; j < hidx; j++)
+        {
+            if (pid == helper[j])
+            {
+                found = 1;
+                break;
             }
+        }
+        if (!found)
+        {
+            helper[hidx++] = pid;
+        }
+    }
+    int tmp_numDies = hidx;
+    if (tmp_numSockets == 1 && tmp_numDies == 1)
+    {
+        for (uint32_t i=0;i<cpuid_topology.numHWThreads;i++)
+        {
+            cpudir = bformat("/sys/devices/system/cpu/cpu%d/topology",i);
+            file = bformat("%s/cluster_id", bdata(cpudir));
+            if (NULL != (fp = fopen (bdata(file), "r")))
+            {
+                bstring src = bread ((bNread) fread, fp);
+                int packageId = ownatoi(bdata(src));
+                hwThreadPool[i].packageId = packageId;
+                if (packageId > last_socket)
+                {
+                    num_sockets++;
+                    last_socket = packageId;
+                }
+                fclose(fp);
+            }
+            bdestroy(file);
+        }
+    }
+    hidx = 0;
+    for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+    {
+        int pid = hwThreadPool[i].packageId;
+        int found = 0;
+        for (int j = 0; j < hidx; j++)
+        {
+            if (pid == helper[j])
+            {
+                found = 1;
+                break;
+            }
+        }
+        if (!found)
+        {
+            helper[hidx++] = pid;
+        }
     }
     cpuid_topology.numSockets = hidx;
+    /* Traverse all sockets to get maximal thread count per socket.
+     * This should fix the code for architectures with "empty" sockets.
+     */
+    int num_threads_per_socket = 0;
+    for (int i = 0; i < cpuid_topology.numSockets; i++)
+    {
+        int threadCount = 0;
+        for (int j = 0; j < cpuid_topology.numHWThreads; j++)
+        {
+            if (helper[i] == hwThreadPool[j].packageId)
+            {
+                threadCount++;
+            }
+        }
+        if (threadCount > num_threads_per_socket)
+        {
+            num_threads_per_socket = threadCount;
+        }
+    }
+
     int first_socket_id = helper[0];
     hidx = 0;
     for (int i = 0; i < cpuid_topology.numHWThreads; i++)
@@ -761,7 +848,7 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
         if (!found)
         {
             helper[hidx++] = did;
-            }
+        }
     }
 
     cpuid_topology.numDies = hidx * cpuid_topology.numSockets;
@@ -769,23 +856,19 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
     {
         cpuid_topology.numDies = 0;
     }
-    num_threads_per_core = 0;
-    int test_core_id = hwThreadPool[0].coreId;
-    int test_socket_id = hwThreadPool[0].packageId;
-    int num_threads_per_socket = 0;
+    int max_thread_sibling_id = 0;
     for (int i = 0; i < cpuid_topology.numHWThreads; i++)
     {
-        if (hwThreadPool[i].packageId == test_socket_id)
+        if (hwThreadPool[i].threadId > max_thread_sibling_id)
         {
-            num_threads_per_socket++;
-            if (hwThreadPool[i].coreId == test_core_id)
-                num_threads_per_core++;
+            max_thread_sibling_id = hwThreadPool[i].threadId;
         }
     }
+    num_threads_per_core = max_thread_sibling_id + 1;
     cpuid_topology.numCoresPerSocket = num_threads_per_socket/num_threads_per_core;
     cpuid_topology.numThreadsPerCore = num_threads_per_core;
     free(helper);
-    return;
+    return 0;
 }
 
 void proc_split_llc_check(CacheLevel* llc_cache)
@@ -841,7 +924,7 @@ void proc_split_llc_check(CacheLevel* llc_cache)
 }
 
 
-void
+int
 proc_init_cacheTopology(void)
 {
     FILE *fp;
@@ -876,6 +959,11 @@ proc_init_cacheTopology(void)
     }
 
     cachePool = (CacheLevel*) malloc(nrCaches * sizeof(CacheLevel));
+    if (!cachePool)
+    {
+        bdestroy(cpudir);
+        return -ENOMEM;
+    }
     for (int i=0;i<nrCaches;i++)
     {
         levelStr = bformat("%s/index%d/level",bdata(cpudir),i);
@@ -1019,5 +1107,5 @@ proc_init_cacheTopology(void)
     bdestroy(cpudir);
     cpuid_topology.numCacheLevels = nrCaches;
     cpuid_topology.cacheLevels = cachePool;
-    return;
+    return 0;
 }
