@@ -1,12 +1,30 @@
 #!/bin/bash -l
 
-for L in $(sinfo -t idle -h --partition=work -o "%n"); do
+for L in $(sinfo -t idle -h --partition=work -o "%n %t" | grep "idle" | cut -d ' ' -f 1); do
     arch="x86"
     depend="build-x86-perf"
+    if [ "$L" = "aurora1" ]; then
+        continue
+    fi
+    if [ "$L" = "applem1studio" ]; then
+        continue
+    fi
     if [ "$L" = "warmup" ]; then
         arch="arm8"
         depend="build-arm8-perf"
     fi
+    if [ "$L" = "lukewarm" ]; then
+        arch="arm8"
+        depend="build-arm8-perf"
+    fi
+    if [ "$L" = "gracehop1" ]; then
+        arch="arm8"
+        depend="build-arm8-perf"
+    fi
+    if [ "$L" = "gracesup1" ]; then
+        arch="arm8"
+        depend="build-arm8-perf"
+    fi
 
     cat <<EOF
 test-$L-perf:
